/******************************************************************************/
/* Copyright (c) 2005-2011, SAS Institute Inc., Cary, NC, USA,                */
/*                       All Rights Reserved.                                 */
/*                                                                            */
/* Name:    migrate_datastore_mo53ga_mo53m3.sas                               */
/*                                                                            */
/* Purpose: Convert an MO 5.3 GA datastore to an MO 5.3 M3 datastore.         */
/*                                                                            */
/*          It is intended that this macro be invoked from the MO SDM (see    */
/*          Usage Notes below).                                               */
/*                                                                            */
/*          Alternatively, this macro can be run manually from an interactive */
/*          SAS session.                                                      */
/*                                                                            */
/*          NOTE 1: The determination of whether this program is being run    */
/*                  from a batch SAS session or an interactive SAS session is */
/*                  made by the program itself and typically requires no user */
/*                  intervention (see below).                                 */
/*                                                                            */
/* Usage:                                                                     */
/*          IF USING THIS FILE MANUALLY, PERFORM THE FOLLOWING:               */
/*          1. Start an interactive SAS session and perform the following     */
/*             steps within that session.                                     */
/*          2. In the Program Editor, open and execute the Marketing          */
/*             Optimiation autoexec (MarketingOptimization_autoexec.sas)      */
/*             if not previously run.                                         */
/*          3. In the Program Editor, open and this program and verify that   */
/*             the following values are correctly set:                        */
/*             - The location of the MO datastore directory.                  */
/*             - The location of the MO uncompiled macros (ucmacros)          */
/*               directory.                                                   */
/*             - The operating system type (WIN or UNIX).                     */
/*          4. In the Program Editor, execute this program.                   */
/*          5. Verify it completes successfully with no ERRORs or unexpected  */
/*             WARNINGs in the log.                                           */
/*                                                                            */
/*          IF THIS FILE IS BEING INVOKED FROM THE SAS SOFTWARE DEPLOYMENT    */
/*          WIZARD FOR MO, PERFORM THE FOLLOWING:                             */
/*          1. The program invocation must specify the fully qualified path   */
/*             and file name of this program in the -SYSIN option.            */
/*          2. The program invocation must specify the -SYSPARM option        */
/*             which specifies the fully qualified path and file name of a    */
/*             text file containing input parameters.                         */
/*             This file should contain a line for each input parameter       */
/*             with the format of each line being an equals-separated pair    */
/*             as follows:                                                    */
/*                  <parameter name>=<parameter value>                        */
/*             like a Java or ANT properties file.                            */
/*             Specifically, the following is a list of parameter names with  */
/*             the possible values listed for each name. levconfig is the     */
/*             required parameter. Missing parameters will be assumed to be   */
/*             as specified below.                                            */
/*                  datastoreDir=<fully qualified path name to directory where*/
/*                    the Marketing Optimization 5.1-format datastore tables  */
/*                    to be migrated to 5.3-format reside>                    */
/*                  ucmacrosDir=<fully qualified path name to directory where */
/*                    the Marketing Optimization 5.3 uncompiled macros (for   */
/*                    SASAUTOS option) reside>                                */
/*                  osType=<WIN || UNX>                                       */
/*                    (Missing parm or <blank> value --> WIN)                 */
/*          3. The program invocation should specify the -LOG option with     */
/*             the fully qualified path and file name to which the SAS log    */
/*             should be directed.                                            */
/*                                                                            */
/* Example SAS program invocation command:                                    */
/* (Note: The line breaks in the example should be eliminated from the        */
/*  command used in the script.)                                              */
/*                                                                            */
/* "C:\Program Files\SAS\SASFoundation\9.2\sas"                               */
/*  -sysin "C:\Program Files\SAS\9.1.3\mktopt\                                */
/*          sasmisc\migrate_datastore_mo53ga_mo53m3.sas"                      */
/*  -sysparm "C:\SAS\Config\Lev1\Applications\SASMarketingOptimization5.3\    */
/*            config\mo_properties_set_server_locale.properties               */
/*  -log "C:\SAS\Config\Lev1\SASApp\Applications\SASMarketingOptimization5.3\ */
/*        Logs                                                                */
/******************************************************************************/

%macro migrate_datastore_mo53ga_mo53m3;

   options notes;
   ODS _ALL_ CLOSE;

   %local macname;
   %let macname=&sysmacroname;
   %let macname=%lowcase(&macname);


   %*-----------------;
   %* Initialization. ;
   %*-----------------;
   %local inputParametersFile;
   %let inputParametersFile =;

   %local interactive_sas_session;
   %let interactive_sas_session = Y;

   %local datastoreDir;
   %let datastoreDir =;

   %local ucmacrosDir;
   %let ucmacrosDir =;

   %local osType;
   %let osType = WIN;

   %local sasautos_prev;
   %let sasautos_prev =;

   %local datastore_migratable;
   %let datastore_migratable = Y;

   %local datastore_notmigrated_reasons;
   %let datastore_notmigrated_reasons = ;


   %*-----------------------------------------------------------------------;
   %* Since the MO autoexec is not run when this program is run under a     ;
   %* batch SAS session, global macro variables must be defined manually.   ;
   %*-----------------------------------------------------------------------;
   %if (%symexist(mometa) eq 0) %then %do;
      %local mometa;
   %end;  
   %if (%symexist(separator) eq 0) %then %do;
      %local separator;
   %end;  
   %if (%symexist(momacro) eq 0) %then %do;
      %local momacro;
   %end;  
   %if (%symexist(modebug) eq 0) %then %do;
      %global modebug;
      %let modebug = 0;
   %end;


   %*--------------------------------------------------------------------------;
   %* Determine whether this program is being run from a batch SAS session or  ;
   %* an interactive SAS session by checking whether or not the SYSPARM option ;
   %* has a value (batch session) or not (interactive session).                ;
   %*                                                                          ;
   %* If this is a batch session, retrieve the input parameters to the program ;
   %* by reading the file specified by the SYSPARM option.                     ;
   %*                                                                          ;
   %* If this is an interactive session, update the hardcoded input parameters.;
   %*--------------------------------------------------------------------------;
   data _null_;
      length inputParametersFile $ 200;
      inputParametersFile = sysparm();
      if (inputParametersFile ~= ' ') then do;
         call symput("inputParametersFile", left(trim(inputParametersFile)));
      end;
   run;

   %if (%nrbquote(&inputParametersFile) eq %str()) %then %do; /* Assume interactive */
      %let interactive_sas_session = Y;

      %*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!;
      %* Verify the following hard-coded parameter settings.                   ;
      %* 1. The SAS install directory where the MO uncompiled macros reside.   ;
      %* 2. The directory where the MO datastore resides.                      ;
      %* 3. The OS type: either WIN or UNIX.                                   ;
      %*                                                                       ;
      %* NOTE: UNIX specifications are case-sensitive!                         ;
      %*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!;
      %let ucmacrosDir = %nrbquote(C:\Program Files\SAS\SASFoundation\9.2\mktopt\ucmacros);                           /* UPDATE */

      %let datastoreDir = %nrbquote(C:\SAS\Config\Lev1\Applications\SASMarketingOptimization5.3\datastore);           /* UPDATE */

      %let osType = WIN;              /* UPDATE */  
   %end;
   %else %do;
      %let interactive_sas_session = N;

      %if (not %sysfunc(fileexist(%nrbquote(&inputParametersFile)))) %then %do;
         %put ERROR: Input Parameters File specified in the -SYSPARM option does not exist.;
         %goto ERROR_EXIT;
      %end;

      %let datastoreDir =;
      %let ucmacrosDir =;
      %let osType =;
      
      filename parms "%nrbquote(&inputParametersFile)";
      data _null_;
         length parmName $ 32;
         length parmValue $ 2048;

         infile parms;
         input;

         parmName = left(trim(scan(_INFILE_, 1, "=")));
         parmValue = left(trim(scan(_INFILE_, 2, "=")));

         if (upcase(parmName) = "DATASTOREDIR") then do;
            call symput("datastoreDir", parmValue);
         end;
         else if (upcase(parmName) = "UCMACROSDIR") then do;
            call symput("ucmacrosDir", parmValue);
         end;
         else if (upcase(parmName) = "OSTYPE") then do;
            call symput("osType", parmValue);
         end;
      run;

      %let datastoreDir = %trim(%left(%nrbquote(&datastoreDir)));
      %let ucmacrosDir = %trim(%left(%nrbquote(&ucmacrosDir)));
      %let osType = %upcase(%trim(%left(&osType)));
   %end;

   %*-----------------------------------------------------------------------;
   %* Process type of OS indicated.                                         ;
   %*-----------------------------------------------------------------------;
   %if (&osType eq WIN) %then %do;
      %let separator = %str(\);
   %end;
   %else %do;
      %let separator = %str(/);
   %end;
 
   %put NOTE: DATASTOREDIR: %nrbquote(&datastoreDir);
   %put NOTE: USMACROSDIR: %nrbquote(&ucmacrosDir);
   %put NOTE: OSTYPE: &osType;
   %put NOTE: SEPARATOR: %nrbquote(&separator);


   %*-----------------------------------------------------------------------;
   %* Since the MO autoexec is not run when this program is run under a     ;
   %* batch SAS session, the SASAUTOS option to include the path to the MO  ;
   %* uncompiled macros must be set up "manually", necessary global         ;
   %* macro variables must be defined, and the libref to the MO datastore   ;
   %* must be defined.  For consistency, use names that are used by the MO  ;
   %* autoexec.                                                             ;
   %*-----------------------------------------------------------------------;
   %let momacro = %nrbquote(&ucmacrosDir);
   %if (not %sysfunc(fileexist(%nrbquote(&momacro)))) %then %do;
      %put ERROR: Directory %upcase(%nrbquote(&momacro)) does not exist.;
      %put ERROR: Specify a valid fully qualified pathname to the file system location of the SAS Marketing Optimization 5.3 uncompiled macros directory.;
      %goto ERROR_EXIT;
   %end;
   %let sasautos_prev = %sysfunc(compress(%sysfunc(getoption(SASAUTOS)),()));
   options sasautos = ("%nrbquote(&momacro)" &sasautos_prev);


   %*-----------------------------------------------------------------------;
   %* Verify that a SAS Marketing Optimization 5.3 datastore directory      ;
   %* exists, is accessible via a connection to the Share Server, and is    ;
   %* not empty.                                                            ;
   %*-----------------------------------------------------------------------;
   %let mometa = %nrbquote(&datastoreDir);
   %if (not %sysfunc(fileexist(%nrbquote(&mometa)))) %then %do;
      %put; %put;
      %put WARNING: Directory %nrbquote(&mometa) either does not exist or is not accessible from the SAS tier machine.;
      %put WARNING: Make sure that directory %nrbquote(&mometa) is the intended location for your SAS Marketing Optimization 5.3 datastore.;

      %let datastore_migratable = N;
      %if (%nrbquote(&datastore_notmigrated_reasons) eq %str()) %then %do;
         %let datastore_notmigrated_reasons = Datastore Directory Not Accessible;
      %end;
      %else %do;
         %let datastore_notmigrated_reasons = %nrbquote(&datastore_notmigrated_reasons, )Datastore Directory Not Accessible;
      %end;
   %end;
   %else %do;
      %if (&interactive_sas_session eq N) %then %do;
         libname mometa "%nrbquote(&mometa)"; /* SDM: SHARE Server not needed */
      %end;
      %else %do;
         libname mometa "&mometa" server=&moshare authdomain="&modatausr_authdomain";
      %end;
      %if (&SYSLIBRC ne 0) %then %do;
         %put; %put;
         %put WARNING: Cannot access directory %nrbquote(&mometa) as a SAS library. The SAS libname statement failed to associate this directory with a libref value (syslibrc=&syslibrc).;
         %put WARNING: Make sure that file system permissions to directory %nrbquote(&mometa) allow read and write access for MO administrators and users, the SAS GeneralServers account (e.g., sassrv) and the MO Data Users account (e.g., modatausr).;
         %put WARNING: Also make sure the SAS/SHARE server is running.;

         %let datastore_migratable = N;
         %if (%nrbquote(&datastore_notmigrated_reasons) eq %str()) %then %do;
            %let datastore_notmigrated_reasons = Libname Failure;
         %end;
         %else %do;
            %let datastore_notmigrated_reasons = %nrbquote(&datastore_notmigrated_reasons, )Libname Failure;
         %end;
      %end;
      %else %do;
         proc contents data=mometa._all_ out=work.tableNames_tmp (keep=memname) noprint nodetails;
         run;
         %if (&syserr gt 4) %then %do;
            %let &_rc = 100;
            %put %mo_error Error encountered using proc contents to get the list of tables in MOMETA (syserr=&syserr).;
            %goto ERROR_EXIT;
         %end;

         %local memberCount;
         %let memberCount = 0;
         proc sql noprint;
            select count(*) into :memberCount
            from work.tableNames_tmp
            where (memname ~= "");
         quit;
         %if (&sqlrc gt 4) %then %do;
            %put ERROR: Error encountered while counting SAS members of directory %nrbquote(&mometa) (sqlrc=&sqlrc).;
            %goto ERROR_EXIT;
         %end;

         %if (&memberCount eq 0) %then %do;
            %put; %put;
            %put WARNING: Directory %nrbquote(&mometa) is empty.;
            %put WARNING: Make sure that directory %nrbquote(&mometa) is the intended location for your SAS Marketing Optimization 5.3 datastore.;

            %let datastore_migratable = N;
            %if (%nrbquote(&datastore_notmigrated_reasons) eq %str()) %then %do;
               %let datastore_notmigrated_reasons = Empty Datastore Directory;
            %end;
            %else %do;
               %let datastore_notmigrated_reasons = %nrbquote(&datastore_notmigrated_reasons, )Empty Datastore Directory;
            %end;
         %end;
      %end;
   %end;

   
   %*------------------------------------------------------------------------;
   %* If any problems precluding datastore migration were found above,       ;
   %* exit.                                                                  ;
   %*------------------------------------------------------------------------;
   %if (&datastore_migratable eq N) %then %do;
      %goto DS_NOT_MIGRATABLE_EXIT;
   %end;


   %*------------------------------------------------------;
   %* Backup the MO Datastore directory before proceeding. ;
   %*------------------------------------------------------;
   %let parent_dir = %nrbquote(&mometa);
   %let new_dir = GATOM3_BACKUP_%sysfunc(translate(%sysfunc(putn(%sysfunc(datetime()),datetime16.)),"_",":"));
   %let backup_directory = %trim(%left(%nrbquote(&parent_dir.&separator.&new_dir)));

   %local cdrc;
   %let cdrc = ;
   data _null_;
      newdir = dcreate("&new_dir", "%nrbquote(&parent_dir)");
      call symput("cdrc", trim(left(newdir)));
   run;
   %if (%bquote(&cdrc) eq %str()) %then %do;
      %put ERROR: I/O error encountered creating %nrbquote(&backup_directory) directory for MO datastore.;
      %goto ERROR_EXIT;
   %end;

   %if (not %sysfunc(fileexist(%nrbquote(&backup_directory)))) %then %do;
      %put ERROR: I/O error: %nrbquote(&backup_directory) directory for MO datastore was not created.;
      %goto ERROR_EXIT;
   %end;

   libname backup "%nrbquote(&backup_directory)";

   proc contents data=mometa._all_ out=work.dsNames_tmp (keep=memname) noprint nodetails;
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered using proc contents to get the list of tables in MOMETA (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;

   proc sql noprint;
      create table work.dsnames as
      select distinct memname
      from work.dsNames_tmp;
   quit;
   %if (&sqlrc gt 4) %then %do;
      %put ERROR: Error encountered while determining SAS members of directory %nrbquote(&mometa) (sqlrc=&sqlrc).;
      %goto ERROR_EXIT;
   %end;

   data work.valid_dsnames;
      length dsname $ 45;
      set work.dsnames;
      dsname = compress("MOMETA."||strip(memname));
      if (open(dsname) ~= 0) then output;
   run;
   %if (&syserr gt 4) %then %do;
      %put ERROR: Error validating dataset names in directory %nrbquote(&mometa) (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end; 

   %local mometa_select_list;
   proc sql noprint;
      select memname into :mometa_select_list separated by " "
      from work.valid_dsnames;
   quit;
   %if (&sqlrc gt 4) %then %do;
      %put ERROR: Error encountered while gathering SAS members of directory %nrbquote(&mometa) into macro variable MOMETA_SELECT_LIST (sqlrc=&sqlrc).;
      %goto ERROR_EXIT;
   %end;

   proc datasets library=mometa nolist;
      copy out=backup;
      select &mometa_select_list;
   quit;
   %if (&syserr gt 4) %then %do;
      %put ERROR: Error using Proc Datasets to copy datasets in directory %nrbquote(&mometa) to directory %nrbquote(&backup_directory) (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end; 


   %*------------------;
   %* Initialization.  ;
   %*------------------;
   %local col_names;

   %mo_constants


   %*-----------------------------------------------------------;      
   %* TABLE = INPUT_DATA (new columns in 5.3)                   ;     
   %* 1. Re-name datetime_created to datetime_create_started    ;
   %*    and add new datetime_create_completed column.          ;
   %*-----------------------------------------------------------;
   %let col_names=;
   %let cust_id_col_name_length = 0;
   proc contents noprint data=mometa.input_data out=work.col_names (keep=name length);
   run;
   proc sql noprint;
      select name into :col_names separated by " " from work.col_names;
      select length into :cust_id_col_name_length from work.col_names where (lowcase(name) = "customer_id_column_name");
   quit;

   %if ((%index(%upcase(&col_names), DATETIME_CREATE_STARTED) = 0) or
        (%index(%upcase(&col_names), DATETIME_CREATE_COMPLETED) = 0)) %then %do;
      %mo_create_tables(lib=work,table=input_data,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %put ERROR: Error encountered creating WORK.INPUT_DATA table.;
         %goto ERROR_EXIT;
      %end;

      /* Concatenate the existing metadata table to the working table, in */
      /* order to populate the working table with the existing rows.      */
      data work.input_data (drop=datetime_created);
         set work.input_data 
             mometa.input_data;
         datetime_create_started = datetime_created;
         if (datetime_create_started ~= .) then do;
            datetime_create_completed = datetime_create_started + 1;
         end;
         else do;
            datetime_create_completed = .;
         end;
      run;

      /* Copy the working table, now populated with existing input_data   */
      /* rows, over to the metadata library.                              */
      data mometa.input_data;
         set work.input_data;
      run;
   %end;


   %*--------------------------------;
   %* Exit processing                ;
   %*--------------------------------;
   %put;
   %put NOTE: MO 5.3 GA datastore successfully migrated to MO 5.3 M3.;
   %goto EXIT;


   %ERROR_EXIT:
   %put;
   %put ERROR: Error encountered while migrating MO 5.3 GA datastore to MO 5.3 M3.;
   %goto EXIT;


   %DS_NOT_MIGRATABLE_EXIT:
   %put; %put;
   %put WARNING: Unable to migrate the SAS Marketing Optimization datastore for the following reason(s) - %nrbquote(&datastore_notmigrated_reasons).; 
   %put; %put;
   %put WARNING: After verifying and correcting the above items, perform the following steps.;
   %put WARNING: Perform manual migration of the 5.3 GA datastore to the 5.3 M3 datastore by running program migrate_datastore_mo53ga_mo53m3.sas from an interactive SAS session, following the instructions in the comments.;


   %EXIT:
   %if (&syscc le 4) %then %do;
      %let syscc = 0;
   %end;


%mend migrate_datastore_mo53ga_mo53m3;


%migrate_datastore_mo53ga_mo53m3;
