/****************************************************************************/
/* Copyright (c) 2005-2011, SAS Institute Inc., Cary, NC, USA,              */
/*                       All Rights Reserved.                               */
/*                                                                          */
/* Name:    mo_generate_portable_files                                      */
/*                                                                          */
/* Purpose: Convert all members of the specified directory, or the single   */
/*          file in that directory, into .dpo or .cpo files based on the    */
/*          specified type. Put the generated files into the specified      */
/*          directory and capture their names in the specified dataset.     */
/*                                                                          */
/* Input:   inDirectory       (required)                                    */
/*          inMemberName      (optional)                                    */
/*          memType           (optional, defaults to DATA)                  */
/*          prependWithName   (optional)                                    */
/*          outDirectory      (required)                                    */
/*          outMemberDataset  (optional, defaults to work.members)          */
/*                                                                          */
/* Output:  &_rc = return code                                              */
/*                                                                          */
/*                                                                          */
/* HISTORY:                                                                 */
/* 5.3:                                                                     */
/* 01May09 Created (KEC)                                                    */
/*                                                                          */
/* 5.4:                                                                     */
/* 09Mar09 Add option to generate portable file for a single specified file */
/*         in the input directory (KEC)                                     */
/* 20Apr10 Enhancements related to support for scenario import (KEC)        */
/*                                                                          */
/* F1MO54, F0MO541                                                          */
/* 09Jan12 S0823209 Replace use of SQL query of Dictionay Tables with use   */
/*         of SAS Proc Contents.                                            */
/*                                                                          */
/****************************************************************************/

%macro mo_generate_portable_files(inDirectory      =
                                 ,inMemberName     =
                                 ,memType          = DATA
                                 ,prependWithName  =
                                 ,options          =
                                 ,outDirectory     =
                                 ,outMemberDataset = work.members
                                 ,_rc              =
                                 );

   %local macname;
   %let macname=&sysmacroname;
   %let macname=%lowcase(&macname);

   %if %nrbquote(&_rc) eq %str() %then %do;
      %local rc;
      %let _rc = rc;
   %end;
   %else %do;
      %global &_rc;
   %end;
   %let &_rc = 0;

   %mo_constants

   %mo_no_notes(off)

   %local local_debug;
   %let local_debug = 0;
   %if (&local_debug eq 1) %then %do;
      %mo_no_notes(on)
      %put;
      %put %mo_note inDirectory: %nrbquote(&inDirectory);
      %put %mo_note inMemberName: %nrbquote(&inMemberName);
      %put %mo_note memType: %nrbquote(&memType);
      %put %mo_note prependWithName: %nrbquote(&prependWithName);
      %put %mo_note options: %nrbquote(&options);
      %put %mo_note outDirectory: %nrbquote(&outDirectory);
      %put %mo_note outMemberDataset: %nrbquote(&outMemberDataset);
      %mo_no_notes(off)
   %end;


   %*-------------------------------------------------------------------------;
   %* Validate input parameters.                                              ;
   %*-------------------------------------------------------------------------;
   %if (%nrbquote(&inDirectory) eq %str()) %then %do;
      %put %mo_error No input directory was specified.;
      %let &_rc = 102;
      %goto ERROR_EXIT;
   %end;
   %else %if (not %sysfunc(fileexist(%nrbquote(&inDirectory)))) %then %do;
      %put %mo_error Specified input directory does not exist.;
      %let &_rc = 102;
      %goto ERROR_EXIT;
   %end;
   %else %do;
      libname mocport "%nrbquote(&inDirectory)";
   %end;

   %if (%nrbquote(&inMemberName) ne %str()) %then %do;
      %if (not %sysfunc(exist(%nrbquote(mocport.&inMemberName)))) %then %do;
         %put %mo_error Specified input member does not exist in input directory.;
         %let &_rc = 102;
         %goto ERROR_EXIT;
      %end;
   %end;

   %if (%nrbquote(&memType) eq %str()) %then %do;
      %put %mo_error No Member Type was specified.;
      %let &_rc = 102;
      %goto ERROR_EXIT;
   %end; 
   %else %if ((%upcase(&memType) ne DATA) and 
              (%upcase(&memType) ne CATALOG)) %then %do;
      %put %mo_error Member Type of &memType is not valid. It must be DATA or CATALOG.;
      %let &_rc = 102;
      %goto ERROR_EXIT;
   %end;
   %let memType = %upcase(&memType);

   %if (%nrbquote(&outDirectory) eq %str()) %then %do;
      %put %mo_error No output directory into which to place the transport .dpo or .cpo files was specified.;
      %let &_rc = 102;
      %goto ERROR_EXIT;
   %end;
   %else %if (not %sysfunc(fileexist(%nrbquote(&outDirectory)))) %then %do;
      %put %mo_error Specified output directory does not exist.;
      %let &_rc = 102;
      %goto ERROR_EXIT;
   %end;


   %*-------------------------------------------------------------------------;
   %* Initialize                                                              ;
   %*-------------------------------------------------------------------------;
   %local numMembers;
   %let numMembers = 0;

   %local fileExt;
   %if (&memType eq DATA) %then %do;
      %let fileExt = dpo;
   %end;
   %else %do;
      %let fileExt = cpo;
   %end;


   %*-------------------------------------------------------------------------;
   %* Gather members of the specified member type from the specified library  ;
   %* into the specified dataset for processing.                              ;
   %*-------------------------------------------------------------------------;
   %if (%nrbquote(&inMemberName) eq %str()) %then %do;
      proc contents data=mocport._all_ out=work.memberNames_tmp(keep=memname memtype) noprint nodetails;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered using proc contents to get the list of tables in directory %nrbquote(&inDirectory) (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;

      proc sql noprint;
         create table work.memberNames as
         select distinct lowcase(memname) as memberName length=45,
                         lowcase(memtype) as memberType length=8,
                         "" as memberFilename length=1024
         from work.memberNames_tmp
         where (upcase(strip(memtype)) = "&memType");
      quit;
      %if (&sqlrc gt 4) %then %do;
         %let &_rc = 107;
         %put %mo_error SQL error occurred while determining members of &memType in directory %nrbquote(&inDirectory) (sqlrc=&sqlrc).;
         %goto ERROR_EXIT;
      %end;
   %end;
   %else %do;
      data work.memberNames;
         length memberName $ 45;
         length memberType $ 8;
         length memberFilename $ 1024;
         memberName = lowcase("%nrbquote(&inMemberName)");
         memberType = lowcase("&memType");
         memberFilename = "";
      run;
   %end;

   data &outMemberDataset (drop=dsName);
      length memberName $ 45;
      set work.memberNames;
      if (upcase(memberType) = "DATA") then do;
         dsName = compress("MOCPORT."||strip(memberName));
         if (open(dsName) ~= 0) then output;
      end;
      else do;
         output;
      end;
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 107;
      %put %mo_error Error occurred while validating members of &memType in directory %nrbquote(&inDirectory) (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;


   %*-------------------------------------------------------------------------;
   %* Generate a transport file into the specified directory for each member. ;
   %*-------------------------------------------------------------------------;
   proc sql noprint;
      select count(*) into :numMembers from &outMemberDataset;
   quit;
   %if (&sqlrc gt 4) %then %do;
      %let &_rc = 107;
      %put %mo_error Error occurred while determining number of members in &outMemberDataset in directory %nrbquote(&inDirectory) (sqlrc=&sqlrc).;
      %goto ERROR_EXIT;
   %end;

   %do mbr = 1 %to &numMembers;
      %local member_&mbr;
      data _null_;
         obsNum = &mbr;
         set &outMemberDataset point=obsNum;
         call symput("member_&mbr", memberName);
         stop;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error reading observation &mbr from dataset &outMemberDataset (syserr=&syserr).;
      %end; 
      %let member_&mbr = %trim(%left(%nrbquote(&&member_&mbr)));

      %if (&prependWithName ne %str ()) %then %do;
         %let mbrShortFilename_&mbr = %nrbquote(&prependWithName.&&member_&mbr...&fileExt);
         %let mbrFilename_&mbr = %nrbquote(&outDirectory.&separator.&prependWithName.&&member_&mbr...&fileExt);
      %end;
      %else %do;
         %let mbrShortFilename_&mbr = %nrbquote(&&member_&mbr...&fileExt);
         %let mbrFilename_&mbr = %nrbquote(&outDirectory.&separator.&&member_&mbr...&fileExt);
      %end;

      filename t&mbr "%nrbquote(&&mbrFilename_&mbr)";
      proc cport %if (%upcase(&memType) eq DATA) %then %do;
                    data=
                 %end;
                 %else %do; 
                    catalog=
                 %end;
                 mocport.&&member_&mbr file=t&mbr %if (%nrbquote(&options) ne %str()) %then %do;
                                                          %nrbquote(&options)
                                                       %end;
                                                       ;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error generating portable transport file for member %nrbquote(&&member_&mbr) in directory %nrbquote(&inDirectory) (syserr=&syserr).;
      %end; 

      data &outMemberDataset;
         set &outMemberDataset;
         if (memberName = "%nrbquote(&&member_&mbr)") then do;
            memberFilename = "%nrbquote(&&mbrShortFilename_&mbr)";
         end;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error updating observation for %nrbquote(&&member_&mbr) from dataset &outMemberDataset (syserr=&syserr).;
      %end; 
   %end;


   %*-------------------------------------------------------------------------;
   %* Exit processing                                                         ;
   %*-------------------------------------------------------------------------;
   %* Successful completion;
   %mo_no_notes(on)
   %put;
   %put %mo_note Successfully generated transport files into directory %nrbquote(&outDirectory) for Member Type &memType in directory %nrbquote(&inDirectory).;
   %goto EXIT;


   %ERROR_EXIT:
   %put;
   %put %mo_error Failed to generate transport files into directory %nrbquote(&outDirectory) for Member Type &memType in directory %nrbquote(&inDirectory).;


   %EXIT:
   %mo_no_notes(off)

%mend mo_generate_portable_files;
