/****************************************************************************/
/* Copyright (c) 2005-2011, SAS Institute Inc., Cary, NC, USA,              */
/*                       All Rights Reserved.                               */
/*                                                                          */
/* Name:    mo_get_input_data_tables                                        */
/*                                                                          */
/* Purpose: Return a dataset containing the datastore table names that      */
/*          comprise an MO input data definition (with an order number      */
/*          assigned to reflect dependencies, i.e., tables that refer to    */
/*          other tables are assigned a higher order number than the tables */
/*          they depend on).                                                */
/*                                                                          */
/* Input:   inputDataTablesDS (required)                                    */
/*                                                                          */
/* Output:  &_rc = return code                                              */
/*                                                                          */
/*                                                                          */
/* HISTORY:                                                                 */
/* 5.4:                                                                     */
/* 18Mar10 Created for scenario export (KEC)                                */
/* 20Apr10 Enhancements related to support for scenario import (KEC)        */
/*                                                                          */
/* F1MO54, F0MO541                                                          */
/* 09Jan12 S0823209 Replace use of SQL query of Dictionay Tables with use   */
/*         of SAS Proc Contents.                                            */
/*                                                                          */
/****************************************************************************/

%macro mo_get_input_data_tables(inputDataTablesDS = work.inputDataTables
                               ,_rc =
                               );


   %local macname;
   %let macname=&sysmacroname;
   %let macname=%lowcase(&macname);

   %if %nrbquote(&_rc) eq %str() %then %do;
      %local rc;
      %let _rc = rc;
   %end;
   %else %global &_rc;
   %let &_rc = 0;

   %mo_constants

   %mo_no_notes(off)


   %*--------------------------------;
   %* Validate parms                 ;
   %*--------------------------------;
   %if (%nrbquote(&inputDataTablesDS) eq %str()) %then %do;
      %put %mo_error No input data tables dataset name was specified.;
      %let &_rc = 102;
      %goto ERROR_EXIT;
   %end;


   %*--------------------------------------;
   %* Collect dataset names containing the ;
   %* input data definition.               ;
   %*--------------------------------------;
   %mo_get_scenario_tables(scenarioTablesDS = work.scenTbls
                          ,_rc = gstrc);
   %if (&gstrc ne 0) %then %do;
      %let &_rc = 100;
      %put %mo_error Error occurred while obtaining datastore table names containing the scenario definition.;
      %goto ERROR_EXIT;
   %end;

   %mo_no_notes(off)
   proc contents data=mometa._all_ out=work.inputDataTablesDS_tmp (keep=memname name) noprint nodetails;
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered using proc contents to get the list of tables in MOMETA (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;

   proc sql noprint;
      create table &inputDataTablesDS as
      select distinct memname as tableName length=32, "INPUTDATA" as tableType length=10
      from work.inputDataTablesDS_tmp as a
      where ((upcase(name) = "INPUT_DATA_ID") and
             (not exists (select *
                          from work.scenTbls
                          where (tableName = a.memname))) and
             (upcase(memname) not in ("INPUT_DATA_VALIDATION_MESSAGES",
                                      "REFRESH_MESSAGES",
                                      "INPUT_DATA_CAMPAIGNS", 
                                      "INPUT_DATA_MOMA_PARAMETERS",
                                      "INPUT_DATA_USAGE_TRACKING")));
   quit;
   %if (&sqlrc gt 4) %then %do;
      %let &_rc = 107;
      %put %mo_error SQL error occurred while determining MOMETA tables containing INPUT_DATA_ID column (sqlrc=&sqlrc).;
      %goto ERROR_EXIT;
   %end;


   %*----------------------------------;
   %* Assign relative processing order ;
   %* numbers to the tables to capture ;
   %* dependences between the input    ;
   %* data tables themselves.          ;
   %*----------------------------------;
   data &inputDataTablesDS;
      set &inputDataTablesDS;
      if (lowcase(tableName) in ("input_data")) then do;
         relativeOrder = 1;
      end;
      else if (lowcase(tableName) in ("input_data_customer_columns", 
                                      "campaigns", 
                                      "comm_numeric_attributes", 
                                      "comm_character_attributes", 
                                      "customer_attributes", 
                                      "customer_offer_attributes", 
                                      "channels", 
                                      "time_periods", 
                                      "prepopulated_constraints", 
                                      "projects")) then do;
         relativeOrder = 2;
      end;
      else if (lowcase(tableName) in ("communications",
                                      "comm_character_distinct_values", 
                                      "customer_attribute_values", 
                                      "prepopulated_contact_policies", 
                                      "agents")) then do;
         relativeOrder = 3;
      end;
      else if (lowcase(tableName) in ("comm_numeric_attribute_defaults",
                                      "comm_character_attribute_values", 
                                      "input_data_measures")) then do;
         relativeOrder = 4;
      end;
      else do;
         relativeOrder = 5;  /* Don't care */
      end;
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error occurred while processing &inputDataTablesDS (syserr=&yserr).;
      %goto ERROR_EXIT;
   %end;

   proc sort data=&inputDataTablesDS;
      by relativeOrder tableName;
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error occurred while sorting &inputDataTablesDS (syserr=&yserr).;
      %goto ERROR_EXIT;
   %end;


   %*--------------------------------;
   %* Exit processing                ;
   %*--------------------------------;
   %* Successful completion;
   %mo_no_notes(on)
   %goto EXIT;


   %ERROR_EXIT:


   %EXIT:
   %mo_no_notes(off)

%mend mo_get_input_data_tables;
