/****************************************************************************/

/* Copyright (c) 2005-2011, SAS Institute Inc., Cary, NC, USA,              */

/*                       All Rights Reserved.                               */

/*                                                                          */

/* Name:    mo_get_scenario_tables                                          */

/*                                                                          */

/* Purpose: Return a dataset containingt the datastore table names that     */

/*          comprise an MO scenario definition (with an order number        */

/*          assigned to reflect dependencies, i.e., tables that refer to    */

/*          other tables are assigned a higher order number than the tables */

/*          they depend on).                                                */

/*                                                                          */

/* Input:   scenarioTables DS (required)                                    */

/*                                                                          */

/* Output:  &_rc = return code                                              */

/*                                                                          */

/*                                                                          */

/* HISTORY:                                                                 */

/* 5.4:                                                                     */

/* 18Mar10 Created for scenario export (KEC)                                */

/* 20Apr10 Enhancements related to support for scenario import (KEC)        */

/*                                                                          */

/* F1MO54, F0MO541                                                          */

/* 09Jan12 S0823209 Replace use of SQL query of Dictionay Tables with use   */

/*         of SAS Proc Contents.                                            */

/*                                                                          */

/****************************************************************************/

%macro mo_get_scenario_tables(scenarioTablesDS = work.scenarioTables

                             ,_rc =

                             );





   %local macname;

   %let macname=&sysmacroname;

   %let macname=%lowcase(&macname);



   %if %nrbquote(&_rc) eq %str() %then %do;

      %local rc;

      %let _rc = rc;

   %end;

   %else %global &_rc;

   %let &_rc = 0;



   %mo_constants



   %mo_no_notes(off)





   %*--------------------------------;

   %* Validate parms                 ;

   %*--------------------------------;

   %if (%nrbquote(&scenarioTablesDS) eq %str()) %then %do;

      %put %mo_error No scenario tables dataset name was specified.;

      %let &_rc = 102;

      %goto ERROR_EXIT;

   %end;





   %*--------------------------------------;

   %* Collect dataset names containing the ;

   %* scenario definition.                 ;

   %*--------------------------------------;

   proc contents data=mometa._all_ out=work.scenarioTablesDS_tmp (keep=memname name) noprint nodetails;

   run;

   %if (&syserr gt 4) %then %do;

      %let &_rc = 100;

      %put %mo_error Error encountered using proc contents to get the list of tables in MOMETA (syserr=&syserr).;

      %goto ERROR_EXIT;

   %end;



   proc sql noprint;

      create table &scenarioTablesDS as

      select distinct memname as tableName length=32, "SCENARIO" as tableType length=10

      from work.scenarioTablesDS_tmp

      where ((upcase(name) = "SCENARIO_ID") and

             (upcase(memname) not in ("REFRESH_MESSAGES",

                                      "COPY_ACROSS_MESSAGES",

                                      "INPUT_DATA_USAGE_TRACKING")));

   quit;

   %if (&sqlrc gt 4) %then %do;

      %let &_rc = 107;

      %put %mo_error SQL error occurred while determining MOMETA tables containing SCENARIO_ID column (sqlrc=&sqlrc).;

      %goto ERROR_EXIT;

   %end;





   %*----------------------------------;

   %* Assign relative processing order ;

   %* numbers to the tables to capture ;

   %* dependences between the scenario ;

   %* tables themselves (for import).  ;

   %*----------------------------------;

   data &scenarioTablesDS;

      set &scenarioTablesDS;

      if (lowcase(tableName) in ("scenarios", 

                                 "scenario_advanced_options", 

                                 "optimization_jobs")) then do;

         relativeOrder = 1;

      end;

      else if (lowcase(tableName) in ("opt_group_assign",

                                      "comm_numeric_attribute_values", 

                                      "constraints",

                                      "contact_policies",

                                      "suppression_rules",

                                      "calculated_attributes",

                                      "offer_filter_measures",

                                      "customer_filter_measures",

                                      "copy_across_messages",

                                      "refresh_messages")) then do;

         relativeOrder = 2;

      end;

      else if (lowcase(tableName) in ("precalculated_attributes", 

                                      "offer_filters", 

                                      "customer_filters")) then do;

         relativeOrder = 3;

      end;

      else if (lowcase(tableName) in ("offer_filter_attribute_values",

                                      "customer_filter_attribute_values")) then do;

         relativeOrder = 4;

      end;

      else do;

         relativeOrder = 5;  /* Don't care */

      end;

   run;

   %if (&syserr gt 4) %then %do;

      %let &_rc = 100;

      %put %mo_error Error occurred while processing &scenarioTablesDS (syserr=&yserr).;

      %goto ERROR_EXIT;

   %end;



   proc sort data=&scenarioTablesDS;

      by relativeOrder tableName;

   run;

   %if (&syserr gt 4) %then %do;

      %let &_rc = 100;

      %put %mo_error Error occurred while sorting &scenarioTablesDS (syserr=&yserr).;

      %goto ERROR_EXIT;

   %end;





   %*--------------------------------;

   %* Exit processing                ;

   %*--------------------------------;

   %* Successful completion;

   %mo_no_notes(on)

   %goto EXIT;





   %ERROR_EXIT:





   %EXIT:

   %mo_no_notes(off)



%mend mo_get_scenario_tables;

