/****************************************************************************/
/* Copyright (c) 2005-2011, SAS Institute Inc., Cary, NC, USA,              */
/*                       All Rights Reserved.                               */
/*                                                                          */
/* Name:    mo_get_users_tables                                             */
/*                                                                          */
/* Purpose: Return a dataset containing the datastore table names that      */
/*          comprise an MO user definition (with an order number assigned   */
/*          to reflect dependencies, i.e., tables that refer to other       */
/*          tables are assigned a higher order number than the tables they  */
/*          depend on).                                                     */
/*                                                                          */
/* Input:   usersTablesDS (required)                                        */
/*                                                                          */
/* Output:  &_rc = return code                                              */
/*                                                                          */
/*                                                                          */
/* HISTORY:                                                                 */
/* 5.4:                                                                     */
/* 18Mar10 Created for scenario export (KEC)                                */
/* 20Apr10 Enhancements related to support for scenario import (KEC)        */
/*                                                                          */
/* F1MO54, F0MO541                                                          */
/* 09Jan12 S0823209 Replace use of SQL query of Dictionay Tables with use   */
/*         of SAS Proc Contents.                                            */
/*                                                                          */
/****************************************************************************/

%macro mo_get_users_tables(usersTablesDS = work.usersTables
                          ,_rc =
                          );


   %local macname;
   %let macname=&sysmacroname;
   %let macname=%lowcase(&macname);

   %if %nrbquote(&_rc) eq %str() %then %do;
      %local rc;
      %let _rc = rc;
   %end;
   %else %global &_rc;
   %let &_rc = 0;

   %mo_constants

   %mo_no_notes(off)


   %*--------------------------------;
   %* Validate parms                 ;
   %*--------------------------------;
   %if (%nrbquote(&usersTablesDS) eq %str()) %then %do;
      %put %mo_error No users tables dataset name was specified.;
      %let &_rc = 102;
      %goto ERROR_EXIT;
   %end;


   %*--------------------------------------;
   %* Collect dataset names containing the ;
   %* user definition.                     ;
   %*--------------------------------------;
   %mo_get_input_data_tables(inputDataTablesDS = work.inputTbls
                          ,_rc = gidtrc);
   %if (&gidtrc ne 0) %then %do;
      %let &_rc = 100;
      %put %mo_error Error occurred while obtaining datastore table names containing the input data definition.;
      %goto ERROR_EXIT;
   %end;

   %mo_get_scenario_tables(scenarioTablesDS = work.scenTbls
                          ,_rc = gstrc);
   %if (&gstrc ne 0) %then %do;
      %let &_rc = 100;
      %put %mo_error Error occurred while obtaining datastore table names containing the scenario definition.;
      %goto ERROR_EXIT;
   %end;

   %mo_no_notes(off)
   proc contents data=mometa._all_ out=work.usersTablesDS_tmp (keep=memname name) noprint nodetails;
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered using proc contents to get the list of tables in MOMETA (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;

   proc sql noprint;
      create table &usersTablesDS as
      select distinct memname as tableName length=32, "USERS" as tableType length=10
      from work.usersTablesDS_tmp as a
      where ((upcase(name) = "USER_ID") and
             (not exists (select *
                          from work.inputTbls
                          where (tableName = a.memname))) and
             (not exists (select *
                          from work.scenTbls
                          where (tableName = a.memname))) and
             (upcase(memname) not in ("REFRESH_MESSAGES",
                                      "PROJECTS")));
   quit;
   %if (&sqlrc gt 4) %then %do;
      %let &_rc = 107;
      %put %mo_error SQL error occurred while determining MOMETA tables containing USER_ID column (sqlrc=&sqlrc).;
      %goto ERROR_EXIT;
   %end;


   %*----------------------------------;
   %* Assign relative processing order ;
   %* numbers to the tables to capture ;
   %* dependences between the users    ;
   %* tables themselves.               ;
   %*----------------------------------;
   data &usersTablesDS;
      set &usersTablesDS;
      if (lowcase(tableName) in ("users")) then do;
         relativeOrder = 1;
      end;
      else do;
         relativeOrder = 2;  
      end;
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error occurred while processing &usersTablesDS (syserr=&yserr).;
      %goto ERROR_EXIT;
   %end;

   proc sort data=&usersTablesDS;
      by relativeOrder tableName;
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error occurred while sorting &usersTablesDS (syserr=&yserr).;
      %goto ERROR_EXIT;
   %end;


   %*--------------------------------;
   %* Exit processing                ;
   %*--------------------------------;
   %* Successful completion;
   %mo_no_notes(on)
   %goto EXIT;


   %ERROR_EXIT:


   %EXIT:
   %mo_no_notes(off)

%mend mo_get_users_tables;
