/****************************************************************************/
/* Copyright (c) 2005-2011, SAS Institute Inc., Cary, NC, USA,              */
/*                       All Rights Reserved.                               */
/*                                                                          */
/* Name:    mo_promote_solution                                             */
/*                                                                          */
/* Purpose: Promote a MO solution                                           */
/*                                                                          */
/* Input:   scenario_id                                                     */
/*          library_metadata_id - FQID of publish or promote SASLibrary     */
/*          name      - solution output dataset name                        */
/*          description - metadata description                              */
/*          notify    - list of email addresses to notify                   */
/*          locale    - locales of email recipients                         */
/*          publish   - 1 = publish all output data to SASLibrary           */
/*          olap      - 1 = generate OLAP cube from solution                */
/*          olap_cube_name                                                  */
/*          olap_cube_description                                           */
/*          olap_cube_overwrite     - 1 = delete cube before creating       */
/*          olap_schema_name                                                */
/*          olap_schema_metadata_id                                         */
/*          imap      - 1 = generate OLAP information map on the cube       */
/*          imap_name                                                       */
/*          imap_description                                                */
/*          imap_overwrite          - 1 = delete imap before creating       */
/*          imap_mappath                                                    */
/*          promote_automatically - 1 = lights out promotion                */
/*                                                                          */
/* Output:  &_rc = return code                                              */
/*          &_nc = number of communications (to pass back to MA)            */
/*                                                                          */
/*                                                                          */
/* HISTORY:                                                                 */
/* 11Feb04 S0237007 Creation (dwh)                                          */
/* 15Jun04 S0256159 Start using name parameter (dwh)                        */
/* 23Aug04 S0265528 Only use passed notify list (dwh)                       */
/* 20Nov04 S0261675 I18N changes to email text (dwh)                        */
/*                                                                          */
/* 4.3:                                                                     */
/* 24Jan05 S0273454 Add publish feature and remove email (dwh)              */
/* 06Apr05 S0273454 Add &_nc for integration with MA (micopp)               */
/* 16May05 S0296213 Allow numeric project_name and scenario_name (dwh)      */
/* 26May05 S0302341 Do not clean out promote metadata library first (dwh)   */
/*                                                                          */
/* 4.3 hotfix:                                                              */
/* 10Oct05 S0324133 Use proc metadata to register table metadata (dwh)      */
/*                                                                          */
/* 4.4:                                                                     */
/* 03Jan06 S0312209 Generate OLAP cube (dwh)                                */
/* 14Feb06 S0312209 Generate OLAP information map (dwh)                     */
/* 21Mar06 S0349416 Standardize global macvar names in MO autoexec (dwh)    */
/* 03Apr06 S0351359 Work around OLAP path length limit (dwh)                */
/* 01May06 S0356464 Add promotion capability to lights out scenario (dwh)   */
/* 06May06 S0357608 Fix timestamp if hours is single digit (dwh)            */
/* 19May06 S0359734 Do not promote failed or infeasible optimization (dwh)  */
/* 12Jun06 S0363156 Separately trap OLAP cube errors (dwh)                  */
/*                                                                          */
/* 5.3                                                                      */
/* 17Jul08 S0483854 MOMA support - Include cell_cd column in OLAP cube      */
/*         produced as part of MO solution promotion. (KEC)                 */
/* 24Sep08 S0483854 Change mechanism for accessing MO Property value when   */
/*                  running on 9.2 (Property objects on the MO Application  */
/*                  object) (KEC)                                           */
/* 18Nov08 S0483854 Remove auto_promote_flag from scenarios table (KEC)     */
/* 14Jan09 S0483854 Remove previously added code that creates the SASFolder */
/*         subfolder hierarchy in the SMC and adds the published solution   */
/*         library and tables to that subfolder.  The midtier will now      */
/*         perform this function (KEC)                                      */
/* 20Nov09 S0556131 Don't register promoted solution dataset in the         */
/*         MO Promoted Solutions SASLibrary if the scenario is associated   */
/*         with input data that is an optimization group. (KEC)             */
/*                                                                          */
/* F1MO54, F0MO541                                                          */
/* 09Jan12 S0823209 Replace use of SQL query of Dictionay Tables with use   */
/*         of SAS Proc Contents.                                            */
/*                                                                          */
/****************************************************************************/

%macro mo_promote_solution(scenario_id             =
                          ,library_metadata_id     =
                          ,name                    =
                          ,description             =
                          ,notify                  =
                          ,locale                  =
                          ,publish                 =
                          ,olap                    =
                          ,olap_cube_name          =
                          ,olap_cube_description   =
                          ,olap_cube_overwrite     =
                          ,olap_schema_name        =
                          ,olap_schema_metadata_id =
                          ,imap                    =
                          ,imap_name               =
                          ,imap_description        =
                          ,imap_overwrite          =
                          ,imap_mappath            =
                          ,promote_automatically   =
                          ,_rc                     =
                          ,_nc                     =
                          );


   %local macname;
   %let macname=&sysmacroname;
   %let macname=%lowcase(&macname);

   %if %bquote(&_rc) eq %str() %then %do;
      %local rc;
      %let _rc = rc;
   %end;
   %else %global &_rc;
   %let &_rc = 0;

   %if %bquote(&_nc) eq %str() %then %do;
      %local nc;
      %let _nc = nc;
   %end;
   %else %global &_nc;
   %let &_nc = 0;

   %mo_constants

   %*--------------------------------;
   %* Set defaults and check         ;
   %*--------------------------------;
   %if (%bquote(&name) eq %str()) %then %let name = &mo_solution;
   %if (%bquote(&publish) eq %str()) %then %let publish = 0;
   %if (%bquote(&locale) eq %str()) %then %do;
      %let locale = &mo_server_locale;
      %if (%bquote(&locale) eq %str()) %then %do;
         %put %mo_warning No locale detected. Obtaining locale directly from the server.;
         %let locale = %sysfunc(getpxLocale());
      %end;      
   %end;
   %let locale = %trim(%lowcase(&locale));
      
   /* Check whether the locale is of POSIX form (XX_XX) */
   %let length_locale = %length(&locale);
   %let underscore_locale = %index(&locale,_);
   %if (&length_locale ne 5) or (&underscore_locale ne 3) %then %do;
      %put %mo_warning The locale is not in POSIX form (XX_XX). Obtaining locale directly from the server.;
      %let locale = %sysfunc(getpxLocale());
   %end;
   
   %if (%bquote(&olap) eq %str()) %then %let olap = 0;
   %if (&olap) %then %do;
      %if (%bquote(&olap_cube_name) eq %str()) %then %let olap_cube_name = &scenario_name.;
      %if (%bquote(&olap_cube_overwrite) eq %str()) %then %let olap_cube_overwrite = &mo_olap_cube_overwrite;
   %end;

   %if (%bquote(&imap) eq %str()) %then %let imap = 1;
   %if (%bquote(&imap_name) eq %str()) %then %let imap_name = &olap_cube_name;
   %if (%bquote(&imap_overwrite) eq %str()) %then %let imap_overwrite = 0;
   %if (%bquote(&imap_mappath) eq %str()) %then %let imap_mappath = &mo_sasfolder_maps;

   %if (%bquote(&promote_automatically) eq %str()) %then %let promote_automatically = 0;

   %if (%bquote(&scenario_id) eq %str()) %then %do;
      %put %mo_error No scenario_id was specified.;
      %let &_rc = 102;
      %goto ERROR_EXIT;
   %end;
   %if (%bquote(&library_metadata_id) eq %str() and &publish) %then %do;
      %put %mo_error No library_metadata_id was specified.;
      %let &_rc = 102;
      %goto ERROR_EXIT;
   %end; 


   %*--------------------------------;
   %* Retrieve metadata              ;
   %*--------------------------------;
   %mo_no_notes(off)
   %local scenario_name project_id user_id;
   data _null_;
      set mometa.scenarios;
      if (scenario_id eq &scenario_id) then do;
         call symput('scenario_name',strip(scenario_name));
         call symput('project_id',project_id);
         call symput('user_id',user_id);
      end;
   run;
   %if (%bquote(&project_id) eq %str()) %then %do;
      %let &_rc = 101;
      %put %mo_error Invalid metadata detected.;
      %put %mo_error Project_id not identified for scenario_id = &scenario_id..;
      %goto ERROR_EXIT;
   %end;
   %if (%bquote(X&scenario_name) eq %str(X)) %then %do;
      %let &_rc = 101;
      %put %mo_error Invalid metadata detected.;
      %put %mo_error Scenario_name not identified for scenario_id = &scenario_id..;
      %goto ERROR_EXIT;
   %end;
   %if (%bquote(&user_id) eq %str()) %then %do;
      %let &_rc = 101;
      %put %mo_error Invalid metadata detected.;
      %put %mo_error User_id not identified for scenario_id = &scenario_id..;
      %goto ERROR_EXIT;
   %end;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error occurred while reading metadata (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;

   %local project_name input_data_id;
   data _null_;
      set mometa.projects;
      if (project_id eq &project_id) then do;
         call symput('project_name',strip(project_name));
         call symput('input_data_id',strip(input_data_id));
      end;
   run;
   %if (%bquote(X&project_name) eq %str(X)) %then %do;
      %let &_rc = 101;
      %put %mo_error Invalid metadata detected.;
      %put %mo_error Project_name not identified for project_id = &project_id..;
      %goto ERROR_EXIT;
   %end;
   %if (%bquote(&input_data_id) eq %str()) %then %do;
      %let &_rc = 101;
      %put %mo_error Invalid metadata detected.;
      %put %mo_error Input_data_id not identified for project_id = &project_id..;
      %goto ERROR_EXIT;
   %end;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error occurred while reading metadata (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;

   %local username uname;
   data _null_;
      set mometa.users;
      if (user_id eq &user_id) then do;
         call symput('username',strip(username));
         call symput('uname',strip(name));
      end;
   run;
   %if (%bquote(&username) eq %str()) %then %do;
      %let &_rc = 101;
      %put %mo_error Invalid metadata detected.;
      %put %mo_error Username not identified for user_id = &user_id..;
      %goto ERROR_EXIT;
   %end;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error occurred while reading metadata (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;

   %local cell_cd_column_status;
   data _null_;
      set mometa.input_data;
      if (input_data_id eq &input_data_id) then do;
         call symput('cell_cd_column_status',strip(cell_cd_column_status));
      end;
   run;
   %if (%bquote(&cell_cd_column_status) eq %str()) %then %do;
      %let &_rc = 101;
      %put %mo_error Invalid metadata detected.;
      %put %mo_error Cell_cd_column_status not identified for input_data_id = &input_data_id..;
      %goto ERROR_EXIT;
   %end;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error occurred while reading metadata (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;
   %mo_no_notes(off)


   %*--------------------------------;
   %* Get Library Info               ;
   %*--------------------------------;
   %mo_no_notes(off)

   %if &publish = 0 %then %do;
      %let library_metadata_id = ;
      %mo_get_metadata_objects(objtype  = SASLibrary
                              ,search   = *[@Name='&mo_promoted_solutions']
                              ,out      = work.promoted
                              ,_rc      = omrrc
                              )
      %if (&omrrc ne 0) %then %do;
         %put %mo_error Error retrieving SASLibrary from OMR (omrrc=&omrrc).;
         %let &_rc=100;
      %end;

      proc sql noprint;
         select Id into :library_metadata_id
         from work.promoted;
      quit;
      %if (&sqlrc gt 4 or &sqlobs eq 0) %then %do;
         %put %mo_error SQL error retrieving library metadata.;
         %let &_rc = 107;
         %goto ERROR_EXIT;
      %end;

      %let library_metadata_id = %trim(%left(&library_metadata_id));
   %end;

   %* Parse library metadata ID for use by PROC METALIB (9.2);
   %local repositoryId libraryId;
   %let repositoryId = %scan(&library_metadata_id, 1, .);
   %let libraryId = %scan(&library_metadata_id, 2, .);
   %if ((&repositoryId eq ) or (&libraryId eq )) %then %do;
      %let &_rc = 102;
      %put %mo_error Incorrectly formatted metadata library ID.;
      %goto ERROR_EXIT;
   %end;

   %* Get associated directory path;
   %mo_get_metadata_objects(objtype   = Directory
                           ,search    = *[UsedByPackages/SASLibrary[@Id='&library_metadata_id']]
                           ,attribute = DirectoryName
                           ,length    = 4096
                           ,out       = work.mo_path
                           ,_rc       = omrrc
                           );
   %if (&omrrc ne 0) %then %do;
      %let &_rc = 100;
      %put %mo_error Error retrieving directory metadata from OMR (omrrc=&omrrc).;
      %goto ERROR_EXIT;
   %end;

   %let path=;
   proc sql noprint;
      select DirectoryName into :path
      from work.mo_path;
   quit;
   %if (&sqlrc gt 4 or &sqlobs eq 0) %then %do;
      %put %mo_error SQL error retrieving directory metadata.;
      %let &_rc = 107;
      %goto ERROR_EXIT;
   %end;

   %let path=%trim(%left(&path));

   %mo_no_notes(on)
   libname _moprom_ "&path";
   %if (&syslibrc ne 0) %then %do;
      %put %mo_error Error assigning _MOPROM_ libref (syslibrc=&syslibrc);
      %let &_rc = 109;
      %goto ERROR_EXIT;
   %end;

   libname _mosolu_ "%mo_pathname(&moproj.&separator.&username.&separator.&project_name.&separator.&scenario_name.&separator.solution)";
   %if (&syslibrc ne 0) %then %do;
      %put %mo_error Error assigning _MOSOLU_ libref (syslibrc=&syslibrc);
      %let &_rc = 109;
      %goto ERROR_EXIT;
   %end;


   %*--------------------------------;
   %* Publish or Promote             ;
   %*--------------------------------;
   %if (&publish) %then %do;

      %*;
      %* Publish all output datasets;
      %*;
      proc contents data=_mosolu_._all_ out=work.solutionTables_tmp (keep=memname) noprint nodetails;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered using proc contents to get the list of tables in _MOSOLU_ (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;

      %local tables;
      proc sql noprint;
         select distinct memname into :tables separated by ' '
         from work.solutionTables_tmp;
      quit;
      %if (&sqlrc gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error occurred while determining tables in solution library (sqlrc=&sqlrc).;
         %goto ERROR_EXIT;
      %end;


      %let wcval = %mo_word_count(&tables);
      %do s=1 %to &wcval;
         %let table=%scan(&tables,&s);

         %if (%bquote(&olap) and (%upcase(&table) eq %upcase(&mo_solution))) %then %do;

            %*;
            %* Join communications table with solution if generating an olap cube;
            %*;

            %mo_generate_communications_view(input_data_id=&input_data_id, _rc=gcvrc)
            %if (&gcvrc gt 0) %then %do;
               %let &_rc = 100;
               %put %mo_error Error occurred while generating communications data.;
               %goto ERROR_EXIT;
            %end;

            %* Create a time_period format if that column exists;
            %mo_no_notes(off)
            proc contents data=work._mo_olap_communications out=work._mo_olap_comm_out noprint;
            run;

            proc sql noprint;
               select * from work._mo_olap_comm_out
               where (upcase(name) eq 'TIME_PERIOD');
            quit;

            %global olap_timeperiods_exist;
            %let olap_timeperiods_exist = &sqlobs;

            %if (&olap_timeperiods_exist) %then %do;

               libname _moolap_ "&moolap";
               %if (&syslibrc ne 0) %then %do;
                  %put %mo_error Error assigning _MOOLAP_ libref (syslibrc=&syslibrc);
                  %let &_rc = 109;
                  %goto ERROR_EXIT;
               %end;

               %* Generate format name;
               %let index=1;
               %if (%sysfunc(exist(_moolap_.formats,catalog))) %then %do;;
                  proc format cntlout=fmtout library=_moolap_;
                  run;

                  proc sql noprint;
                     create table work.mo_formats as
                     select distinct(fmtname)
                     from work.fmtout;
                  quit;

                  %let index=0;
                  %do %until (&sqlobs eq 0);
                     %let index=%eval(&index+1);
                     proc sql noprint;
                        select * from work.mo_formats
                        where upcase(fmtname) eq "MO_TIMEPERIOD&index._";
                     quit;
                  %end;
               %end;

               %let fmtname = mo_timeperiod&index._;

               data work.time_period_map (keep=label start fmtname type);
                  length label $32.;
                  length orderc $12.;
                  set mometa.time_periods;
                  where input_data_id = &input_data_id;

                  orderc=left(put(order,12.));
                  label=time_period;
                  start=orderc;
                  fmtname="&fmtname";
                  type='N';
               run;

               %mo_no_notes(on)
               proc format cntlin=work.time_period_map library=_moolap_;
               run;
               %if (&syserr gt 4) %then %do;
                  %let &_rc = 100;
                  %put %mo_error Error occurred while creating time_period format (syserr=&syserr).;
                  %goto ERROR_EXIT;
               %end;

               libname _moolap_ clear;
            %end;

            %* Double-check consistent appearance of cell_cd column in MO    ;
            %* datastore and in MO solution. Note: This check may be removed ;
            %* after testing is complete.                                    ;
            %mo_no_notes(off)
            proc contents data=_mosolu_.&mo_solution out=work.soln_vars noprint;
            run;
            %if (&syserr gt 4) %then %do;
               %let &_rc = 100;
               %put %mo_error Error occurred while determining MO solution dataset columns (syserr=&syserr).;
               %goto ERROR_EXIT;
            %end;

            %local cell_cd_in_soln;
            %local cell_cd_in_datastore;
            %let cell_cd_in_soln = 0;
            %let cell_cd_in_datastore = 0;
            proc sql noprint;
               select count(*) into :cell_cd_in_soln from work.soln_vars
               where (lowcase(name) eq 'cell_cd');
               select count(*) into :cell_cd_in_datastore from work._mo_olap_comm_out
               where (lowcase(name) eq 'cell_cd');
            quit;
            %if (&sqlrc gt 4) %then %do;
               %let &_rc = 100;
               %put %mo_error Error occurred while determining cell_cd existence in datastore and solution (sqlrc=&sqlrc).;
               %goto ERROR_EXIT;
            %end;

            %if (((&cell_cd_column_status ne &_CELL_CD_NOT_PRESENT_) and
                  ((&cell_cd_in_soln eq 0) or (&cell_cd_in_datastore eq 0))) or
                 ((&cell_cd_column_status eq &_CELL_CD_NOT_PRESENT_) and
                  ((&cell_cd_in_soln eq 1) or (&cell_cd_in_datastore eq 1)))) %then %do;
               %let &_rc = 101;
               %put %mo_error Inconsistent use of cell_cd column in input data or MO datastore or MO solution, input data = &cell_cd_column_status, MO datastore = &cell_cd_in_datastore, MO solution = &cell_cd_in_soln.;
               %goto ERROR_EXIT;
            %end;


            %mo_no_notes(off)
            %* Do the join;
            proc sql noprint;
               create view work.&mo_solution._view as
               select a.*, b.*
               from _mosolu_.&mo_solution as a, work._mo_olap_communications as b
               where ((a.campaign_cd = b.campaign_cd) and 
                      (a.communication_cd = b.communication_cd)
                   %if (&cell_cd_in_soln) %then %do;
                      and (a.cell_cd = b.cell_cd)
                   %end;
                     );
            quit;
            %if (&sqlrc gt 4) %then %do;
               %let &_rc = 100;
               %put %mo_error Error occurred while joining communication data to mo_solution (sqlrc=&sqlrc).;
               %goto ERROR_EXIT;
            %end;

            %mo_no_notes(on)
            data _moprom_.&mo_solution (compress=yes);
               set work.&mo_solution._view;
            run;
            %if (&syserr gt 4) %then %do;
               %let &_rc = 100;
               %put %mo_error Error occurred while creating olap version of &table dataset (syserr=&syserr).;
               %goto ERROR_EXIT;
            %end;

            %mo_no_notes(off)
            proc datasets library=work nolist nowarn;
               delete &mo_solution;
            quit;
         %end;
         %else %do;
            %*;
            %* Otherwise just copy table;
            %*;

            %mo_no_notes(on)
            data _moprom_.&table (compress=yes);
               set _mosolu_.&table;
            run;
            %if (&syserr gt 4) %then %do;
               %let &_rc = 100;
               %put %mo_error Error occurred while copying &table dataset (syserr=&syserr).;
               %goto ERROR_EXIT;
            %end;
         %end;

         %* Register table in metadata;
         proc metalib;
            omr (libid="&libraryId" repid="&repositoryId");
            select (&table);
         run;
         %if (&syserr gt 4) %then %do;
            %let &_rc = 113;
            %put %mo_error Error occurred while registering &table dataset metadata (syserr=&syserr).;
            %goto ERROR_EXIT;
         %end;
      %end;


      %*;
      %* Create the cube;
      %*;
      %if (%bquote(&olap)) %then %do;
         %mo_rpt_olap_cube(scenario_id             = &scenario_id
                          ,library_metadata_id     = &library_metadata_id
                          ,detail_table            = &mo_solution
                          ,olap_cube_name          = &olap_cube_name
                          ,olap_cube_description   = &olap_cube_description
                          ,olap_cube_overwrite     = &olap_cube_overwrite
                          ,olap_schema_name        = &olap_schema_name
                          ,olap_schema_metadata_id = &olap_schema_metadata_id
                       %if (&olap_timeperiods_exist) %then %do;
                          ,olap_time_period_format = &fmtname..
                       %end;
                       %else %do;
                          ,olap_time_period_format =
                       %end;
                          ,olap_include_cell_cd    = &cell_cd_in_soln
                          ,imap                    = &imap
                          ,imap_name               = &imap_name
                          ,imap_description        = &imap_description
                          ,imap_overwrite          = &imap_overwrite
                          ,imap_mappath            = &imap_mappath
                          ,path                    = &path
                          ,locale                  = &locale
                          ,_rc                     = orc
                          )
         %if (&orc gt 0) %then %do;
            %let &_rc = 223;
            %put %mo_error Error occurred while creating olap cube for scenario = &scenario_id (orc=&orc).;
         %end;

         %symdel olap_timeperiods_exist;
      %end;
   %end;

   %else %do;

      %*;
      %* Promote solution;
      %*;

      %* Skip promote if scheduled optimization failed;
      %global moerror;
      %if (&promote_automatically) and (%bquote(&moerror) ne %str(0)) %then %goto OPT_EXIT;

      %* Append timestamp to promoted solution dataset name for lights-out;
      %if (&promote_automatically) %then %do;
         %if (%length(&name) gt 18) %then %let name = %substr(&name,1,18);
         data _null_;
            dt = datetime() - (&mo_GMT_offset);
            datepart = datepart(dt);
            timepart = timepart(dt);
            datenum = put(datepart,yymmdd6.);
            timenum = strip(compress(put(timepart,time8.),':'));
            if (length(timenum) eq 5) then timenum='0'||timenum;
            append = '_'||datenum||'_'||timenum;
            call symput('timestamp_append',append);
         run;
         %let name=&name.&timestamp_append;
      %end;

      %* Copy the solution dataset;
      %mo_no_notes(on)
      data _moprom_.&name (compress=yes);
         set _mosolu_.&mo_solution;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error occurred while copying solution dataset (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;

      %* Register solution dataset metadata ;
      %* if NOT an optimization group.      ;
      %local isOptGroup;
      %let isOptGroup = N;

      data _null_;
         set mometa.input_data;
         if (input_data_id = &input_data_id) then do;
            if (moma_input ~= &_NOT_MOMA_INPUT_) then do;
               call symput("isOptGroup", "Y");
            end;
         end;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error occurred while reading input_data table to determine if scenario is associated with an optimization group (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;

      %if (&isOptGroup eq N) %then %do;
         proc metalib;
            omr (libid="&libraryId" repid="&repositoryId");
            select (&name);
         run;
         %if (&syserr gt 4) %then %do;
            %let &_rc = 113;
            %put %mo_error Error occurred while registering &name dataset metadata (syserr=&syserr).;
            %goto ERROR_EXIT;
         %end;

         %* Set Desc metadata attribute if running lights out;
         %if (%bquote(&description) ne %str()) %then %do;
            %mo_get_metadata_objects(objtype  = PhysicalTable
                                    ,search   = *[TablePackage/SASLibrary[@Id='&library_metadata_id']]
                                    ,out      = work.motables
                                    ,_rc      = omrrc
                                    );

            data _null_;
               set work.motables;
               if (upcase(Name) eq "%upcase(&name)") then call symput('table_metadata_id',id);
            run;

            %mo_no_notes(on)
            data _null_;
               rc1=metadata_setattr("omsobj:&table_metadata_id","Name","&name");
               rc2=metadata_setattr("omsobj:&table_metadata_id","Desc","&description");
               call symput('rc1',strip(rc1));
               call symput('rc2',strip(rc2));
            run;
            %if (&syserr gt 4 or &rc1 ne 0 or &rc2 ne 0) %then %do;
               %let &_rc = 200;
               %put %mo_warning Error occurred while updating &name dataset metadata (syserr=&syserr).;
            %end;
         %end;
      %end;

      %* Determine number of unique communications in promoted solution (used by MA);
      proc sql noprint;
         select count(*)
            into :&_nc
            from _mosolu_.offer_summary_nway
            where offers > 0;
      quit;
      %if (&sqlrc gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error occurred while determining number of communications in solution (sqlrc=&sqlrc).;
         %goto ERROR_EXIT;
      %end;

      %* Update promotion flags;
      %mo_create_tables(lib=work,table=scenarios,_rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 100;
         %put %mo_error Error occurred creating WORK.SCENARIOS table.;
         %goto ERROR_EXIT;
      %end;
      %mo_no_notes(off)
      data work.scenarios;
         set mometa.scenarios;
         where (scenario_id eq &scenario_id);

         promote = 1;
         promote_solution_dataset_name="&name";
         promote_solution_dataset_desc="&description";
         promote_num_communications = &&&_nc;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 103;
         %put %mo_error I/O error encountered while updating WORK.SCENARIOS table for scenario_id = &scenario_id (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;
      %mo_no_notes(on)

      %mo_record_update(metadata_table = mometa.scenarios
                       ,update_table   = work.scenarios
                       ,by             = scenario_id
                       ,updatemode     = nomissingcheck
                       ,_rc            = updrc);
      %if (&updrc ne 0) %then %do;
         %let &_rc = 103;
         %goto ERROR_EXIT;
      %end;
   %end;


   %*--------------------------------;
   %* Exit processing                ;
   %*--------------------------------;
   %* Successful completion;
   %if (&&&_rc eq 0) %then %do;
      %put;
      %if (&publish) %then
         %put %mo_note Successfully published scenario = &scenario_name with scenario_id = &scenario_id..;
      %else
         %put %mo_note Successfully promoted scenario = &scenario_name with scenario_id = &scenario_id..;
   %end;
   %else %do;
      %put;
      %if (&publish) %then
         %put %mo_warning Encountered abnormal condition publishing scenario = &scenario_name with scenario_id = &scenario_id (rc=&&&_rc).;
      %else
         %put %mo_warning Encountered abnormal condition promoting scenario = &scenario_name with scenario_id = &scenario_id (rc=&&&_rc).;
   %end;
   %goto EXIT;


   %ERROR_EXIT:
   %put;
   %put %mo_error Error encountered while promoting scenario_id = &scenario_id (rc=&&&_rc).;

   %* Update promotion flags;
   %mo_create_tables(lib=work,table=scenarios,_rc=cudrc)
   %if (&cudrc ne 0) %then %do;
      %let &_rc = 100;
      %put %mo_error Error occurred creating WORK.SCENARIOS table.;
      %goto EXIT;
   %end;
   %mo_no_notes(off)
   data work.scenarios;
      set mometa.scenarios;
      where (scenario_id eq &scenario_id);

      promote = 2;
      promote_solution_dataset_name='';
      promote_solution_dataset_desc='';
      promote_num_communications=0;
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 103;
      %put %mo_error I/O error encountered while updating WORK.SCENARIOS table for scenario_id = &scenario_id (syserr=&syserr).;
      %goto EXIT;
   %end;
   %mo_no_notes(on)

   %mo_record_update(metadata_table = mometa.scenarios
                    ,update_table   = work.scenarios
                    ,by             = scenario_id
                    ,updatemode     = nomissingcheck
                    ,_rc            = updrc);
   %if (&updrc ne 0) %then %do;
      %let &_rc = 103;
      %goto EXIT;
   %end;
   
   %goto EXIT;


   %OPT_EXIT:
   %put;
   %put %mo_error Promotion of scenario_id = &scenario_id not executed due to optimization problems (moerror=&moerror).;
   %put;


   %EXIT:

   %* Clean up;
   libname _moprom_ clear;
   libname _mosolu_ clear;

%mend mo_promote_solution;
