/****************************************************************************/
/* Copyright (c) 2005-2011, SAS Institute Inc., Cary, NC, USA,              */
/*                       All Rights Reserved.                               */
/*                                                                          */
/* Name:    mo_refresh_restore_original_ids                                 */
/*                                                                          */
/* Purpose: During the input data refresh process, the original input_data, */
/*          projects, and scenarios are copied into new entries (rows)      */
/*          in the input_data, projects, and scenarios tables with new      */
/*          ID values. The old rows with the original entries are deleted.  */
/*          This macro restores the original input_data_id, project_id, and */
/*          scenario_id values for the refreshed entries.                   */
/*                                                                          */
/* Input:   restore_ids_table - SAS dataset containing the following        */
/*          variables for each unique input_data/project/scenario           */
/*          combination impacted by the refresh:                            */
/*             - original input_data_id                                     */
/*             - new input_data_id                                          */
/*             - original project_id                                        */
/*             - new project_id                                             */
/*             - original scenario_id                                       */
/*             - new scenario_id                                            */
/*          lock - Indicates whether the UID lock needs to be obtained. Set */
/*          to LOCK if the invoking macro has NOT locked the UID table or   */
/*          to NOLOCK otherwise.                                            */
/*                                                                          */
/* Output:  _rc = Return code indicating success (0) or failure (non-0) of  */
/*                the ID restoration process                                */
/*                                                                          */
/* HISTORY:                                                                 */
/* 5.3:                                                                     */
/* 28Jul08 Creation (KEC)                                                   */
/* 24Sep08 S0483854 Change mechanism for accessing MO Property value when   */
/*         running on 9.2 (Property objects on the MO Application object)   */
/*         (KEC)                                                            */
/*                                                                          */
/* F1MO54, F0MO541                                                          */
/* 09Jan12 S0823209 Replace use of SQL query of Dictionay Tables with use   */
/*         of SAS Proc Contents.                                            */
/*                                                                          */
/****************************************************************************/
%macro mo_refresh_restore_original_ids(restore_IDs_tbl = work.restore_IDs_table
                                      ,lock            = LOCK
                                      ,_rc             =
                                      );


   %local macname;
   %let macname=&sysmacroname;
   %let macname=%lowcase(&macname);

   %if %bquote(&_rc) eq %str() %then %do;
      %local rc;
      %let _rc = rc;
   %end;
   %else %do;
      %global &_rc;
   %end;
   %let &_rc = 0;

   %mo_constants;

   %if (%bquote(&restore_IDs_tbl) eq %str()) %then %do;
      %put %mo_error No table containing IDs was specified.;
      %let &_rc = 102;
      %goto ERROR_EXIT;
   %end;


   %*---------------------------------;
   %* Useful macros.                  ;
   %*---------------------------------;

   %***** MACRO RESTORE_IDS *****;
   %macro restore_IDs(IDs_tbl, num_obs, metadata_table,
                      has_idid_col, has_pid_col, has_sid_col, _rc);

      %local macname;
      %let macname=&sysmacroname;
      %let macname=%lowcase(&macname);

      %if %bquote(&_rc) eq %str() %then %do;
         %local rc;
         %let _rc = rc;
      %end;
      %else %do;
         %global &_rc;
      %end;
      %let &_rc = 0;

      %mo_create_tables(lib=work, table=&metadata_table, _rc=cudrc)
      %if (&cudrc ne 0) %then %do;
         %let &_rc = 100;
         %put %mo_error Error occurred creating %upcase(WORK.&metadata_table) table.;
      %end;

      %if (%sysfunc(exist(work.&metadata_table))) %then %do;
         %do row = 1 %to &num_obs;
            data _null_;
               row = &row;
               set &IDs_tbl point=row;
               call symput("orig_idid", put(original_input_data_id, 8.));
               call symput("new_idid", put(new_input_data_id, 8.));
               call symput("orig_pid", put(original_project_id, 8.));
               call symput("new_pid", put(new_project_id, 8.));
               call symput("orig_sid", put(original_scenario_id, 8.));
               call symput("new_sid", put(new_scenario_id, 8.));
               stop;
            run;
   
            proc sql noprint;
               delete from work.&metadata_table;
   
               insert into work.&metadata_table
               select * from mometa.&metadata_table
               where (
                      %if (&has_idid_col) %then %do;
                         (input_data_id = &new_idid)
                         %if ((&has_pid_col) or (&has_sid_col)) %then %do;
                            and
                         %end;
                      %end;
                      %if (&has_pid_col) %then %do;
                         (project_id = &new_pid)
                         %if (&has_sid_col) %then %do;
                            and
                         %end;
                      %end;
                      %if (&has_sid_col) %then %do;
                         (scenario_id = &new_sid)
                      %end;
                      );
            quit;
            %if (&sqlrc gt 4) %then %do;
               %let &_rc = 107;
               %put %mo_error SQL error encountered while retrieving rows from %upcase(WORK.&metadata_table) table (sqlrc=&sqlrc).;
            %end;
   
            data work.&metadata_table;
               set work.&metadata_table;
               %if (&has_idid_col) %then %do;
                  input_data_id = &orig_idid;
               %end;
               %if (&has_pid_col) %then %do;
                  project_id = &orig_pid;
               %end;
               %if (&has_sid_col) %then %do;
                  scenario_id = &orig_sid;
               %end;
            run;
            %if (&syserr gt 4) %then %do;
               %let &_rc = 100;
               %put %mo_error Error encountered while updating %upcase(WORK.&metadata_table) table (syserr=&syserr).;
            %end;
   
            %mo_no_notes(on)
            %mo_record_delete(metadata_table = mometa.&metadata_table
                             ,where          = %if (&has_idid_col) %then %do;
                                                  (input_data_id = &new_idid)
                                                  %if ((&has_pid_col) or 
                                                       (&has_sid_col)) %then %do;
                                                     and
                                                  %end;
                                               %end;
                                               %if (&has_pid_col) %then %do;
                                                  (project_id = &new_pid)
                                                  %if (&has_sid_col) %then %do;
                                                     and
                                                  %end;
                                               %end;
                                               %if (&has_sid_col) %then %do;
                                                  (scenario_id = &new_sid)
                                               %end;
                             ,_rc            = delrc);
            %if (&delrc ne 0) %then %do;
               %let &_rc = 103;
            %end;
   
            %mo_record_append(metadata_table = mometa.&metadata_table
                             ,update_table   = work.&metadata_table
                             ,_rc            = updrc);
            %if (&updrc ne 0) %then %do;
               %let &_rc = 103;
            %end;
         %end;      
      %end;      
      %else %do;
         %let &_rc = 300;
         %put %mo_warning %upcase(WORK.&metadata_table) table was not created because %upcase(&metadata_table) is not a table name known within Marketing Optimization.;
      %end;
   %mend restore_IDs;


   %*---------------------------------;
   %* Obtain lock for metadata update ;
   %*---------------------------------;
   %if (%upcase(&lock) eq LOCK) %then %do;
      %mo_lock_dataset(data=mometa.uid,_rc=lckrc)
      %if (&lckrc ne 0) %then %do;
         %put %mo_error Error obtaining lock of MOMETA.UID.;
         %let &_rc = 106;
         %goto ERROR_EXIT;
      %end;
   %end;


   %*----------------------------------------------------------------;
   %* For each metadata table that has an input_data_id, project_id, ;
   %* and/or scenario_id, restore original IDs for those entries     ;
   %* (rows) that were assigned new IDs during the refresh process.  ;
   %*----------------------------------------------------------------;
   %local num_restore_obs;
   data _null_;
      set &restore_IDs_tbl nobs=numObs;
      call symput("num_restore_obs", put(numObs, 8.));
      stop;
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered obtaining observation count from &restore_IDs_tbl table (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;

   proc contents data=mometa._all_ out=work.tableNames_tmp (keep=memname name) noprint nodetails;
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered using proc contents to get the list of tables in MOMETA (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;

   %local table_list;
   %let table_list = ;
   proc sql noprint;
      select distinct memname into :table_list separated by ' '
      from work.tableNames_tmp
      where (lowcase(name) in ("input_data_id", "project_id", "scenario_id"));
   quit;
   %if (&sqlrc gt 4) %then %do;
      %put %mo_error SQL error occurred retrieving data from WORK.TABLENAMES_TMP (sqlrc=&sqlrc).;
      %let &_rc = 107;
      %goto ERROR_EXIT;
   %end;
   %let table_list = %lowcase(&table_list);
   %let num_tables = %mo_word_count(&table_list);

   %local loop;
   %local has_input_data_id_col has_project_id_col has_scenario_id_col;
   %do loop = 1 %to &num_tables;
      %let current_table = %scan(&table_list, &loop);
      %let has_input_data_id_col = 0;
      %let has_project_id_col = 0;
      %let has_scenario_id_col = 0;

      %let col_names=;
      proc contents noprint data=mometa.&current_table out=work.col_names (keep=name);
      run;
      proc sql noprint;
         select name into :col_names separated by " " 
         from work.col_names;
      quit; 
      %let col_names = %lowcase(&col_names);

      %let has_input_data_id_col = %eval(%index(&col_names, input_data_id) > 0);
      %let has_project_id_col = %eval(%index(&col_names, project_id) > 0);
      %let has_scenario_id_col = %eval(%index(&col_names, scenario_id) > 0);
      %if ((&has_input_data_id_col) or (&has_project_id_col) or 
           (&has_scenario_id_col)) %then %do;
         %restore_IDs(&restore_IDs_tbl, &num_restore_obs, &current_table, 
                      &has_input_data_id_col, &has_project_id_col, 
                      &has_scenario_id_col, idrc);
      %end;
   %end;


   %*---------------------------------;
   %* Exit processing                 ;
   %*---------------------------------;
   %goto EXIT;

   %ERROR_EXIT:
   %put;
   %put %mo_error Error restoring original input_data_id, project_id, and scenario_id values (rc=&&&_rc).;

   %EXIT:
   %if (&lock eq LOCK) %then %do;
      %mo_unlock_dataset(data=mometa.uid)
   %end;

%mend mo_refresh_restore_original_ids;
