/****************************************************************************************/
/* Copyright (c) 2005-2011, SAS Institute Inc., Cary, NC, USA,                          */
/*                       All Rights Reserved.                                           */
/*                                                                                      */
/* Name:    mo_validate_input_data                                                      */
/*                                                                                      */
/* Purpose: Validate MO input datasets                                                  */
/*                                                                                      */
/* Input    library_metadata_id     - OMR SASLibrary FQID                               */
/*          moma_input              - Flag to indicate if input data is MOMA            */
/*          uses_prioritization     - Flag to indicate priority-based input data        */
/*          uses_contact_history    - Flag to indicate contact history in input data    */
/*          customer_table          - Name of Customer Attributes table                 */
/*          cust_col_metadata_table - Name of Customer Control table                    */
/*          campaign_table          - Name of Campaigns table                           */
/*          communication_table     - Name of Communications table                      */
/*          channel_table           - Name of Channel table                  (optional) */
/*          time_period_table       - Name of Time Period table              (optional) */
/*          agent_capacity_table    - Name of Agent table                    (optional) */
/*          constraints_table       - Name of Constraints table              (optional) */
/*          contact_policy_table    - Name of Contact Policies table         (optional) */
/*          contact_history_table   - Name of Contact History table          (optional) */
/*          communication_history_table - Name of Communication History table(optional) */
/*          customer_id_column      - Name of Customer Identifer column      (optional) */
/*          household_id_column     - Name of Household Identifier column    (optional) */
/*          customer_rank_measure   - Name of Customer Rank measure          (optional) */
/*          priority_column         - Name of Priority column                (optional) */
/*          agent_column            - Name of Agent column                   (optional) */
/*                                                                                      */
/* Output:  &_time_period_indicator - Name of macro variable into which to return an    */
/*                                    indication of whether time period values are      */
/*                                    specified in the input data.                      */
/*          &_cell_cd_indicator     - Name of macro variable into which to return an    */
/*                                    indication of whether a cell_cd column exists in  */
/*                                    the Customer Control and Communications table     */
/*                                    that is required, along with communication_cd and */
/*                                    campaign_cd, to uniquely identify a communication.*/
/*          &_rc = return code                                                          */
/*                                                                                      */
/* Return Codes:                                                                        */
/*          0     completed validation with no warnings nor errors detected             */
/*          4     completed validation with warnings but no errors detected             */
/*          8     completed validation with errors detected                             */
/*                                                                                      */
/*                                                                                      */
/* HISTORY:                                                                             */
/* 09Apr04 S0244828 Creation (dwh)                                                      */
/* 04Aug04 S0261122 Check for numeric_field length greater than $28. (dwh)              */
/* 06Aug04 S0263394 Allow mixed case column names in CUSTOMER and CUSTOMER_CONTROL (dwh)*/
/* 09Aug04 S0263395 Check for reserved columns in CUSTOMER and CUSTOMER_CONTROL (dwh)   */
/* 09Aug04 S0262692 Validate a mean exists for every variance in CUSTOMER_CONTROL (dwh) */
/* 11Aug04 S0256259 Detect missing groups, subgroups, time_periods and set flags (dwh)  */
/* 16Aug04 S0256465 Send input data validation errors to client (dwh)                   */
/* 18Aug04 S0255874 Removed check for blank camp_desc and comm_desc (dwh)               */
/* 30Aug04 S0266199 Clarify numeric_field length validation warning (dwh)               */
/* 30Aug04 S0266337 Remove notes from message text file returned to the client (dwh)    */
/* 13Sep04 S0267783 Make column check case insensitive (dwh)                            */
/*                                                                                      */
/* 4.3:                                                                                 */
/* 11Oct04 S0256473 Convert GUI messages for I18N (dwh)                                 */
/* 11Oct04 S0268954 Warn when defined time_periods not specified in communcations (dwh) */
/* 07Dec04 S0268539 Handle leading blanks (dwh)                                         */
/* 08Dec04 S0270524 Fail immediately if column attributes are not correct (dwh)         */
/* 13Jan05 S0273454 Input data defined in OMR (dwh)                                     */
/* 15Mar05 S0273454 Validate agent constraints table (yl)                               */
/* 27May05 S0302470 Disallow embedded or leading spaces in camp_code, comm_code (dwh)   */
/* 03Jun05 S0303133 Specify SUMSIZE when checking unique cust_ids (dwh)                 */
/*                                                                                      */
/* 4.4:                                                                                 */
/* 08Dec05 S0308315 User PROC SQL instead of SUMMARY for unique customer_id check (dwh) */
/* 09Dec05 S0323564 Fix return codes to detect error with subsequent warning (dwh)      */
/* 09Dec05 S0310100 Detect multiple missing agent capacities (dwh)                      */
/* 09Dec05 S0307510 Check that contact_policy_segment_column is character (dwh)         */
/* 20Feb06 S0312209 Add product_values_exist column (dwh)                               */
/* 02Mar06 S0347842 Allow non-unique column_name in CUSTOMER_CONTROL table (dwh)        */
/* 20Mar06 S0349410 Change infeasibilies from ERRORs to WARNINGs (dwh)                  */
/* 22Mar06 S0349405 Make non-required input data columns optional (dwh)                 */
/* 20Apr06 S0355117 Do not check for agent column in the communication table (dwh)      */
/* 11May06 S0358726 Add QUIT after PROC SQL statements (dwh)                            */
/* 16May06 S0359034 Add logical expression operators to reserved word list (dwh)        */
/* 17May06 S0359029 Check for columns/tables that are defined in metadata but not in    */
/*                  the underlying table (dwh)                                          */
/* 22May06 S0359844 Check for character attribute columns gt 256 (dwh)                  */
/* 02Jun06 S0361315 Check for blanks in case reading from Oracle (dwh)                  */
/* 07Jun06 S0361134 Support a SEQUENCE column in the timeperiod table for Oracle (dwh)  */
/* 08Jun06 S0359844 Treat character attribute column gt 256 as an error condition (dwh) */
/* 08Jun06 S0363037 Catch AGENT table with time_periods but no TIME_PERIOD table (dwh)  */
/*                                                                                      */
/* 5.1:                                                                                 */
/* 12Dec06 S0395314 Convert to new input data formats for 5.1 (micopp)                  */
/*                                                                                      */
/* 5.3:                                                                                 */
/* 27May08 S0483854 Introduce target_level as a new field in the                        */
/*         prepopulated_contact_policies and contact_policies tables.                   */
/*         This is a per-contact-policy variant of the household_level                  */
/*         field that was eliminated from the scenario_advanced_options                 */
/*         table. (KEC)                                                                 */
/* 15Jul08 S0483854 MOMA support - Add validation checks for cell_cd column in MOMA 5.3 */
/*         Communications and Customer Control input data tables. (KEC)                 */
/* 23Jul08 S0483854 Add support for "report only" constraint type.                      */
/* 31Jul08 S0525195 Issue warning if agent table / agent column is specified but        */
/*         no rows in the Communication table use agents.                               */
/* 05Aug08 S0482173 For Customer Control table, change the check of numeric_measure     */
/*         column values to enforce a maximum length of 26 characters (down from 28).   */
/*         For the Campaign, Communication, Customer, Channel, and TimePeriod tables    */
/*         introduce a new check on user-defined numeric columns to enforce a max       */
/*         length of 26 characters for the column name itself (since these may also     */
/*         serve as numeric measures. (KEC)                                             */
/* 06Aug08 S0518329 Change warning message issued to the GUI wizard results window and  */
/*         to the SAS log when filters are found that result in the selection of no     */
/*         communications or no customers. (KEC)                                        */
/* 06Aug08 S0408011 Add warning message issued to the GUI wizard results window and to  */
/*         the SAS log when a Channel table is specified, but the Communication table   */
/*         has no channel_cd column or has a channel_cd column but contains null values */
/*         for all rows. (KEC)                                                          */
/* 14Aug08 S0483854 MOMA support - Add validation checks for multi-part key customer ID */
/*         column name. (KEC)                                                           */
/* 27Oct08 S0543261 MOMA support - For Customer table column names of numeric type,     */
/*         other than explicitly identified numeric measures for which a 26-character   */
/*         restriction remains, change the 26-character length restriction to a         */
/*         standard 32-character length restriction. (KEC)                              */
/* 11Dec08 S0483854 Add validation checks for the new interpretation of the starte time */
/*         period and end time period columns in the Prepopulated Contact Policies      */
/*         Table, i.e., which are used to 1) define the scope at which a constraint     */
/*         is to be applied (all/aggregate, customer, household) and 2) whether the     */
/*         constraint is to be included in sensitivity analysis (if SA requested for a  */
/*         scenario optimization), respectively. (KEC)                                  */
/* 11Dec08 S0483854 Add validation checks for the two new columns in the Prepopulated   */
/*         Constraints table which are used to 1) define the scope at which a constraint*/
/*         is to be applied (all/aggregate, customer, household) and 2) whether the     */
/*         constraint is to be included in sensitivity analysis (if SA requested for a  */
/*         scenario optimization), respectively. (KEC)                                  */
/* 09Jan09 S0555734 Remove use of min function to "correct" rolling_period_cnt and      */
/*         blocking_period_cnt that is larger than the difference between the actual    */
/*         start period and actual end period (KEC)                                     */
/* 09Jan09 S0557725 Add validation check for Prepopulated Constraints.  The             */
/*         "include in sensitivity analysis" flag should not be allowed to be           */
/*         set to 1 (on) for report-only or customer-level constraints. Issue an error  */
/*         if this situation is found.                                                  */
/*         Add two other checks not covered by this defect that should result in a      */
/*         warning (not an error) if found : 1) Check for communication_cd values in    */
/*         the Communications table that do not exist in the Customer Control table.    */
/*         2) Check whether any of the selected customer attributes also exist as       */
/*         column_nm values in the Customer Control table.                              */
/* 09Jan09 S0557364 Add check similar to 2) above. Specifically, check whether any of   */
/*         the selected customer attributes also exist as numeric_measure values in the */
/*         Customer Control table.                                                      */
/* 18May09 S0562537 - Change the check for the number of distinct values of a customer  */
/*         attribute to use a configurable maximum, i.e., an mo property macro variable */
/*         and change the case where the number exceeds the maximum from a warning to   */
/*         an error (KEC).                                                              */
/*                                                                                      */
/* 5.4:                                                                                 */
/* 03Feb10 S0639443 Add validation checks for new mandatory inclusions columns in the   */
/*         Communications input data table (KEC)                                        */
/* 20Apr10 Enhancements while implementing scenario import (KEC)                        */
/* 20Jul10 S0684495 Fix handling of customer IDs longer than 8-bytes in customer ID     */
/*         uniqueness check block of code (KEC)                                         */
/* 28Jul10 S0662063 Bring forward changes from the hot fix branch f2mo53 for this       */
/*         file. Also, eliminated most "separated by" code for consistency (KEC).       */
/* 29Jul10 S0689073 Add check that flags an error on Constraints table rows for which   */
/*         the constraint_computation_cd is AVERAGE or SUMMARY, but the numeric_measure */
/*         value is blank (KEC)                                                         */
/* 29Jul10 S0689114 Add check that flags an error on Contact Policy table rows that     */
/*         indicate blocking policies, but for which the blocking filter text columns   */
/*         are blank (KEC)                                                              */
/* 19Aug10 Add support for new prioritization columns, new contact history              */
/*         columns. (KEC)                                                               */
/* 14Oct10 S0697719 For MOMA input data, bypass check for communications in the         */
/*         Communication table that do not exist in the Customer Control table. (KEC)   */
/* 02Nov10 S0714454 Flag communication filters that reference comm numeric attributes or*/
/*         comm character attributes for which all values are missing as invalid (KEC)  */
/* 09Jul12 S0838701 Make sure customer ID column(s), household ID column and agent      */
/*         column are excluded from enforcement of maximum of 1000 distinct values for  */
/*         customer character attributes. (KEC)                                         */
/* 09Jul12 S0797975 When creating working tables that correspond to customer input      */
/*         that fill in missing columns optional columns, exclude the customer ID       */ 
/*         from the contact history table (if present in the input data tables)         */
/*         in order to avoid the type conflict that will occur in the merge that is     */
/*         used to produce that working table for contact history if the customer       */
/*         contact history table has that column as a numeric, since we treat that      */
/*         column as a character. It is OK to exclude the customer ID from the contact  */
/*         history working table, because validation of that column in that table       */
/*         is not done via the working table (it is done by another mechanism). (KEC)   */
/****************************************************************************************/

%macro mo_validate_input_data(library_metadata_id                =
                             ,moma_input                         =
                             ,uses_prioritization                =
                             ,uses_contact_history               =
                             ,customer_table                     =
                             ,cust_col_metadata_table            =
                             ,campaign_table                     =
                             ,communication_table                =
                             ,channel_table                      =
                             ,time_period_table                  =
                             ,agent_capacity_table               =
                             ,constraints_table                  =
                             ,contact_policy_table               =
                             ,contact_history_table              =
                             ,communication_history_table        =
                             ,customer_id_column                 =
                             ,household_id_column                =
                             ,customer_rank_measure              =
                             ,priority_column                    =
                             ,agent_column                       =
                             ,_time_period_indicator             =
                             ,_cell_cd_indicator                 =
                             ,_rc                                =
                             );

   %local macname macn;
   %let macname=&sysmacroname;
   %let macname=%lowcase(&macname);

   %if %bquote(&_rc) eq %str() %then %do;
      %local rc;
      %let _rc = rc;
   %end;
   %else %global &_rc;
   %let &_rc = 0;

   %if (%bquote(&_time_period_indicator) eq %str()) %then %do;
      %put %mo_error Non-null value for _TIME_PERIOD_INDICATOR is required.;
      %let &_rc = 102;
      %goto ERROR_EXIT;
   %end;
   %else %do;
      %if (not %symglobl(&_time_period_indicator)) %then %do;
         %global &_time_period_indicator;
         %let &_time_period_indicator = ;
      %end;
   %end;

   %if (%bquote(&_cell_cd_indicator) eq %str()) %then %do;
      %put %mo_error Non-null value for _CELL_CD_INDICATOR is required.;
      %let &_rc = 102;
      %goto ERROR_EXIT;
   %end;
   %else %do;
      %if (not %symglobl(&_cell_cd_indicator)) %then %do;
         %global &_cell_cd_indicator;
         %let &_cell_cd_indicator = ;
      %end;
   %end;

   %if (&uses_prioritization eq 1) %then %do;
      %if (%bquote(&priority_column) eq %str()) %then %do;
         %put %mo_error Use of prioritization is indicated but no priority_column was specified.;
         %let &_rc = 102;
         %goto ERROR_EXIT;
      %end;
      %if (%bquote(&customer_rank_measure_name) eq %str()) %then %do;
          %put %mo_error Use of prioritization is indicated but no customer_rank_measure_name was specified.;
         %let &_rc = 102;
         %goto ERROR_EXIT;
      %end;
   %end;

   %if (&uses_contact_history eq 1) %then %do;
      %if (%bquote(&contact_history_table) eq %str()) %then %do;
         %put %mo_error Use of contact history is indicated but no contact_history_table was specified.;
         %let &_rc = 102;
         %goto ERROR_EXIT;
      %end;
      %if (%bquote(&communication_history_table) eq %str()) %then %do;
         %put %mo_error Use of contact history is indicated but no communication_history_table was specified.;
         %let &_rc = 102;
         %goto ERROR_EXIT;
      %end;
   %end;

   %mo_constants


   %*------------------;
   %* MO Reserved Words;
   %*------------------;
   %let reserved = 'and' 'or' 'not' 'in' 'lt' 'gt' 'le' 'ge' 'eq' 'ne';
   %let communication_reserved = (&reserved 'offer_id' 'order_no' 'input_data_id' 'campaign_id' 'campaign_code' 'campaign_description'
                                  'channel_id' 'channel' 'channel_type' 'channel_units' 'time_period_id' 'order'
                                  'communication_id' 'communication_code' 'communication_description'
                                  'comm_attribute_id' 'comm_char_attribute_id' 'comm_attribute_name' 'comm_attribute_value_id' 'comm_attribute_distinct_value_id'
                                  'comm_attribute_value');
   %let customer_control_reserved= (&reserved '_fraction' '_rc');
   %let customer_reserved = (&reserved 'segment_id' 'constraint_key' 'maxcontact_key' 'rolling_key' 'custattr_key' '_numpos' '_filterval' '_maxcontacts_');
   %let customer_reserved_suffix = _fraction_ _rc_ _maxcontacts_g _filterval_g _numpos_g;

   %mo_no_notes(off)
   %if (%bquote(&library_metadata_id) ne %str()) %then %do;
      libname moinput meta libid=&library_metadata_id;
   %end;
   %if (&syslibrc ne 0) %then %do;
      %put %mo_error Error assigning MOINPUT libref (syslibrc=&syslibrc);
      %let &_rc = 109;
      %goto ERROR_EXIT;
   %end;

   %* Set up validation message file;
   data _null_;
      length path $4096.;
      path=pathname('WORK');
      call symput('workpath',strip(path));
   run;

   %global message_file;
   %let message_file=&workpath.&separator.mo_messages.txt;

   %if (%sysfunc(fileexist("&message_file"))) %then %do;
      %mo_delete_file_item(item=&message_file,_rc=drc)
      %if (&drc ne 0) %then %do;
         %let &_rc = 100;
         %put %mo_error Error deleting mo_messages.txt.;
         %goto ERROR_EXIT;
      %end;
   %end;

   filename momsg "&message_file";
   %if (&sysfilrc ne 0 and &sysfilrc ne 1) %then %do;
      %put %mo_error Error assigning MOMSG filref (sysfilrc=&sysfilrc);
      %let &_rc = 125;
      %goto ERROR_EXIT;
   %end;

   %local setfile;
   %let setfile= file momsg lrecl=512 mod;


   %*----------------------------------------------------------;
   %* Check that table metadata matches table definition.      ;
   %* The following code is based on having previously defined ;
   %* the moinput libref using the libname statement with the  ;
   %* meta option (i.e., the metadata engine).                 ;
   %*----------------------------------------------------------;
   %let tablelist = campaign cust_col_metadata communication customer;
   %if (%bquote(&channel_table) ne %str()) %then %let tablelist = &tablelist channel;
   %if (%bquote(&time_period_table) ne %str()) %then %let tablelist = &tablelist time_period;
   %if (%bquote(&agent_capacity_table) ne %str()) %then %let tablelist = &tablelist agent_capacity;
   %if (%bquote(&constraints_table) ne %str()) %then %let tablelist = &tablelist constraints;
   %if (%bquote(&contact_policy_table) ne %str()) %then %let tablelist = &tablelist contact_policy;
   %if (&uses_contact_history eq 1) %then %do;
      %if (%bquote(&contact_history_table) ne %str()) %then %let tablelist = &tablelist contact_history;
      %if (%bquote(&communication_history_table) ne %str()) %then %let tablelist = &tablelist communication_history;
   %end;
   %do i=1 %to %mo_word_count(&tablelist);
      %let table=%scan(&tablelist,&i);
      %mo_no_notes(on)
      data _null_;
         set moinput.&&&table._table;
         if _N_ eq 1 then stop;
      run;
      %if (&syserr gt 4) %then %do;
         %mo_no_notes(off)
         %put %mo_error Table metadata check failed for table = &&&table._table (syserr=&syserr).;
         %put;
         data _null_;
            &setfile;
            putlog ' ';
            putlog "%mo_error An error occurred reading the %upcase(&&&table._table) table.  Possible causes of the error include -";
            putlog "%mo_error      1. The table metadata may not reflect the current table structure.";
            putlog "%mo_error      2. The ID used by the SAS session may not have sufficient access privileges to the physical table. The ID in use may have been either the actual user ID or the sassrv ID.";
            put    "input.data.validation.invalid.metadata.error.message.fmt.txt,1,%upcase(&&&table._table)";
            putlog ' ';
            call symput("&_rc",'8');
         run;
      %end;
      %else %put %mo_note Completed metadata check for table=&&&table._table (syserr=&syserr).;
   %end;
   %if (&&&_rc ge 8) %then %goto ERROR_EXIT;


   %*-------------------------------------------------------------------------;
   %* Check table/column attributes for all input tables, except the customer ;
   %* and contact history tables which have their own special checks.         ;
   %*-------------------------------------------------------------------------;
   %local anyFatalRequiredColumnProblem;
   %let anyFatalRequiredColumnProblem = 0;

   %mo_no_notes(on)
   %mo_create_tables(lib=work, table=INPUT, labels=Y, _rc=cudrc)
   %if (&cudrc ne 0) %then %do;
      %let &_rc = 100;
      %put %mo_error Error occurred creating INPUT tables in WORK library.;
      %goto ERROR_EXIT;
   %end;

   data work.communication_orig_mo; /* KEC: Why save this in this way? */
      set work.communication;
   run;   

   %mo_no_notes(off)
   %let tablelist = campaign cust_col_metadata communication;
   %if (%bquote(&channel_table) ne %str()) %then %let tablelist = &tablelist channel;
   %if (%bquote(&time_period_table) ne %str()) %then %let tablelist = &tablelist time_period;
   %if (%bquote(&agent_capacity_table) ne %str()) %then %let tablelist = &tablelist agent_capacity;
   %if (%bquote(&constraints_table) ne %str()) %then %let tablelist = &tablelist constraints;
   %if (%bquote(&contact_policy_table) ne %str()) %then %let tablelist = &tablelist contact_policy;
   %if (&uses_contact_history eq 1) %then %do;
      %if (%bquote(&contact_history_table) ne %str()) %then %let tablelist = &tablelist contact_history;
      %if (%bquote(&communication_history_table) ne %str()) %then %let tablelist = &tablelist communication_history;
   %end;

   %* Set flags for use below related to the treatment of -                  ;
   %* 1. PRIORITY column for the communication and communication history     ;
   %*    tables based on the uses_prioritization flag setting,               ;
   %* 2. UNIQUE_COMM_MAPPING_ID, CAMPAIGN_CD, and COMMUNICATION_CD           ;
   %*    columns for the communication history and contact history tables    ;
   %*    based on the uses_contact_history flag setting and on the           ;
   %*    moma_input flag setting (MOMA) or presence/absence of the           ;
   %*    UNIQUE_COMM_MAPPING_ID column (MOSA),                               ;
   %* 3. MANDATORY, EXCLUDE_FROM_CONSTRAINTS, and EXCLUDE_FROM_CONTACT_POLS  ;
   %*    columns for the communication table based on the presence/absence   ;
   %*    of the MANDATORY column.                                            ;

   %* Prioritization flags ;
   %local priority_col_required;
   %let priority_col_required = 0;
   %if (&uses_prioritization eq 1) %then %do;
      %let priority_col_required = 1;
   %end;

   %* Contact History flags ;
   %local hist_mapping_col_required hist_mapping_col_type;  
   %local hist_campcommcd_cols_required;
   %let hist_mapping_col_required = 0;
   %let hist_mapping_col_type = .;
   %let hist_campcommcd_cols_required = 0;
   %if (&uses_contact_history eq 1) %then %do;

      %local dsid hist_mapping_col_varnum hist_mapping_col_vartype;  
      %let hist_mapping_col_varnum = 0;
      %let hist_mapping_col_vartype = ;
      %let dsid = %sysfunc(open(moinput.&communication_history_table));
      %if (&dsid = 0) %then %do;
         data _null_;
            &setfile;
            putlog ' ';
            %putlog "%mo_error The specified &communication_history_table table does not exist in MOINPUT.";
            put "input.data.validation.table.not.exist.error.message.fmt.txt,1,&communication_history_table";            
            call symput("&_rc",'8');
         run;
         %let anyFatalRequiredColumnProblem = %eval(&anyFatalRequiredColumnProblem or 1);
      %end;
      %else %do;
         %let hist_mapping_col_varnum = %sysfunc(varnum(&dsid, unique_comm_mapping_id));
         %if (&hist_mapping_col_varnum ne 0) %then %do;
            %let hist_mapping_col_vartype = %sysfunc(vartype(&dsid, &hist_mapping_col_varnum));
         %end;
      %end;
      %let dsid = %sysfunc(close(&dsid));

      %if (&moma_input eq 1) %then %do;
         %let hist_mapping_col_required = 1;
      %end;
      %else %do;
         %if (&hist_mapping_col_varnum ne 0) %then %do;
            %let hist_mapping_col_required = 1;
         %end;
         %else %do;
            %let hist_campcommcd_cols_required = 1;
         %end; 
      %end;

      %if (&hist_mapping_col_required eq 1) %then %do;
         %if (%upcase(&hist_mapping_col_vartype) eq N) %then %do;
            %let hist_mapping_col_type = 1;
         %end;
         %else %if (%upcase(&hist_mapping_col_vartype) eq C) %then %do;
            %let hist_mapping_col_type = 2;
         %end;
         %else %do;
            %let hist_mapping_col_type = .;
         %end;
      %end;
   %end;

   %* Mandatory Inclusions flags ;
   %local mandatory_cols_required;
   %let mandatory_cols_required = 0;

   %local dsid mandatory_col_varnum;  
   %let mandatory_col_varnum = 0;
   %let dsid = %sysfunc(open(moinput.&communication_table));
   %if (&dsid = 0) %then %do;
      data _null_;
         &setfile;
         putlog ' ';
         %putlog "%mo_error The specified &communication_table table does not exist in MOINPUT.";
         put "input.data.validation.table.not.exist.error.message.fmt.txt,1,&communication_table";            
         call symput("&_rc",'8');
      run;
      %let anyFatalRequiredColumnProblem = %eval(&anyFatalRequiredColumnProblem or 1);
   %end;
   %else %do;
      %let mandatory_col_varnum = %sysfunc(varnum(&dsid, mandatory));
      %if (&mandatory_col_varnum ne 0) %then %do;
         %let mandatory_cols_required = 1;
      %end;
   %end;
   %let dsid = %sysfunc(close(&dsid));

   %do i=1 %to %mo_word_count(&tablelist);
      %let table=%scan(&tablelist,&i);
      %put;

      %* Gather actual input data table column info ;
      %* and comparison table required column info. ;
      proc contents data=moinput.&&&table._table out=actual_tbl_contents_&i (keep=name type length) noprint;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered obtaining contents of MOINPUT.&&&table._table (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;

      %* Adjust the required column names for those     ;
      %* tables for which enforcement is not done here. ;
      data work.actual_tbl_contents_&i;
         set work.actual_tbl_contents_&i;
         name=upcase(name);
         %if (&table eq agent_capacity) %then %do;
            if compbl(trim(left(name))) ne "%upcase(&agent_column)";
         %end;

         %if (&table eq contact_history) %then %do;
            if compbl(trim(left(name))) ne "%upcase(&customer_id_column)";
         %end;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered processing work.actual_tbl_contents_&i (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;

      proc sort data=actual_tbl_contents_&i;
      by name;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered sorting work.actual_tbl_contents_&i (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;

      proc contents data=work.&table out=comparison_tbl_contents_&i (keep=name type length label) noprint;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered obtaining contents of work.&table (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;

      %* Adjust the required column names for those tables for which  ;
      %* the requirement for certain columns is variable based on     ;
      %* other conditions, or for which enforcement is not done here. ;
      data work.comparison_tbl_contents_&i;
         set work.comparison_tbl_contents_&i end=last;
         name=upcase(name);

         %* Use subsetting IF to filter output to eliminate ;
         %* columns not actually required.                  ;
         %if (&table eq agent_capacity) %then %do;
            if compbl(trim(left(name))) ne "AGENT_ID"; /* chk elsewhere */
         %end;
         %if (&table eq contact_history) %then %do;
            if (compbl(trim(left(name))) ne "CUSTOMER_ID"); /* chk elsewhere */
         %end;

         %* Adjust requirements criteria for variable columns. ;
         %if (&table eq communication_history) or
             (&table eq contact_history) %then %do;
            if (compbl(trim(left(name))) eq "UNIQUE_COMM_MAPPING_ID") then do;
               %if (&hist_mapping_col_required eq 0) %then %do;
                  label = "OPTIONAL";
               %end;
               %else %do;
                  type = &hist_mapping_col_type;
                  if (type = 1) then do;
                     length = 8;
                  end;
               %end;
            end;
         %end;

         %if (&table eq contact_history) %then %do;
            %if (&hist_campcommcd_cols_required eq 1) %then %do;
               if (compbl(trim(left(name))) eq "CAMPAIGN_CD") or
                  (compbl(trim(left(name))) eq "COMMUNICATION_CD") then do;
                  label = "REQUIRED";
               end;
            %end;
         %end;

         %if (&table eq communication) or
             (&table eq communication_history) %then %do;
            %if (&mandatory_cols_required eq 1) %then %do;
               if (compbl(trim(left(name))) eq "MANDATORY") or
                  (compbl(trim(left(name))) eq "EXCLUDE_FROM_CONSTRAINTS") or
                  (compbl(trim(left(name))) eq "EXCLUDE_FROM_CONTACT_POLS") then do;
                  label = "REQUIRED";
               end;
            %end;

            %if (&priority_col_required eq 1) %then %do;
               if (last = 1) then do;
                  output;

                  name = upcase("&priority_column");
                  type = 1;
                  length = 8;
                  label = "REQUIRED";
               end;
            %end;
         %end;

         output;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered processing work.comparison_tbl_contents_&i (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;

      proc sort data=comparison_tbl_contents_&i;
      by name;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered sorting work.comparison_tbl_contents_&i (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;

      data compare_&i;
         length from $8.;
         merge work.actual_tbl_contents_&i (in=a rename=(type=type_new length=length_new))
               work.comparison_tbl_contents_&i (in=b rename=(type=type_old length=length_old label=status));
         by name;
         name=upcase(name);
         if (a and b) then from = 'both';
         else if (a) then from = 'new';
         else from='old';
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered creating work.compare_&i (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;

      %mo_no_notes(on);

      data _null_;
         set compare_&i;
         &setfile;

         if (from eq 'new') then do;
            putlog ' ';
            putlog "%mo_note Additional column detected in %upcase(&&&table._table) table: " name;
            putlog ' ';
         end;
         else if (from eq 'old') then do;
            if (status eq 'REQUIRED') then do;
               putlog ' ';
               putlog "%mo_error Missing column detected in %upcase(&&&table._table) table: " name;
               put    "input.data.validation.missing.column.error.message.fmt.txt,2,%upcase(&&&table._table)," name;
               putlog ' ';
               call symput("&_rc",'8');
            end;
         end;
         else do;
            if (type_new ne type_old) then do;
               putlog ' ';
               putlog "%mo_error Column is defined with the incorrect type in %upcase(&&&table._table) table: " name;
               put    "input.data.validation.invalid.column.type.error.message.fmt.txt,2,%upcase(&&&table._table)," name;
               putlog ' ';
               call symput("&_rc",'8');
            end;

            if (length_new lt length_old) then do;
               if (type_new eq 1) then do;
                  putlog ' ';
                  putlog "%mo_note Numeric column is defined with length shorter than 8. in %upcase(&&&table._table) table: " name;
                  putlog ' ';
               end;
               else if (type_new eq 2) then do;
                  putlog ' ';
                  putlog "%mo_note Character column is defined with length shorter than expected in %upcase(&&&table._table) table: " name;
                  putlog ' ';
               end;
            end;

            if ((length_new gt length_old) and (length_old ne 32)) then do;
               if (upcase(name) ne 'NUMERIC_MEASURE' or "&table" ne "cust_col_metadata") then do;
                  putlog ' ';
                  putlog "%mo_note Character column is defined with length greater than expected in %upcase(&&&table._table) table: " name ".  Truncation may result.";
               end;
               else do;
                  putlog ' ';
                  putlog "%mo_note Character column is defined with length greater than expected in %upcase(&&&table._table) table: " name ".";
               end;
               putlog ' ';
            end;

            if ((("&table" = "communication") or 
                 ("&table" = "communication_history")) 
               and
                ((upcase(name) = 'EXCLUDE_FROM_CONSTRAINTS') or 
                 (upcase(name) = 'EXCLUDE_FROM_CONTACT_POLS')) 
               and
                 (&mandatory_cols_required = 0)) then do;
               if (upcase(name) = 'EXCLUDE_FROM_CONSTRAINTS') then do; 
                  putlog ' ';
                  putlog "%mo_error Invalid column detected in %upcase(&&&table._table) table: EXCLUDE_FROM_CONSTRAINTS. It is not valid because the MANDATORY column is not present.";
                  put "input.data.validation.invalid.column.error.message.fmt.txt,3,EXCLUDE_FROM_CONSTRAINTS,%upcase(&&&table._table),MANDATORY";
               end;
               else do; /* upcase(name) = EXCLUDE_FROM_CONTACT_POLS */
                  putlog ' ';
                  putlog "%mo_error Invalid column detected in %upcase(&&&table._table) table: EXCLUDE_FROM_CONTACT_POLS. It is not valid because the MANDATORY column is not present.";
                  put "input.data.validation.invalid.column.error.message.fmt.txt,3,EXCLUDE_FROM_CONTACT_POLS,%upcase(&&&table._table),MANDATORY";
               end;
               putlog ' ';
               call symput("&_rc",'8');
            end;
         end;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered processing work.compare_&i (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;

      %* Capture the fact that there was a problem with a required column ;
      %* but do not stop or exit until all tables have been processed.    ;
      %if (&&&_rc ge 8) %then %do;
         %let anyFatalRequiredColumnProblem = %eval(&anyFatalRequiredColumnProblem or 1);
      %end;
      
      %mo_no_notes(off);
   %end;

   %*;
   %* Discontinue validation processing if there were one or more  ;
   %* problems found with a required column in one or more tables. ;
   %*;
   %if (&anyFatalRequiredColumnProblem eq 1) %then %do;
      %goto ERROR_EXIT;
   %end;

   %*----------------------------------------------------------;
   %* Check that customer_id_column exists in customer table   ;
   %* and, if contact history is being used, that it also      ;
   %* exists in the contact history table.                     ;
   %*                                                          ;
   %* If a multi-part key is specified, check for all columns. ;
   %*----------------------------------------------------------;

   %*;
   %* Given the possibility of a multi-part key within customer_id_column,    ;
   %* issue a warning if the length of the de-referenced value of             ;
   %* &customer_id_column is greater than 256, since it will be truncated to  ;
   %* that length when stored in the datastore input_data table.              ;
   %* Set up useful local macro variables for use throughout this macro to be ;
   %* used in dealing with the multiple column names (multiple keys) embedded ;
   %* within the de-referenced value of &customer_id_column.                  ;
   %*;
   %local cust_id_col_length;
   %let cust_id_col_length = %length(&customer_id_column);
   %if (&cust_id_col_length gt 256) %then %do;
      data _null_;
         &setfile;
         putlog ' ';
         putlog "%mo_error Length of CUSTOMER_ID_COLUMN_NAME specified for %upcase(&customer_table) table is &cust_id_col_length, but cannot have length greater than 256.";
         put "input.data.validation.long.customer_id.column.error.message.fmt.txt,2,%upcase(&customer_table,&cust_id_col_length)";
         putlog ' ';
         call symput("&_rc",'8');
       run;
   %end;

   %local numCustIdCols custIdColNum;
   %local custIdColNameListComma;
   %local custIdColNameListNoComma;
   %local custIdColNameListQuote;

   %let custIdColNameListComma =;
   %let custIdColNameListNoComma =;
   %let custIdColNameListQuote =;
   %let numCustIdCols = %mo_word_count(&customer_id_column, %str(*));

   %do custIdColNum = 1 %to &numCustIdCols;
      %local custIdCol&custIdColNum;
      %let custIdCol&custIdColNum = %scan(&customer_id_column, &custIdColNum, *);

      %let custIdColNameListComma = &custIdColNameListComma, &&custIdCol&custIdColNum;
      %let custIdColNameListNoComma = &custIdColNameListNoComma &&custIdCol&custIdColNum;
      %let custIdColNameListQuote = %nrbquote(&custIdColNameListQuote)%str(",")&&custIdCol&custIdColNum;
   %end;
   %let custIdColNameListComma = %substr(%nrbquote(&custIdColNameListComma), 3);
   %let custIdColNameListQuote = %nrbquote(%substr(%nrbquote(&custIdColNameListQuote), 4));

   proc contents data=moinput.&customer_table out=work.customer_contents (keep=name type length) noprint;
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered obtaining contents of moinput.&customer_table (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;

   data work.customer_contents_custIDCols; /* For use later */
      set work.customer_contents(where=(upcase(name) in ("%upcase(%nrbquote(&custIdColNameListQuote))")));
      name = upcase(name);
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered processing work.customer_contents_custIDCols (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;

   proc sort data=work.customer_contents_custIDCols;
      by name type;
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered sorting work.customer_contents_custIDCols (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;

   %let cust_id_check_tablelist = customer;

   %if (&uses_contact_history eq 1) %then %do;
      proc contents data=moinput.&contact_history_table out=work.contact_history_contents (keep=name type length) noprint;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered obtaining contents of moinput.&contact_history_table (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;

      data work.history_contents_custIDCols; /* For use later */
         set work.contact_history_contents(where=(upcase(name) in ("%upcase(%nrbquote(&custIdColNameListQuote))")));
         name = upcase(name);
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered processing work.history_contents_custIDCols (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;

      proc sort data=history_contents_custIDCols;
         by name type;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered sorting work.history_contents_custIDCols (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;

      %let cust_id_check_tablelist = &cust_id_check_tablelist contact_history;
   %end;

   %do i=1 %to %mo_word_count(&cust_id_check_tablelist);
      %let table=%scan(&cust_id_check_tablelist,&i);
      %put;

      %do custIdColNum = 1 %to &numCustIdCols;
         proc sql noprint;
            select *
            from work.&table._contents
            where (upcase(name) eq "%upcase(&&custIdCol&custIdColNum)");
         quit;
         %if (&sqlobs eq 0) %then %do;
            data _null_;
               &setfile;
               putlog ' ';
               putlog "%mo_error Customer ID column = &&custIdCol&custIdColNum (key &custIdColNum from &customer_id_column) not found in %upcase(&table) table.";
               put "input.data.validation.missing.customer_id.column.error.message.fmt.txt,2,&custIdColNum,%upcase(&&&table._table)";
               putlog '';
               call symput("&_rc",'8');
            run;
            %goto ERROR_EXIT;
         %end;
      %end;
   %end;


   %*-------------------------------------------------;
   %* Check that agent_column is not called agent_id. ;
   %*-------------------------------------------------;
   %if (%bquote(&agent_column) ne %str() and %bquote(&agent_capacity_table) ne %str()) %then %do;
      %if (%upcase(%bquote(&agent_column)) eq AGENT_ID) %then %do;
         data _null_;
            &setfile;
            putlog ' ';
            putlog "%mo_error The column named AGENT_ID in the %upcase(&agent_capacity_table) table is not allowed. It is a SAS Marketing Optimization reserved name.";
            put "input.data.validation.reserved.column.error.message.fmt.txt,2,AGENT_ID,%upcase(&agent_capacity_table)";            
            call symput("&_rc",'8');
         run;
         %goto ERROR_EXIT;
      %end;
   %end;


   %*----------------------------------------------------------------;
   %* Check that agent_column exists in the Customer table.          ;
   %*----------------------------------------------------------------;
   %if (%bquote(&agent_column) ne %str() and %bquote(&agent_capacity_table) ne %str()) %then %do;
      proc sql noprint;
         select *
         from work.customer_contents
         where (upcase(name) eq "%upcase(&agent_column)");
      quit;
      %if (&sqlobs eq 0) %then %do;
         data _null_;
            &setfile;
            putlog ' ';
            putlog "%mo_error Customer agent column = &agent_column not found in %upcase(&customer_table).";
            put "input.data.validation.missing.agent.column.error.message.fmt.txt,2,%upcase(&agent_column),%upcase(&customer_table)";
            call symput("&_rc",'8');
         run;
         %goto ERROR_EXIT;
      %end;
   %end;


   %*------------------------------------------------------------------;
   %* Check that the household_id_column exists in the Customer table. ;
   %*------------------------------------------------------------------;
   %local household_flag;
   %let household_flag = 0;
   %if (%bquote(&household_id_column) ne %str()) %then %do;
      proc sql noprint;
         select *
         from work.customer_contents
         where (upcase(name) eq "%upcase(&household_id_column)");
      quit;
      %if (&sqlobs eq 0) %then %do;
         data _null_;
            &setfile;
            putlog ' ';
            putlog "%mo_error Household ID column = &household_id_column not found in %upcase(&customer_table).";
            put "input.data.validation.missing.household.column.error.message.fmt.txt,2,%upcase(&household_id_column),%upcase(&customer_table)";
            call symput("&_rc",'8');
         run;
         %goto ERROR_EXIT;
      %end;
      %else %do;
         %let household_flag = 1;
      %end;
   %end;


   %*---------------------------------------------------------------;
   %* Check that some numeric columns exists in the Customer table. ;
   %*---------------------------------------------------------------;
   proc sql noprint;
      select *
      from work.customer_contents
      where (upcase(name) not in ("%upcase(%nrbquote(&custIdColNameListQuote))")
         %if (%bquote(&agent_column) ne %str() and %bquote(&agent_capacity_table) ne %str()) %then %do;
            and upcase(name) ne "%upcase(&agent_column)"
         %end;
         %if (%bquote(&household_id_column) ne %str()) %then %do;
            and upcase(name) ne "%upcase(&household_id_column)"
         %end;
         and type eq 1);
   quit;
   %if (&sqlobs eq 0) %then %do;
      data _null_;
         &setfile;
         putlog ' ';
         putlog "%mo_error No numeric columns found in %upcase(&customer_table).";
         put "input.data.validation.missing.numeric.column.error.message.fmt.txt,1,%upcase(&customer_table)";
         call symput("&_rc",'8');
      run;
      %goto ERROR_EXIT;
   %end;


   %*--------------------------------------------------------------------;
   %* Create views of input tables that fill in missing optional columns.;
   %*--------------------------------------------------------------------;
   %local use_alt_type_list;
   %let use_alt_type_list = ;
   %if ((&uses_contact_history eq 1) and 
        (&hist_mapping_col_type eq 1)) %then %do;
      %let use_alt_type_list = unique_comm_mapping_id;
   %end;
   
   %mo_create_tables(lib=work, table=INPUT, labels=N, alt_types=&use_alt_type_list _rc=cudrc)
   %if (&cudrc ne 0) %then %do;
      %let &_rc = 100;
      %put %mo_error Error occurred creating INPUT tables in WORK library.;
      %goto ERROR_EXIT;
   %end;

   %do i=1 %to %mo_word_count(&tablelist);
      %let table=%scan(&tablelist,&i);

      %mo_no_notes(off)
      proc datasets library=work nolist nowarn;
         delete &table._in_mo_tbl / memtype=all;
      quit;

      %mo_no_notes(on)
      data work.&table._in_mo_tbl;
         merge moinput.&&&table._table 
               work.&table
                  %if (&table eq contact_history) %then %do;
                     (drop=customer_id)
                  %end;
         ;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered creating work.&table._in_mo_tbl (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;
   %end;
   %mo_no_notes(off)


   %*-----------------------------------;
   %* Check for tables with no records  ;
   %*-----------------------------------;
   %do i=1 %to %mo_word_count(&tablelist);
      %let table=%scan(&tablelist,&i);

      %mo_no_notes(off)
      %if (not %mo_obs_in(work.&table._in_mo_tbl)) %then %do;
         %if (&table eq agent_capacity) or 
             (&table eq time_period) or 
             (&table eq channel) or 
             (&table eq constraints) or 
             (&table eq contact_policy) %then %do;
            data _null_;
               &setfile;
               putlog ' ';
               putlog "%mo_warning The %upcase(&&&table._table) table contains no data.";
               put    "input.data.validation.empty.table.warning.message.fmt.txt,1,%upcase(&&&table._table)";
               putlog ' ';
               call symput("&_rc",put(max(&&&_rc,4),1.));
            run;
         %end;
         %else %if ((&table ne contact_history) and
                    (&table ne communication_history)) %then %do;
            data _null_;
               &setfile;
               putlog ' ';
               putlog "%mo_error The %upcase(&&&table._table) table contains no data.  This data is required to create an input data definition.";
               put    "input.data.validation.empty.table.error.message.fmt.txt,1,%upcase(&&&table._table)";
               putlog ' ';
               call symput("&_rc",'8');
            run;
         %end;
      %end;
   %end;

   %if (&uses_contact_history eq 1) %then %do;
      %if ((not %mo_obs_in(work.communication_history_in_mo_tbl)) and
           (not %mo_obs_in(work.contact_history_in_mo_tbl))) %then %do;
         data _null_;
            &setfile;
            putlog ' ';
            putlog "%mo_warning The %upcase(&communication_history_table) table contains no data.";
            putlog "%mo_warning The %upcase(&contact_history_table) table contains no data.";
            put    "input.data.validation.empty.table.warning.message.fmt.txt,1,%upcase(&communication_history_table)";
            put    "input.data.validation.empty.table.warning.message.fmt.txt,1,%upcase(&contact_history_table)";
            putlog ' ';
            call symput("&_rc",put(max(&&&_rc,4),1.));
         run;
      %end;
      %else %if ((%mo_obs_in(work.communication_history_in_mo_tbl)) and
                 (not %mo_obs_in(work.contact_history_in_mo_tbl))) %then %do;
         data _null_;
            &setfile;
            putlog ' ';
            putlog "%mo_warning The %upcase(&contact_history_table) table contains no data.";
            put    "input.data.validation.empty.table.warning.message.fmt.txt,1,%upcase(&contact_history_table)";
            putlog ' ';
            call symput("&_rc",'4');
         run;
      %end;
      %else %if ((not %mo_obs_in(work.communication_history_in_mo_tbl)) and
                 (%mo_obs_in(work.contact_history_in_mo_tbl))) %then %do;
         data _null_;
            &setfile;
            putlog ' ';
            putlog "%mo_error The %upcase(&communication_history_table) table contains no data.  This data is required in order to use contact history in scenario optimizations that use this input data.";
            put    "input.data.validation.empty.table.contactHist.error.message.fmt.txt,1,%upcase(&communication_history_table)";
            putlog ' ';
            call symput("&_rc",'8');
         run;
      %end;
   %end;   

   %if (&&&_rc ge 8) %then %goto ERROR_EXIT;


   %*---------------------------------------------;
   %* Check that the customer table is not empty  ;
   %*---------------------------------------------;

   %if (not %mo_obs_in(moinput.&customer_table)) %then %do;
      data _null_;
         &setfile;
         putlog ' ';
         putlog "%mo_error The customer table &customer_table contains no data. This data is required to create an input data definition.";
         put    "input.data.validation.empty.table.error.message.fmt.txt,1,%upcase(&customer_table)";
         putlog  ' ';
         call symput("&_rc",'8');
      run;
   %end;
   %if (&&&_rc ge 8) %then %goto ERROR_EXIT;


   %*--------------------------------;
   %* Check CAMPAIGNS table data     ;
   %*--------------------------------;
   %CAMPAIGNS:

   %*;
   %* Do basic CAMPAIGN data validation;
   %*;
   %local num_distinct_campaign_cds;
   %let num_distinct_campaign_cds = 0; /* Set later if table is in input data */

   data work.&campaign_table;
      set work.campaign_in_mo_tbl;
      &setfile;

      campaign_cd = trim(left(campaign_cd));
      if (campaign_cd eq '') then do;
         putlog ' ';
         putlog "%mo_error Blank value detected for CAMPAIGN_CD in observation " _N_ +(-1) " of %upcase(&campaign_table) table.";
         put    "input.data.validation.blank.value.error.message.fmt.txt,3,CAMPAIGN_CD," _N_ +(-1)",%upcase(&campaign_table)";
         putlog ' ';
         call symput("&_rc",'8');
      end;
      else do;
         if (substr(campaign_cd,1,1) eq ' ') then do;
            putlog ' ';
            putlog "%mo_error CAMPAIGN_CD in observation " _N_ +(-1) " of %upcase(&campaign_table) table must not have leading blanks.";
            put    "input.data.validation.leading.space.error.message.fmt.txt,3,CAMPAIGN_CD," _N_ +(-1)",%upcase(&campaign_table)";
            putlog ' ';
            call symput("&_rc",'8');
         end;
         if (anyspace(substr(campaign_cd,1,length(campaign_cd))) gt 0) then do;
            putlog ' ';
            putlog "%mo_error CAMPAIGN_CD in observation " _N_ +(-1) " of %upcase(&campaign_table) table must not have embedded spaces.";
            put    "input.data.validation.embedded.space.error.message.fmt.txt,3,CAMPAIGN_CD," _N_ +(-1)",%upcase(&campaign_table)";
            putlog ' ';
            call symput("&_rc",'8');
         end;
         if (index(campaign_cd,',') gt 0) then do;
            putlog ' ';
            putlog "%mo_error CAMPAIGN_CD in observation " _N_ +(-1) " of %upcase(&campaign_table) table must not contain commas.";
            put    "input.data.validation.special.char.name.error.message.fmt.txt,3,CAMPAIGN_CD," _N_ +(-1)",%upcase(&campaign_table)";
            putlog ' ';
            call symput("&_rc",'8');
         end;
         if (length(campaign_cd) > 32) then do;
            putlog ' ';
            putlog "%mo_error Length of CAMPAIGN_CD in observation " _N_ +(-1) " of %upcase(&campaign_table) table cannot be greater than 32.";
            put    "input.data.validation.column.length.error.message.fmt.txt,3,CAMPAIGN_CD," _N_ +(-1)",%upcase(&campaign_table)";
            putlog ' ';
            call symput("&_rc",'8');
         end;
      end;
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered creating work.&campaign_table (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;

   %*;
   %* Check for duplicate CAMPAIGN_CDs;
   %*;
   proc sort data=work.&campaign_table;
      by campaign_cd;
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered sorting work.&campaign_table (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;

   data _null_;
      set work.&campaign_table;
      &setfile;
      retain prev_camp_code;
      if (_N_ gt 1) then do;
         if (campaign_cd eq prev_camp_code) then do;
            putlog ' ';
            putlog "%mo_error Duplicate CAMPAIGN_CD detected in %upcase(&campaign_table) table where CAMPAIGN_CD = " campaign_cd ".";
            put    "input.data.validation.duplicate.value.error.message.fmt.txt,4,CAMPAIGN_CD,%upcase(&campaign_table),CAMPAIGN_CD," campaign_cd;
            putlog ' ';
            call symput("&_rc",'8');
         end;
      end;
      prev_camp_code = campaign_cd;
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered processing work.&campaign_table (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;

   %*;
   %* Build table containing the set of distinct CAMPAIGN_CDs;
   %*;
   proc sql noprint;
      create table work.distinct_campaign_cds as
      select distinct(compbl(trim(left(campaign_cd)))) as campaign_cd
      from work.campaign_in_mo_tbl;

      select count(*) into :num_distinct_campaign_cds
      from work.distinct_campaign_cds;
   quit;
   %if (&sqlrc gt 4) %then %do;
      %let &_rc = 107;
      %put %mo_error SQL error encountered obtaining distinct campaign_cds (sqlrc=&sqlrc).;
      %goto ERROR_EXIT;
   %end;


   %*--------------------------------;
   %* Check CHANNELS table data      ;
   %*--------------------------------;
   %put;
   %*;
   %* Do basic CHANNEL data validation;
   %*;
   %local num_distinct_channel_cds;
   %let num_distinct_channel_cds = 0; /* Set later if table is in input data */

   %if (%bquote(&channel_table) ne %str()) %then %do;
      data work.&channel_table;
         set work.channel_in_mo_tbl;
         &setfile;

         channel_cd=compbl(trim(left(channel_cd)));
         uom_cd=compbl(trim(left(uom_cd)));

         if (channel_cd eq '') then do;
            putlog ' ';
            putlog "%mo_error Blank value detected for CHANNEL_CD in observation " _N_ +(-1) " of %upcase(&channel_table) table.";
            put    "input.data.validation.blank.value.error.message.fmt.txt,3,CHANNEL_CD," _N_ +(-1)",%upcase(&channel_table)";
            putlog ' ';
            call symput("&_rc",'8');
         end;
         if (length(channel_cd) > 32) then do;
            putlog ' ';
            putlog "%mo_error Length of CHANNEL_CD in observation " _N_ +(-1) " of %upcase(&channel_table) table cannot be greater than 32.";
            put    "input.data.validation.column.length.error.message.fmt.txt,3,CHANNEL_CD," _N_ +(-1)",%upcase(&channel_table)";
            putlog ' ';
            call symput("&_rc",'8');
         end;
         if (length(uom_cd) > 32) then do;
            putlog ' ';
            putlog "%mo_error Length of UOM_CD in observation " _N_ +(-1) " of %upcase(&channel_table) table cannot be greater than 32.";
            put    "input.data.validation.column.length.error.message.fmt.txt,3,UOM_CD," _N_ +(-1)",%upcase(&channel_table)";
            putlog ' ';
            call symput("&_rc",'8');
         end;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered creating work.&channel_table (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;

      %*;
      %* Check for duplicate CHANNEL_CDs;
      %*;
      proc sort data=work.&channel_table;
         by channel_cd;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered sorting work.&channel_table (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;

      data _null_;
         set work.&channel_table;
         &setfile;
         retain prev_channel;
         if (_N_ gt 1) then do;
            if (channel_cd eq prev_channel) then do;
               putlog ' ';
               putlog "%mo_error Duplicate CHANNEL_CD detected in %upcase(&channel_table) table where CHANNEL_CD = " channel_cd ".";
               put    "input.data.validation.duplicate.value.error.message.fmt.txt,4,CHANNEL_CD,%upcase(&channel_table),CHANNEL_CD," channel_cd;
               putlog ' ';
               call symput("&_rc",'8');
            end;
         end;
         prev_channel = channel_cd;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered processing work.&channel_table (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;

      %*;
      %* Build table containing the set of distinct CHANNEL_CDs;
      %*;
      proc sql noprint;
         create table work.distinct_channel_cds as
         select distinct(compbl(trim(left(channel_cd)))) as channel_cd
         from work.channel_in_mo_tbl;

         select count(*) into :num_distinct_channel_cds
         from work.distinct_channel_cds;
      quit;
      %if (&sqlrc gt 4) %then %do;
         %let &_rc = 107;
         %put %mo_error SQL error encountered obtaining distinct channel_cds (sqlrc=&sqlrc).;
         %goto ERROR_EXIT;
      %end;
   %end;


   %*--------------------------------;
   %* Check TIME_PERIOD table data   ;
   %*--------------------------------;
   %TIMEPERIOD:

   %*;
   %* Do basic TIME_PERIOD data validation;
   %*;
   %local num_distinct_time_periods;
   %let num_distinct_time_periods = 0; /* Set later if table is in input data */

   %if (%bquote(&time_period_table) ne %str()) %then %do;
      %put;
      data work.&time_period_table;
         set work.time_period_in_mo_tbl;
         &setfile;

         time_period=compbl(trim(left(time_period)));

         if (time_period eq '') then do;
            putlog ' ';
            putlog "%mo_error Blank value detected for TIME_PERIOD in observation " _N_ +(-1) " of %upcase(&time_period_table) table.";
            put    "input.data.validation.blank.value.error.message.fmt.txt,3,TIME_PERIOD," _N_ +(-1)",%upcase(&time_period_table)";
            putlog ' ';
            call symput("&_rc",'8');
         end;
         if (order_no eq .) then do;
            putlog ' ';
            putlog "%mo_error Missing value detected for ORDER_NO in observation " _N_ +(-1) " of %upcase(&time_period_table) table.";
            put    "input.data.validation.missing.value.error.message.fmt.txt,3,ORDER_NO," _N_ +(-1)",%upcase(&time_period_table)";
            putlog ' ';
            call symput("&_rc",'8');
         end;
         if (length(time_period) > 32) then do;
            putlog ' ';
            putlog "%mo_error Length of TIME_PERIOD in observation " _N_ +(-1) " of %upcase(&time_period_table) table cannot be greater than 32.";
            put    "input.data.validation.column.length.error.message.fmt.txt,3,TIME_PERIOD," _N_ +(-1)",%upcase(&time_period_table)";
            putlog ' ';
            call symput("&_rc",'8');
         end;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered creating work.&time_period_table (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;

      %*;
      %* Check for duplicate TIME_PERIODs;
      %*;
      proc sort data=work.&time_period_table;
         by time_period;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered sorting work.&time_period_table (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;

      data _null_;
         set work.&time_period_table;
         &setfile;
         retain prev_time_period;
         if (_N_ gt 1) then do;
            if (time_period eq prev_time_period) then do;
               putlog ' ';
               putlog "%mo_error Duplicate TIME_PERIOD detected in %upcase(&time_period_table) table where TIME_PERIOD = " time_period ".";
               put    "input.data.validation.duplicate.value.error.message.fmt.txt,4,TIME_PERIOD,%upcase(&time_period_table),TIME_PERIOD," time_period;
               putlog ' ';
               call symput("&_rc",'8');
           end;
         end;
         prev_time_period = time_period;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered processing work.&time_period_table (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;

      %*;
      %* Check for duplicate ORDER_NOs;
      %*;
      proc sort data=work.&time_period_table;
         by order_no;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered sorting work.&time_period_table (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;

      data _null_;
         set work.&time_period_table;
         &setfile;
         retain prev_order;
         if (_N_ gt 1) then do;
            if (order_no eq prev_order) then do;
               putlog ' ';
               putlog "%mo_error Duplicate ORDER_NO detected in %upcase(&time_period_table) table where ORDER_NO = " order_no ".";
               put    "input.data.validation.duplicate.value.error.message.fmt.txt,4,ORDER_NO,%upcase(&time_period_table),ORDER_NO," order_no;
               putlog ' ';
               call symput("&_rc",'8');
           end;
         end;
         prev_order = order_no;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered processing work.&time_period_table (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;

      %*;
      %* Build list of distinct TIME_PERIODs;
      %*;
      proc sql noprint;
         create table work.distinct_time_periods as
         select distinct(compbl(trim(left(time_period)))) as time_period
         from work.time_period_in_mo_tbl;

         select count(*) into :num_distinct_time_periods
         from work.distinct_time_periods;
      quit;
      %if (&sqlrc gt 4) %then %do;
         %let &_rc = 107;
         %put %mo_error SQL error encountered obtaining distinct time_periods (sqlrc=&sqlrc).;
         %goto ERROR_EXIT;
      %end;
   %end;

   %*;
   %* If contact history is being used, check that a non-empty Time  ;
   %* Periods table was specified.                                   ;
   %*;
   %if (&uses_contact_history eq 1) %then %do;
      %if ((%bquote(&time_period_table) eq %str()) or 
           (&num_distinct_time_periods eq 0)) %then %do;
         data _null_;
            &setfile;
            putlog ' ';
            putlog "%mo_error Input data that uses Contact History requires a Time Period table containing time period values.";
            put    "input.data.validation.contactHist.timePds.not.defined.error.message.fmt.txt,0";
            putlog ' ';
            call symput("&_rc",'8');
         run;
      %end;
   %end;


   %*-----------------------------------------------------;
   %* Check COMMUNICATIONS table data and, if applicable, ;
   %* COMMUNICATIONS HISTORY table data.                  ;
   %*-----------------------------------------------------;
   %COMM:

   %*;
   %* Helper macro mo_validate_comm_tbl ;
   %*;
   %macro mo_validate_comm_tbl(input_libname =
                              ,comm_tbl_name =
                              ,comm_tbl_in_mo =
                              ,mandatory_cols_rqd_flg =
                              ,priority_col_rqd_flg =
                              ,priority_col =
                              ,this_is_hist_tbl_flg =
                              ,hist_mapping_col_rqd_flg =
                              ,hist_mapping_col_type =
                              ,distinct_camp_cd_tbl =
                              ,num_distinct_camp_cds =
                              ,chan_tbl_name =
                              ,distinct_chan_cd_tbl =
                              ,num_distinct_chan_cds =
                              ,time_pd_tbl_name =
                              ,distinct_time_pd_tbl =
                              ,num_distinct_time_pds =
                              ,cust_tbl_name = 
                              ,agnt_tbl_name =
                              ,agnt_col =
                              ,_comm_tbl_work =
                              ,_distinct_comm_cd_tbl =
                              ,_num_distinct_comm_cds =
                              ,_distinct_cell_cd_tbl =
                              ,_num_distinct_cell_cds =
                              ,_cell_cd_stat =
                              ,_time_pd_stat =
                              ,_setfile = &setfile
                              ,_rc =
                              );

      %local macname;
      %let macname=&sysmacroname;
      %let macname=%lowcase(&macname);

      %*;
      %* If TIME_PERIOD values from the Time Period table exist, issue a       ;
      %* warning if no TIME_PERIOD values exist in the Communication table.    ;
      %* Also, set flag indicating Time Periods are in use.                    ;
      %*;
      %local num_comm_time_periods;
      %let num_comm_time_periods = 0;
      proc sql noprint;
         create table work.comm_time_periods as
         select distinct(compbl(trim(left(time_period)))) as time_period 
         from &comm_tbl_in_mo
         where (time_period ~= "");

         select count(*) into :num_comm_time_periods
         from work.comm_time_periods;
      quit;
      %if (&sqlrc gt 4) %then %do;
         %let &_rc = 107;
         %put %mo_error SQL error encountered retrieving time_periods from &comm_tbl_in_mo (sqlrc=&sqlrc).;
         %goto VAL_COMM_ERROR_EXIT;
      %end;

      %if ((&num_comm_time_periods eq 0) and 
           (&num_distinct_time_pds ne 0)) %then %do;
         data _null_;
            &_setfile;
            putlog ' ';
            putlog "%mo_warning Time periods are specified in the %upcase(&time_pd_tbl_name) table but none are specified in the %upcase(&comm_tbl_name) table.";
            put    "input.data.validation.missing.time_period.warning.message.fmt.txt,2,%upcase(&time_pd_tbl_name),%upcase(&comm_tbl_name)";
            putlog ' ';
            call symput("&_rc",put(max(&&&_rc,4),1.));
         run;
      %end;

      %if (&num_comm_time_periods eq 0) %then %do;
         %let &_time_pd_stat = 0;
      %end;
      %else %do;
         %let &_time_pd_stat = 1;
      %end;

      %*;
      %* If channel_cd values from the Channel table exist, then issue a       ;
      %* warning if no channel_cd values exist in the Communication table.     ;
      %*;
      %local num_comm_channel_cds;
      %let num_comm_channel_cds = 0;
      proc sql noprint;
         create table work.comm_channel_cds as
         select distinct(compbl(trim(left(channel_cd)))) as channel_cd 
         from &comm_tbl_in_mo
         where (channel_cd ~= "");

         select count(*) into :num_comm_channel_cds
         from work.comm_channel_cds;
      quit;
      %if (&sqlrc gt 4) %then %do;
         %let &_rc = 107;
         %put %mo_error SQL error encountered retrieving channel_cds from the &comm_tbl_in_mo (sqlrc=&sqlrc).;
         %goto VAL_COMM_ERROR_EXIT;
      %end;

      %if ((&num_comm_channel_cds eq 0) and
           (&num_distinct_chan_cds ne 0)) %then %do;
         data _null_;
            &_setfile;
            putlog ' ';
            putlog "%mo_warning Channel codes are specified in the %upcase(&chan_tbl_name) table but none are specified in the %upcase(&comm_tbl_name) table.";
            put    "input.data.validation.missing.channel_cd.communication.warning.message.fmt.txt,2,%upcase(&chan_tbl_name),%upcase(&comm_tbl_name)";
            putlog ' ';
            call symput("&_rc",put(max(&&&_rc,4),1.));
         run;
      %end;

      %*;
      %* Determine if the optional CELL_CD column is present in the actual     ;
      %* input data Communication table (not its corresponding In-MO table).   ;
      %*;
      %local cell_cd_in_comm_table;
      %let cell_cd_in_comm_table = 0;

      %local dsid cell_cd_col_varnum;  
      %let cell_cd_col_varnum = 0;
      %let dsid = %sysfunc(open(&input_libname..&comm_tbl_name));
      %if (&dsid = 0) %then %do;
         data _null_;
            &setfile;
            putlog ' ';
            %putlog "%mo_error The specified &comm_tbl_name table does not exist in &input_libname.";
            put "input.data.validation.table.not.exist.error.message.fmt.txt,1,&comm_tbl_name";            
            call symput("&_rc",'8');
         run;
         %goto ERROR_EXIT;
      %end;
      %let cell_cd_col_varnum = %sysfunc(varnum(&dsid, cell_cd));
      %if (&cell_cd_col_varnum ne 0) %then %do;
         %let cell_cd_in_comm_table = 1;
      %end;
      %let dsid = %sysfunc(close(&dsid));

      %*;
      %* Do basic Communication table data validation.             ;
      %*                                                           ;
      %* Note that checking that campaign_cd and channel_cd values ;
      %* appear in the Campaign and regular Communication table is ;
      %* not a relevant check for the Communication History table. ;
      %*;

      data &comm_tbl_in_mo;
         set &comm_tbl_in_mo;
         nomatch_channel_cd = 0;
         nomatch_campaign_cd = 0;
         nomatch_time_period = 0;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered creating nomatch columns in &comm_tbl_in_mo (syserr=&syserr).;
         %goto VAL_COMM_ERROR_EXIT;
      %end;

      proc sql noprint;
         %if (&this_is_hist_tbl_flg eq 0) %then %do; 
            %if (%bquote(&chan_tbl_name) ne %str()) %then %do; 
               update &comm_tbl_in_mo
                  set nomatch_channel_cd = 1
                  where channel_cd not in 
                     (select channel_cd from &distinct_chan_cd_tbl);
            %end;

            update &comm_tbl_in_mo
               set nomatch_campaign_cd = 1
               where campaign_cd not in 
                  (select campaign_cd from &distinct_camp_cd_tbl);
         %end;

         %if (%bquote(&time_pd_tbl_name) ne %str()) %then %do;
            update &comm_tbl_in_mo
               set nomatch_time_period = 1
               where time_period not in 
                  (select time_period from &distinct_time_pd_tbl);
         %end;
      quit;
      %if (&sqlrc gt 4) %then %do;
         %let &_rc = 107;
         %put %mo_error SQL error encountered updating nomatch columns in &comm_tbl_in_mo (sqlrc=&sqlrc).;
         %goto VAL_COMM_ERROR_EXIT;
      %end;

      data &_comm_tbl_work (drop=nomatch_channel_cd nomatch_campaign_cd nomatch_time_period);
         set &comm_tbl_in_mo;
         &_setfile;

         time_period=compbl(trim(left(time_period)));
         channel_cd=compbl(trim(left(channel_cd)));
         campaign_cd=trim(left(campaign_cd));
         communication_cd=trim(left(communication_cd));
         %if (&cell_cd_in_comm_table eq 1) %then %do;
            cell_cd=trim(left(cell_cd));
         %end;
         %if ((&this_is_hist_tbl_flg eq 1) and 
              (&hist_mapping_col_rqd_flg eq 1)) %then %do;
            %if (&hist_mapping_col_type eq 2) %then %do;
               unique_comm_mapping_id=trim(left(unique_comm_mapping_id));
            %end;
         %end;

         if (mandatory = .) then do;
            mandatory = 0;
         end;
         if (exclude_from_constraints = .) then do;
            exclude_from_constraints = 0;
         end;
         if (exclude_from_contact_pols = .) then do;
            exclude_from_contact_pols = 0;
         end;

         _mo_column_sort_order = _n_;

         %if ((&this_is_hist_tbl_flg eq 1) and 
              (&hist_mapping_col_rqd_flg eq 1)) %then %do;
            %if (&hist_mapping_col_type eq 1) %then %do;
               if (unique_comm_mapping_id eq .) then do;
                  putlog ' ';
                  putlog "%mo_error Missing value detected for UNIQUE_COMM_MAPPING_ID in observation " _N_ +(-1) " of %upcase(&comm_tbl_name) table.";
                  put    "input.data.validation.missing.value.error.message.fmt.txt,3,UNIQUE_COMM_MAPPING_ID," _N_ +(-1)",%upcase(&comm_tbl_name)";
                  putlog ' ';
                  call symput("&_rc",'8');
               end;
            %end;
            %else %do;
               if (unique_comm_mapping_id eq '') then do;
                  putlog ' ';
                  putlog "%mo_error Blank value detected for UNIQUE_COMM_MAPPING_ID in observation " _N_ +(-1) " of %upcase(&comm_tbl_name) table.";
                  put    "input.data.validation.blank.value.error.message.fmt.txt,3,UNIQUE_COMM_MAPPING_ID," _N_ +(-1)",%upcase(&comm_tbl_name)";
                  putlog ' ';
                  call symput("&_rc",'8');
               end;
               else do;
                  if (substr(unique_comm_mapping_id,1,1) eq ' ') then do;
                     putlog ' ';
                     putlog "%mo_error UNIQUE_COMM_MAPPING_ID in observation " _N_ +(-1) " of %upcase(&comm_tbl_name) table must not have leading blanks.";
                     put    "input.data.validation.leading.space.error.message.fmt.txt,3,UNIQUE_COMM_MAPPING_ID," _N_ +(-1)",%upcase(&comm_tbl_name)";
                     putlog ' ';
                     call symput("&_rc",'8');
                  end;
                  if (anyspace(substr(unique_comm_mapping_id,1,length(unique_comm_mapping_id))) gt 0) then do;
                     putlog ' ';
                     putlog "%mo_error UNIQUE_COMM_MAPPING_ID in observation " _N_ +(-1) " of %upcase(&comm_tbl_name) table must not have embedded spaces.";
                     put    "input.data.validation.embedded.space.error.message.fmt.txt,3,UNIQUE_COMM_MAPPING_ID," _N_ +(-1)",%upcase(&comm_tbl_name)";
                     putlog ' ';
                     call symput("&_rc",'8');
                  end;
                  if (index(unique_comm_mapping_id,',') gt 0) then do;
                     putlog ' ';
                     putlog "%mo_error UNIQUE_COMM_MAPPING_ID in observation " _N_ +(-1) " of %upcase(&comm_tbl_name) table must not contain commas.";
                     put    "input.data.validation.special.char.name.error.message.fmt.txt,3,UNIQUE_COMM_MAPPING_ID," _N_ +(-1)",%upcase(&comm_tbl_name)";
                     putlog ' ';
                     call symput("&_rc",'8');
                  end;
                  if (length(unique_comm_mapping_id) > 32) then do;
                     putlog ' ';
                     putlog "%mo_error Length of UNIQUE_COMM_MAPPING_ID in observation " _N_ +(-1) " of %upcase(&comm_tbl_name) table cannot be greater than 32.";
                     put    "input.data.validation.column.length.error.message.fmt.txt,3,UNIQUE_COMM_MAPPING_ID," _N_ +(-1)",%upcase(&comm_tbl_name)";
                     putlog ' ';
                     call symput("&_rc",'8');
                  end;
               end;
            %end;
         %end;
         if (campaign_cd eq '') then do;
            putlog ' ';
            putlog "%mo_error Blank value detected for CAMPAIGN_CD in observation " _N_ +(-1) " of %upcase(&comm_tbl_name) table.";
            put    "input.data.validation.blank.value.error.message.fmt.txt,3,CAMPAIGN_CD," _N_ +(-1)",%upcase(&comm_tbl_name)";
            putlog ' ';
            call symput("&_rc",'8');
         end;
         else do;
            if (substr(campaign_cd,1,1) eq ' ') then do;
               putlog ' ';
               putlog "%mo_error CAMPAIGN_CD in observation " _N_ +(-1) " of %upcase(&comm_tbl_name) table must not have leading blanks.";
               put    "input.data.validation.leading.space.error.message.fmt.txt,3,CAMPAIGN_CD," _N_ +(-1)",%upcase(&comm_tbl_name)";
               putlog ' ';
               call symput("&_rc",'8');
            end;
            if (anyspace(substr(campaign_cd,1,length(campaign_cd))) gt 0) then do;
               putlog ' ';
               putlog "%mo_error CAMPAIGN_CD in observation " _N_ +(-1) " of %upcase(&comm_tbl_name) table must not have embedded spaces.";
               put    "input.data.validation.embedded.space.error.message.fmt.txt,3,CAMPAIGN_CD," _N_ +(-1)",%upcase(&comm_tbl_name)";
               putlog ' ';
               call symput("&_rc",'8');
            end;
            if (index(campaign_cd,',') gt 0) then do;
               putlog ' ';
               putlog "%mo_error CAMPAIGN_CD in observation " _N_ +(-1) " of %upcase(&comm_tbl_name) table must not contain commas.";
               put    "input.data.validation.special.char.name.error.message.fmt.txt,3,CAMPAIGN_CD," _N_ +(-1)",%upcase(&comm_tbl_name)";
               putlog ' ';
               call symput("&_rc",'8');
            end;
            if (length(campaign_cd) > 32) then do;
               putlog ' ';
               putlog "%mo_error Length of CAMPAIGN_CD in observation " _N_ +(-1) " of %upcase(&comm_tbl_name) table cannot be greater than 32.";
               put    "input.data.validation.column.length.error.message.fmt.txt,3,CAMPAIGN_CD," _N_ +(-1)",%upcase(&comm_tbl_name)";
               putlog ' ';
               call symput("&_rc",'8');
            end;
            if (nomatch_campaign_cd = 1) then do;
               putlog ' ';
               putlog "%mo_error Invalid CAMPAIGN_CD detected in observation " _N_ +(-1) " of %upcase(&comm_tbl_name) table.";
               put    "input.data.validation.invalid.value.error.message.fmt.txt,3,CAMPAIGN_CD," _N_ +(-1)",%upcase(&comm_tbl_name)";
               putlog ' ';
               call symput("&_rc",'8');
            end;
         end;
         if (communication_cd eq '') then do;
            putlog ' ';
            putlog "%mo_error Blank value detected for COMMUNICATION_CD in observation " _N_ +(-1) " of %upcase(&comm_tbl_name) table.";
            put    "input.data.validation.blank.value.error.message.fmt.txt,3,COMMUNICATION_CD," _N_ +(-1)",%upcase(&comm_tbl_name)";
            putlog ' ';
            call symput("&_rc",'8');
         end;
         else do;
            if (substr(communication_cd,1,1) eq ' ') then do;
               putlog ' ';
               putlog "%mo_error COMMUNICATION_CD in observation " _N_ +(-1) " of %upcase(&comm_tbl_name) table must not have leading blanks.";
               put    "input.data.validation.leading.space.error.message.fmt.txt,3,COMMUNICATION_CD," _N_ +(-1)",%upcase(&comm_tbl_name)";
               putlog ' ';
               call symput("&_rc",'8');
            end;
            if (anyspace(substr(communication_cd,1,length(communication_cd))) gt 0) then do;
               putlog ' ';
               putlog "%mo_error COMMUNICATION_CD in observation " _N_ +(-1) " of %upcase(&comm_tbl_name) table must not have embedded spaces.";
               put    "input.data.validation.embedded.space.error.message.fmt.txt,3,COMMUNICATION_CD," _N_ +(-1)",%upcase(&comm_tbl_name)";
               putlog ' ';
               call symput("&_rc",'8');
            end;
            if (index(communication_cd,',') gt 0) then do;
               putlog ' ';
               putlog "%mo_error COMMUNICATION_CD in observation " _N_ +(-1) " of %upcase(&comm_tbl_name) table must not contain commas.";
               put    "input.data.validation.special.char.name.error.message.fmt.txt,3,COMMUNICATION_CD," _N_ +(-1)",%upcase(&comm_tbl_name)";
               putlog ' ';
               call symput("&_rc",'8');
            end;
            if (length(communication_cd) > 32) then do;
               putlog ' ';
               putlog "%mo_error Length of COMMUNICATION_CD in observation " _N_ +(-1) " of %upcase(&comm_tbl_name) table cannot be greater than 32.";
               put    "input.data.validation.column.length.error.message.fmt.txt,3,COMMUNICATION_CD," _N_ +(-1)",%upcase(&comm_tbl_name)";
               putlog ' ';
               call symput("&_rc",'8');
            end;
         end;
         %if (&cell_cd_in_comm_table eq 1) %then %do;
            if (cell_cd eq '') then do;
               putlog ' ';
               putlog "%mo_error Blank value detected for CELL_CD in observation " _N_ +(-1) " of %upcase(&comm_tbl_name) table.";
                put    "input.data.validation.blank.value.error.message.fmt.txt,3,CELL_CD," _N_ +(-1)",%upcase(&comm_tbl_name)";
               putlog ' ';
               call symput("&_rc",'8');
            end;
            else do;
               if (substr(cell_cd,1,1) eq ' ') then do;
                  putlog ' ';
                  putlog "%mo_error CELL_CD in observation " _N_ +(-1) " of %upcase(&comm_tbl_name) table must not have leading blanks.";
                  put    "input.data.validation.leading.space.error.message.fmt.txt,3,CELL_CD," _N_ +(-1)",%upcase(&comm_tbl_name)";
                  putlog ' ';
                  call symput("&_rc",'8');
               end;
               if (anyspace(substr(cell_cd,1,length(cell_cd))) gt 0) then do;
                  putlog ' ';
                  putlog "%mo_error CELL_CD in observation " _N_ +(-1) " of %upcase(&comm_tbl_name) table must not have embedded spaces.";
                  put    "input.data.validation.embedded.space.error.message.fmt.txt,3,CELL_CD," _N_ +(-1)",%upcase(&comm_tbl_name)";
                  putlog ' ';
                  call symput("&_rc",'8');
               end;
               if (index(cell_cd,',') gt 0) then do;
                  putlog ' ';
                  putlog "%mo_error CELL_CD in observation " _N_ +(-1) " of %upcase(&comm_tbl_name) table must not contain commas.";
                  put    "input.data.validation.special.char.name.error.message.fmt.txt,3,CELL_CD," _N_ +(-1)",%upcase(&comm_tbl_name)";
                  putlog ' ';
                  call symput("&_rc",'8');
               end;
               if (length(cell_cd) > 32) then do;
                  putlog ' ';
                  putlog "%mo_error Length of CELL_CD in observation " _N_ +(-1) " of %upcase(&comm_tbl_name) table cannot be greater than 32.";
                  put    "input.data.validation.column.length.error.message.fmt.txt,3,CELL_CD," _N_ +(-1)",%upcase(&comm_tbl_name)";
                  putlog ' ';
                  call symput("&_rc",'8');
               end;
            end;
         %end;
         if ((time_period ne '') and (nomatch_time_period = 1)) then do;
            putlog ' ';
            putlog "%mo_error Invalid TIME_PERIOD detected in observation " _N_ +(-1) " of %upcase(&comm_tbl_name) table.";
            put    "input.data.validation.invalid.value.error.message.fmt.txt,3,TIME_PERIOD," _N_ +(-1)",%upcase(&comm_tbl_name)";
            putlog ' ';
            call symput("&_rc",'8');
         end;
         %if (%bquote(&chan_tbl_name) ne %str()) %then %do;
            if ((channel_cd ne '') and (nomatch_channel_cd = 1)
               %if (%bquote(&agnt_col) ne %str() and %bquote(&agnt_tbl_name) ne %str()) %then %do;
                  and (upcase(compbl(trim(left(channel_cd)))) ne upcase("&agnt_col")) 
               %end;      
               ) then do;
               putlog ' ';
               putlog "%mo_error Invalid CHANNEL_CD detected in observation " _N_ +(-1) " of %upcase(&comm_tbl_name) table.";
               put    "input.data.validation.invalid.value.error.message.fmt.txt,3,CHANNEL_CD," _N_ +(-1)",%upcase(&comm_tbl_name)";
               putlog ' ';
               call symput("&_rc",'8');
            end;
         %end;
         if (length(time_period) > 32) then do;
            putlog ' ';
            putlog "%mo_error Length of TIME_PERIOD in observation " _N_ +(-1) " of %upcase(&comm_tbl_name) table cannot be greater than 32.";
            put    "input.data.validation.column.length.error.message.fmt.txt,3,TIME_PERIOD," _N_ +(-1)",%upcase(&comm_tbl_name)";
            putlog ' ';
            call symput("&_rc",'8');
         end;
         if (length(channel_cd) > 32) then do;
            putlog ' ';
            putlog "%mo_error Length of CHANNEL_CD in observation " _N_ +(-1) " of %upcase(&comm_tbl_name) table cannot be greater than 32.";
            put    "input.data.validation.column.length.error.message.fmt.txt,3,CHANNEL_CD," _N_ +(-1)",%upcase(&comm_tbl_name)";
            putlog ' ';
            call symput("&_rc",'8');
         end;
         if (unit_usage lt 0 and unit_usage ne .) then do;
            putlog ' ';
            putlog "%mo_warning Negative UNIT_USAGE detected in observation " _N_ +(-1) " of %upcase(&comm_tbl_name) table.  Unintended results or infeasibilities may occur.";
            put    "input.data.validation.negative.value.warning.message.fmt.txt,3,UNIT_USAGE," _N_ +(-1)",%upcase(&comm_tbl_name)";
            putlog ' ';
            call symput("&_rc",put(max(&&&_rc,4),1.));
         end;
         %if (&mandatory_cols_rqd_flg eq 1) %then %do;
            if (mandatory not in (0,1)) then do;
               putlog ' ';
               putlog "%mo_error Invalid MANDATORY value detected in observation " _N_ +(-1) " of %upcase(&comm_tbl_name) table. MANDATORY must be 0 or 1.";
               put    "input.data.validation.valid.flg.error.message.fmt.txt,3,MANDATORY," _N_+(-1)",%upcase(&comm_tbl_name)";
               putlog ' ';
               call symput("&_rc",'8');
            end;
            if (exclude_from_constraints not in (0,1)) then do;
               putlog ' ';
               putlog "%mo_error Invalid EXCLUDE_FROM_CONSTRAINTS value detected in observation " _N_ +(-1) " of %upcase(&comm_tbl_name) table. EXCLUDE_FROM_CONSTRAINTS must be 0 or 1.";
               put    "input.data.validation.valid.flg.error.message.fmt.txt,3,EXCLUDE_FROM_CONSTRAINTS," _N_+(-1)",%upcase(&comm_tbl_name)";
               putlog ' ';
               call symput("&_rc",'8');
            end;
            if (exclude_from_contact_pols not in (0,1)) then do;
               putlog ' ';
               putlog "%mo_error Invalid EXCLUDE_FROM_CONTACT_POLS value detected in observation " _N_ +(-1) " of %upcase(&comm_tbl_name) table. EXCLUDE_FROM_CONTACT_POLS must be 0 or 1.";
               put    "input.data.validation.valid.flg.error.message.fmt.txt,3,EXCLUDE_FROM_CONTACT_POLS," _N_+(-1)",%upcase(&comm_tbl_name)";
               putlog ' ';
               call symput("&_rc",'8');
            end;
            if (mandatory = 0) then do;
               if (exclude_from_constraints eq 1) then do;
                  putlog ' ';
                  putlog "%mo_warning EXCLUDE_FROM_CONSTRAINTS value of 1 has no effect when MANDATORY value is 0 in observation " _N_ +(-1) " of %upcase(&comm_tbl_name) table.";
                  putlog ' ';
               end;
               if (exclude_from_contact_pols eq 1) then do;
                  putlog ' ';
                  putlog "%mo_warning EXCLUDE_FROM_CONTACT_POLS value of 1 has no effect when MANDATORY value is 0 in observation " _N_ +(-1) " of %upcase(&comm_tbl_name) table.";
                  putlog ' ';
               end;
            end;
         %end;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered creating &_comm_tbl_work (syserr=&syserr).;
         %goto VAL_COMM_ERROR_EXIT;
      %end;

      %*;
      %* Count rows in the Communication table for use in following checks. ;
      %*;
      %local num_comm_tbl_rows num_unique_camp_comm num_unique_camp_comm_cell;
      %let num_comm_tbl_rows = 0;
      %let num_unique_camp_comm = 0;
      %let num_unique_camp_comm_cell = 0;
      proc sql noprint;
         select count(*)
               ,count(distinct trim(left(campaign_cd)) || "/" ||
                               trim(left(communication_cd)))
            %if (&cell_cd_in_comm_table eq 1) %then %do;
               ,count(distinct trim(left(campaign_cd)) || "/" ||
                               trim(left(communication_cd)) || "/" ||
                               trim(left(cell_cd)))
            %end;
         into :num_comm_tbl_rows
             ,:num_unique_camp_comm 
            %if (&cell_cd_in_comm_table eq 1) %then %do;
             ,:num_unique_camp_comm_cell
            %end;
         from &input_libname..&comm_tbl_name;
      quit;
      %if (&sqlrc gt 4) %then %do;
         %let &_rc = 107;
         %put %mo_error SQL error encountered obtaining information from &input_libname..&comm_tbl_name (sqlrc=&sqlrc).;
         %goto VAL_COMM_ERROR_EXIT;
      %end;

      %*;
      %* Check for duplicate CAMPAIGN_CD/COMMUNICATION_CD combinations.        ;
      %* If duplicate CAMPAIGN_CD/COMMUNICATION_CD combinations exist,         ;
      %* and the optional CELL_CD column is present, check for duplicate       ;
      %* CAMPAIGN_CD/COMMUNICATION_CD/CELL_CD combinations.                    ;
      %*                                                                       ;
      %* This check excluded if this is the Communication history table and    ;
      %* the UNIQUE_COMM_MAPPING_ID column is being used.                      ;
      %*;
      %if ((&this_is_hist_tbl_flg eq 0) or 
           (&hist_mapping_col_rqd_flg eq 0)) %then %do;

         %*;
         %* If there are duplicate CAMPAIGN_CD/COMMUNICATION_CD combos AND the ;
         %* CELL_CD is either not present or, if present, does not provide     ;
         %* uniqueness, identify duplicate combos.                             ;
         %*;
         %if ((&num_unique_camp_comm lt &num_comm_tbl_rows) and
              (&num_unique_camp_comm_cell lt &num_comm_tbl_rows)) %then %do;
            proc sort data=&_comm_tbl_work;
               by campaign_cd communication_cd
               %if (&cell_cd_in_comm_table eq 1) %then %do;
                  cell_cd
               %end;
               ;
            run;
            %if (&syserr gt 4) %then %do;
               %let &_rc = 100;
               %put %mo_error Error encountered sorting &_comm_tbl_work (syserr=&syserr).;
               %goto VAL_COMM_ERROR_EXIT;
            %end;

            data _null_;
               set &_comm_tbl_work;
               retain prev_camp_code prev_comm_code
                   %if (&cell_cd_in_comm_table eq 1) %then %do;
                      prev_cell_cd
                   %end;
               ;
               &_setfile;
               if (_N_ gt 1) then do;
                  if ((campaign_cd eq prev_camp_code)
                      and (communication_cd eq prev_comm_code) 
                   %if (&cell_cd_in_comm_table eq 1) %then %do;
                      and (cell_cd eq prev_cell_cd)
                   %end;
                     ) then do;
                     putlog ' ';
                     %if (&cell_cd_in_comm_table eq 1) %then %do;
                        putlog ' ';
                        putlog "%mo_error Duplicate CAMPAIGN_CD/COMMUNICATION_CD/CELL_CD combination detected in %upcase(&comm_tbl_name) table where CAMPAIGN_CD = " campaign_cd " and COMMUNICATION_CD = " communication_cd " and CELL_CD = " cell_cd ".";
                        put    "input.data.validation.duplicate.combination.cell.error.message.fmt.txt,10,CAMPAIGN_CD,COMMUNICATION_CD,CELL_CD,%upcase(&comm_tbl_name),CAMPAIGN_CD," campaign_cd ",COMMUNICATION_CD," communication_cd ",CELL_CD," cell_cd;
                     %end;
                     %else %do;
                        putlog ' ';
                        putlog "%mo_error Duplicate CAMPAIGN_CD/COMMUNICATION_CD combination detected in %upcase(&comm_tbl_name) table where CAMPAIGN_CD = " campaign_cd " and COMMUNICATION_CD = " communication_cd ".";
                        put    "input.data.validation.duplicate.combination.error.message.fmt.txt,7,CAMPAIGN_CD,COMMUNICATION_CD,%upcase(&comm_tbl_name),CAMPAIGN_CD," campaign_cd ",COMMUNICATION_CD," communication_cd;
                     %end;
                     putlog ' ';
                     call symput("&_rc",'8');
                  end;
               end;
               prev_camp_code = campaign_cd;
               prev_comm_code = communication_cd;
               %if (&cell_cd_in_comm_table eq 1) %then %do;
                  prev_cell_cd = cell_cd;
               %end;
            run;
            %if (&syserr gt 4) %then %do;
               %let &_rc = 100;
               %put %mo_error Error encountered processing &_comm_tbl_work for duplicate campaign_cd/communication_cd/cell_cd combinations (syserr=&syserr).;
               %goto VAL_COMM_ERROR_EXIT;
            %end;
         %end;
      %end;
   
      %*;
      %* Set indicator flag indicating level of cell_cd column usefulness:     ;
      %*    - not present                                                      ;
      %*    - present, but does not provide uniqueness for                     ;
      %*      duplicate CAMPAIGN_CD/COMMUNICATION_CD combos, i.e.,             ;
      %*      duplicate CAMPAIGN_CD/COMMUNICATION_CD/CELL_CD combos exist      ;
      %*    - present, but not needed to provide uniqueness because            ;
      %*      no duplicate CAMPAIGN_CD/COMMUNICATION_CD combos exist           ;
      %*    - present AND used to provide uniqueness because                   ;
      %*      duplicate CAMPAIGN_CD/COMMUNICATION_CD combos exist              ;
      %*;
      %if (&cell_cd_in_comm_table eq 0) %then %do;
         %let &_cell_cd_stat = &_CELL_CD_NOT_PRESENT_;
      %end;
      %else %do; /* CELL_CD present */
         %if (&num_unique_camp_comm eq &num_comm_tbl_rows) %then %do;
            %let &_cell_cd_stat = &_CELL_CD_UNIQUE_UNNEEDED_;
         %end;
         %else %do; /* duplicate CAMPAIGN_CD/COMMUNICATION_CD combos */ 
            %if (&num_unique_camp_comm_cell eq &num_comm_tbl_rows) %then %do;
               %let &_cell_cd_stat = &_CELL_CD_UNIQUE_;
            %end;
            %else %do; /* duplicate CAMPAIGN_CD/COMMUNICATION_CD/CELL_CD combos */ 
               %let &_cell_cd_stat = &_CELL_CD_NON_UNIQUE_;
            %end;
         %end;
      %end;

      %*;
      %* Warn if all PRIORITY column values are missing. ;
      %*;
      %if (&priority_col_rqd_flg eq 1) %then %do;
         %local num_missing_priority_values;
         %let num_missing_priority_values = 0;
         proc sql noprint;
            select nmiss(&priority_col) into :num_missing_priority_values 
            from &input_libname..&comm_tbl_name;
         quit;
         %if (&sqlrc gt 4) %then %do;
            %let &_rc = 107;
            %put %mo_error SQL error encountered counting %upcase(&priority_col) missing values in &input_libname..&comm_tbl_name (sqlrc=&sqlrc).;
            %goto VAL_COMM_ERROR_EXIT;
         %end;

         %if (&num_missing_priority_values eq &num_comm_tbl_rows) %then %do;
            data _null_;
               &setfile;
               putlog ' ';
               putlog "%mo_warning All rows contain missing values for the %upcase(&priority_col) column in the %upcase(&comm_tbl_name) table.";
               put    "input.data.validation.missing.all.values.warning.message.fmt.txt,2,%upcase(&priority_col),%upcase(&comm_tbl_name)";
               putlog ' ';
               call symput("&_rc",put(max(&&&_rc,4),1.));
            run;
         %end;
      %end;

      %*;
      %* Build list of of distinct COMMUNICATION_CDs and ;
      %* distinct CELL_CDS, if present.                  ;
      %*;
      %let &_num_distinct_comm_cds = 0;
      %let &_num_distinct_cell_cds = 0;
      proc sql noprint;
         create table &_distinct_comm_cd_tbl as
         select distinct(compbl(trim(left(communication_cd)))) as communication_cd
         from &comm_tbl_in_mo;

         select count(*) into :&_num_distinct_comm_cds
         from &_distinct_comm_cd_tbl;

         %if (&cell_cd_in_comm_table eq 1) %then %do;
            create table &_distinct_cell_cd_tbl as
            select distinct(compbl(trim(left(cell_cd)))) as cell_cd
            from &comm_tbl_in_mo;

            select count(*) into :&_num_distinct_cell_cds
            from &_distinct_cell_cd_tbl;
         %end;
      quit;
      %if (&sqlrc gt 4) %then %do;
         %let &_rc = 107;
         %put %mo_error SQL error encountered obtaining distinct communication_cds and/or cell_cds from &comm_tbl_in_mo (sqlrc=&sqlrc).;
         %goto VAL_COMM_ERROR_EXIT;
      %end;

      %*;
      %* Check for CHANNEL_CD and UNIT_USAGE when agents are present. ;
      %* Do this only for the regular Communications table.           ;
      %*;
      %local channel_cd_in_comm_table;
      %let channel_cd_in_comm_table = 0;

      %local dsid channel_cd_col_varnum;  
      %let channel_cd_col_varnum = 0;
      %let dsid = %sysfunc(open(&input_libname..&comm_tbl_name));
      %if (&dsid = 0) %then %do;
         data _null_;
            &setfile;
            putlog ' ';
            %putlog "%mo_error The specified &comm_tbl_name table does not exist in &input_libname.";
            put "input.data.validation.table.not.exist.error.message.fmt.txt,1,&comm_tbl_name";            
            call symput("&_rc",'8');
         run;
         %goto ERROR_EXIT;
      %end;
      %let channel_cd_col_varnum = %sysfunc(varnum(&dsid, channel_cd));
      %if (&channel_cd_col_varnum ne 0) %then %do;
         %let channel_cd_in_comm_table = 1;
      %end;
      %let dsid = %sysfunc(close(&dsid));

      %local num_comm_agents;
      %let num_comm_agents = 0;
      %if ((&this_is_hist_tbl_flg eq 0) and (%bquote(&agnt_tbl_name) ne %str())) %then %do;

         %* If agents are in use, then a CHANNEL_CD column must exist in the   ;
         %* Communication table.                                               ;
         %if (&channel_cd_in_comm_table eq 0) %then %do;
            data _null_;
               &_setfile;
               putlog ' ';
               putlog "%mo_error Missing column detected in %upcase(&comm_tbl_name) table: CHANNEL_CD.";
               putlog "%mo_error The %upcase(&comm_tbl_name) table must contain a CHANNEL_CD column when agent constraints are used.";
               put "input.data.validation.missing.column.error.message.fmt.txt,2,%upcase(&comm_tbl_name),CHANNEL_CD";
               putlog ' ';
               call symput("&_rc",'8');
            run;
            %if (&syserr gt 4) %then %do;
               %let &_rc = 100;
               %put %mo_error Error encountered checking &comm_tbl_name for channel_cd with agents (syserr=&syserr).;
               %goto VAL_COMM_ERROR_EXIT;
            %end;
         %end;
         %else %do;
            proc sql noprint;
               select count(*) into :num_comm_agents
               from &input_libname..&comm_tbl_name
               where (upcase(channel_cd) eq upcase("&agnt_col"));
            quit;
            %if (&sqlrc gt 4) %then %do;
               %let &_rc = 107;
               %put %mo_error SQL error encountered retrieving agent channel_cd information from &input_libname..&comm_tbl_name (sqlrc=&sqlrc).;
               %goto VAL_COMM_ERROR_EXIT;
            %end;

            %* Issue a warning if Communication table has no rows that use     ;
            %* the specified agent column as the channel_cd value.             ;
            %if (&num_comm_agents eq 0) %then %do;
               data _null_;
                  &_setfile;
                  putlog ' ';
                  putlog "%mo_warning Agents are specified in the %upcase(&agnt_tbl_name) and %upcase(&cust_tbl_name) tables but none are specified in the %upcase(&comm_tbl_name) table.";
                  put    "input.data.validation.missing.agent.communication.warning.message.fmt.txt,4,%upcase(&agnt_col),%upcase(&agnt_tbl_name),%upcase(&cust_tbl_name),%upcase(&comm_tbl_name)";
                  putlog ' ';
                  call symput("&_rc",put(max(&&&_rc,4),1.));
               run;
            %end;

            %* Unit usage must be specified for each agent row in the          ;
            %* Communication table.                                            ;
            %else %do;
               data _null_;
                  set &_comm_tbl_work;
                  &_setfile;
                  if ((upcase(channel_cd) eq upcase("&agnt_col")) and (unit_usage eq .)) then do;
                     length msg msg1 msg2 $512.;
                     msg1= "%mo_error Must specify an UNIT_USAGE in the %upcase(&comm_tbl_name) table for every record where CHANNEL_CD = "|| strip(channel_cd);
                     msg2= " because agent capacity constraints are specified in the %upcase(&agnt_tbl_name) table.";
                     msg= strip(msg1) || ' ' || strip(msg2);
                     putlog ' ';
                     putlog msg;
                     put    "input.data.validation.missing.unit_usage.agent.error.message.fmt.txt,5,UNIT_USAGE,%upcase(&comm_tbl_name),CHANNEL_CD," channel_cd +(-1)",%upcase(&agnt_tbl_name)";
                     putlog ' ';
                     call symput("&_rc",'8');
                  end;
               run;
               %if (&syserr gt 4) %then %do;
                  %let &_rc = 100;
                  %put %mo_error Error encountered checking for agent unit_usage (syserr=&syserr).;
                  %goto VAL_COMM_ERROR_EXIT;
               %end;
            %end;
         %end;
      %end;

      %goto VAL_COMM_EXIT;

      %*;
      %* Exit processing ;
      %*;
      %VAL_COMM_ERROR_EXIT:
      %put %mo_error Failed to validate %upcase(&comm_tbl_name) table.;

      %VAL_COMM_EXIT:
   %mend mo_validate_comm_tbl;


   %*;
   %* Validate the Communication table.;
   %*;
   %mo_validate_comm_tbl(input_libname = moinput
                        ,comm_tbl_name = &communication_table
                        ,comm_tbl_in_mo = work.communication_in_mo_tbl
                        ,mandatory_cols_rqd_flg = &mandatory_cols_required
                        ,priority_col_rqd_flg = &priority_col_required
                        ,priority_col = &priority_column
                        ,this_is_hist_tbl_flg = 0
                        ,hist_mapping_col_rqd_flg = 0
                        ,hist_mapping_col_type = 0
                        ,distinct_camp_cd_tbl = work.distinct_campaign_cds
                        ,num_distinct_camp_cds = &num_distinct_campaign_cds
                        ,chan_tbl_name = &channel_table
                        ,distinct_chan_cd_tbl = work.distinct_channel_cds
                        ,num_distinct_chan_cds = &num_distinct_channel_cds
                        ,time_pd_tbl_name = &time_period_table
                        ,distinct_time_pd_tbl = work.distinct_time_periods
                        ,num_distinct_time_pds = &num_distinct_time_periods 
                        ,cust_tbl_name = &customer_table
                        ,agnt_tbl_name = &agent_capacity_table
                        ,agnt_col = &agent_column
                        ,_comm_tbl_work = work.&communication_table
                        ,_distinct_comm_cd_tbl = work.distinct_communication_cds
                        ,_num_distinct_comm_cds = num_distinct_communication_cds
                        ,_distinct_cell_cd_tbl = work.distinct_cell_cds
                        ,_num_distinct_cell_cds = num_distinct_cell_cds
                        ,_cell_cd_stat = &_cell_cd_indicator
                        ,_time_pd_stat = &_time_period_indicator
                        ,_rc = &_rc);


   %*-----------------------------------------------------------------------;
   %* If contact history is in use, check COMMUNICATION HISTORY table data. ;
   %*-----------------------------------------------------------------------;
   %COMM_HIST:

   %*;
   %* If contact history is in use:                                            ;
   %* 1. Verify that the regular Communication table and the Communication     ;
   %*    History table have identical format, i.e., identical column           ;
   %*    names/types  (except for the mapping column).                         ;
   %* 2. Validate the Communication History table individually.                ;
   %* 3. Verify that the Communication History table has unique values in the  ;
   %*    UNIQUE_COMM_MAPPING_ID column (MOMA or MOSA w/ the mapping column).   ;
   %*                                                                          ;
   %*    If the UNIQUE_COMM_MAPPING_ID column is not being used (MOSA), the    ;
   %*    Communication History table will be checked for unique combinations   ;
   %*    of campaign_cd/communication_cd by the mo_validate_comm_tbl macro.    ;
   %*;
   %if (&uses_contact_history eq 1) %then %do;

      %*;
      %* Compare table structures. ;
      %*;
      proc contents data=moinput.&communication_table out=work.contents_comm_reg (keep=name type) noprint;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered obtaining contents of MOINPUT.&communication_table (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;

      data work.contents_comm_reg;
         set work.contents_comm_reg;
         name=upcase(name);
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered processing work.contents_comm_reg (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;

      proc sort data=contents_comm_reg;
         by name;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered sorting work.contents_comm_reg (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;

      proc contents data=moinput.&communication_history_table out=contents_comm_hist (keep=name type) noprint;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered obtaining contents of MOINPUT.&communication_history_table (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;

      data work.contents_comm_hist;
         set work.contents_comm_hist;
         name=upcase(name);
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered processing work.contents_comm_hist (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;

      proc sort data=contents_comm_hist;
         by name;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered sorting work.contents_comm_hist (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;

      data work.compare_reg_and_hist;
         length colIn $4.;
         merge work.contents_comm_reg (in=a rename=(type=regType)) 
               work.contents_comm_hist (in=b rename=(type=histType));
         by name;

         if (a and b) then do;
            colIn = "both";
         end;
         else if (a) then do;
            colIn = "reg";
         end;
         else do;
            colIn="hist";
         end;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered creating work.compare_reg_and_hist (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;
   
      data _null_;
         set work.compare_reg_and_hist;
         &setfile;
         if (colIn eq "both") then do;
            if (regType ~= histType) then do;
               putlog ' ';
               putlog "%mo_error Column has different data types in the %upcase(&communication_table) table and the %upcase(&communication_history_table) table: " name;
               put "input.data.validation.contactHist.type.mismatch.error.message.fmt.txt,3," name ",%upcase(&communication_table),%upcase(&communication_history_table)";
               putlog ' ';
               call symput("&_rc",'8');
            end;
         end;
         else if (colIn eq "reg") then do;
            putlog ' ';
            putlog "%mo_error Column in %upcase(&communication_table) table not found in %upcase(&communication_history_table) table: " name;
           put "input.data.validation.contactHist.missing.column.error.message.fmt.txt,3," name ",%upcase(&communication_table),%upcase(&communication_history_table)";
           putlog ' ';
           call symput("&_rc",'8');
         end;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered processing work.compare_reg_and_hist (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;

      %*;
      %* Validate the Communication History table.;
      %*;
      %mo_validate_comm_tbl(input_libname = moinput
                           ,comm_tbl_name = &communication_history_table
                           ,comm_tbl_in_mo = work.communication_history_in_mo_tbl
                           ,mandatory_cols_rqd_flg = &mandatory_cols_required
                           ,priority_col_rqd_flg = &priority_col_required
                           ,priority_col = &priority_column
                           ,this_is_hist_tbl_flg = 1
                           ,hist_mapping_col_rqd_flg = &hist_mapping_col_required
                           ,hist_mapping_col_type = &hist_mapping_col_type
                           ,distinct_camp_cd_tbl = work.distinct_campaign_cds
                           ,num_distinct_camp_cds = &num_distinct_campaign_cds
                           ,chan_tbl_name = &channel_table
                           ,distinct_chan_cd_tbl = work.distinct_channel_cds
                           ,num_distinct_chan_cds = &num_distinct_channel_cds
                           ,time_pd_tbl_name = &time_period_table
                           ,distinct_time_pd_tbl = work.distinct_time_periods
                           ,num_distinct_time_pds = &num_distinct_time_periods 
                           ,cust_tbl_name = &customer_table
                           ,agnt_tbl_name = &agent_capacity_table
                           ,agnt_col = &agent_column
                           ,_comm_tbl_work = work.&communication_history_table
                           ,_distinct_comm_cd_tbl = work.distinct_comm_hist_cds
                           ,_num_distinct_comm_cds = num_distinct_comm_hist_cds
                           ,_distinct_cell_cd_tbl = work.distinct_cell_hist_cds
                           ,_num_distinct_cell_cds = num_distinct_cell_hist_cds
                           ,_cell_cd_stat = cell_cd_dont_care
                           ,_time_pd_stat = time_period_dont_care
                           ,_rc = &_rc);

      %*;
      %* If the UNIQUE_COMM_MAPPING_ID column is being used, ;
      %* verify that all column values are actually unique.  ;
      %*;
      %if (&hist_mapping_col_required eq 1) %then %do;
         %local hist_tbl_row_count;
         %local unique_mapping_id_count;
         %let hist_tbl_row_count = 0;
         %let unique_mapping_id_count = 0;
         proc sql noprint;
            select count(*), count (distinct unique_comm_mapping_id)
            into :hist_tbl_row_count, :unique_mapping_id_count 
            from moinput.&communication_history_table;
         quit;
         %if (&sqlrc gt 4) %then %do;
            %let &_rc = 107;
            %put %mo_error SQL error encountered counting rows in moinput.&communication_history_table (sqlrc=&sqlrc).;
            %goto ERROR_EXIT;
         %end;

         %if (&unique_mapping_id_count ne &hist_tbl_row_count) %then %do;
            data _null_;
               &setfile;
               putlog ' ';
               putlog "%mo_error The values for the UNIQUE_COMM_MAPPING_ID column in the %upcase(&communication_history_table) table must be unique.";
               put    "input.data.validation.column.values.notunique.error.message.txt,2,UNIQUE_COMM_MAPPING_ID,%upcase(&communication_history_table)";
               putlog ' ';
               call symput("&_rc",put(max(&&&_rc,4),1.));
            run;

            proc sort data=moinput.&communication_history_table(keep=unique_comm_mapping_id) out=work.mapping_ids;
               by unique_comm_mapping_id;
            run;
            %if (&syserr gt 4) %then %do;
               %let &_rc = 100;
               %put %mo_error Error encountered sorting moinput.&communication_history_table (syserr=&syserr).;
               %goto ERROR_EXIT;
            %end;

            data _null_;
               set work.mapping_ids;
               &setfile;
               retain prev_id;
               if (_N_ gt 1) then do;
                  if (unique_comm_mapping_id eq prev_id) then do;
                     putlog ' ';
                     putlog "%mo_error Duplicate UNIQUE_COMM_MAPPING_ID detected in %upcase(&communication_history_table) table where UNIQUE_COMM_MAPPING_ID = " unique_comm_mapping_id ".";
                     put    "input.data.validation.duplicate.value.error.message.fmt.txt,4,UNIQUE_COMM_MAPPING_ID,%upcase(&campaign_table),UNIQUE_COMM_MAPPING_ID," unique_comm_mapping_id;
                     putlog ' ';
                     call symput("&_rc",'8');
                  end;
               end;
               prev_id = unique_comm_mapping_id;
            run;
            %if (&syserr gt 4) %then %do;
               %let &_rc = 100;
               %put %mo_error Error encountered processing work.mapping_ids for duplicate values of unique_comm_mapping_id (syserr=&syserr).;
               %goto ERROR_EXIT;
            %end;
         %end; 
      %end;
   %end;


   %*-----------------------------------------------------------------;
   %* If contact history is in use, check CONTACT HISTORY table data. ;
   %*-----------------------------------------------------------------;
   %CONT_HIST:

   %*;
   %* If contact history is in use:                                        ;
   %* 1. Validate the CONTACT HISTORY table individually.                  ;
   %* 2. Determine if the CONTACT HISTORY table has values for the         ;
   %*    UNIQUE_COMM_MAPPING_ID column that appear in the COMMUNICATION    ;
   %*    HISTORY table (MOMA or MOSA with the mapping column).             ;
   %*    If the UNIQUE_COMM_MAPPING_ID column is not being used (MOSA),    ;
   %*    determine if the CONTACT HISTORY has values for combinations of   ;
   %*    campaign_cd/communication_cd that appear in the COMMUNICATION     ;
   %*    HISTORY table.                                                    ;
   %* 3. Determine if the CONTACT HISTORY table Customer ID column type(s) ;
   %*    match the corresponding types of the Customer ID columns in the   ;
   %*    Customer table. [Customer ID column name presence was ascertained ;
   %*    earlier in this macro.]                                           ;
   %*;
   %if (&uses_contact_history eq 1) %then %do;

      %*;
      %* Do basic Contact History table data validation.             ;
      %*;
      data work.&contact_history_table;
         set work.contact_history_in_mo_tbl;
         &setfile;

         %if (&hist_mapping_col_required eq 1) %then %do;
            %if (&hist_mapping_col_type eq 2) %then %do;
               unique_comm_mapping_id=trim(left(unique_comm_mapping_id));
            %end;
         %end;
         %else %do;
            campaign_cd=trim(left(campaign_cd));
            communication_cd=trim(left(communication_cd));
         %end;

         %if (&hist_mapping_col_required eq 1) %then %do;
            %if (&hist_mapping_col_type eq 1) %then %do;
               if (unique_comm_mapping_id eq .) then do;
                  putlog ' ';
                  putlog "%mo_error Missing value detected for UNIQUE_COMM_MAPPING_ID in observation " _N_ +(-1) " of %upcase(&contact_history_table) table.";
                  put    "input.data.validation.missing.value.error.message.fmt.txt,3,UNIQUE_COMM_MAPPING_ID," _N_ +(-1)",%upcase(&contact_history_table)";
                  putlog ' ';
                  call symput("&_rc",'8');
               end;
            %end;
            %else %do;
               if (unique_comm_mapping_id eq '') then do;
                  putlog ' ';
                  putlog "%mo_error Blank value detected for UNIQUE_COMM_MAPPING_ID in observation " _N_ +(-1) " of %upcase(&contact_history_table) table.";
                  put    "input.data.validation.blank.value.error.message.fmt.txt,3,UNIQUE_COMM_MAPPING_ID," _N_ +(-1)",%upcase(&contact_history_table)";
                  putlog ' ';
                  call symput("&_rc",'8');
               end;
               else do;
                  if (substr(unique_comm_mapping_id,1,1) eq ' ') then do;
                     putlog ' ';
                     putlog "%mo_error UNIQUE_COMM_MAPPING_ID in observation " _N_ +(-1) " of %upcase(&contact_history_table) table must not have leading blanks.";
                     put    "input.data.validation.leading.space.error.message.fmt.txt,3,UNIQUE_COMM_MAPPING_ID," _N_ +(-1)",%upcase(&contact_history_table)";
                     putlog ' ';
                     call symput("&_rc",'8');
                  end;
                  if (anyspace(substr(unique_comm_mapping_id,1,length(unique_comm_mapping_id))) gt 0) then do;
                     putlog ' ';
                     putlog "%mo_error UNIQUE_COMM_MAPPING_ID in observation " _N_ +(-1) " of %upcase(&contact_history_table) table must not have embedded spaces.";
                     put    "input.data.validation.embedded.space.error.message.fmt.txt,3,UNIQUE_COMM_MAPPING_ID," _N_ +(-1)",%upcase(&contact_history_table)";
                     putlog ' ';
                     call symput("&_rc",'8');
                  end;
                  if (index(unique_comm_mapping_id,',') gt 0) then do;
                     putlog ' ';
                     putlog "%mo_error UNIQUE_COMM_MAPPING_ID in observation " _N_ +(-1) " of %upcase(&contact_history_table) table must not contain commas.";
                     put    "input.data.validation.special.char.name.error.message.fmt.txt,3,UNIQUE_COMM_MAPPING_ID," _N_ +(-1)",%upcase(&contact_history_table)";
                     putlog ' ';
                     call symput("&_rc",'8');
                  end;
                  if (length(unique_comm_mapping_id) > 32) then do;
                     putlog ' ';
                     putlog "%mo_error Length of UNIQUE_COMM_MAPPING_ID in observation " _N_ +(-1) " of %upcase(&contact_history_table) table cannot be greater than 32.";
                     put    "input.data.validation.column.length.error.message.fmt.txt,3,UNIQUE_COMM_MAPPING_ID," _N_ +(-1)",%upcase(&contact_history_table)";
                     putlog ' ';
                     call symput("&_rc",'8');
                  end;
               end;
            %end;
         %end;
         
         %else %do;  /* campaign_cd, communication_cd required */
            if (campaign_cd eq '') then do;
               putlog ' ';
               putlog "%mo_error Blank value detected for CAMPAIGN_CD in observation " _N_ +(-1) " of %upcase(&contact_history_table) table.";
               put    "input.data.validation.blank.value.error.message.fmt.txt,3,CAMPAIGN_CD," _N_ +(-1)",%upcase(&contact_history_table)";
               putlog ' ';
               call symput("&_rc",'8');
            end;
            else do;
               if (substr(campaign_cd,1,1) eq ' ') then do;
                  putlog ' ';
                  putlog "%mo_error CAMPAIGN_CD in observation " _N_ +(-1) " of %upcase(&contact_history_table) table must not have leading blanks.";
                  put    "input.data.validation.leading.space.error.message.fmt.txt,3,CAMPAIGN_CD," _N_ +(-1)",%upcase(&contact_history_table)";
                  putlog ' ';
                  call symput("&_rc",'8');
               end;
               if (anyspace(substr(campaign_cd,1,length(campaign_cd))) gt 0) then do;
                  putlog ' ';
                  putlog "%mo_error CAMPAIGN_CD in observation " _N_ +(-1) " of %upcase(&contact_history_table) table must not have embedded spaces.";
                  put    "input.data.validation.embedded.space.error.message.fmt.txt,3,CAMPAIGN_CD," _N_ +(-1)",%upcase(&contact_history_table)";
                  putlog ' ';
                  call symput("&_rc",'8');
               end;
               if (index(campaign_cd,',') gt 0) then do;
                  putlog ' ';
                  putlog "%mo_error CAMPAIGN_CD in observation " _N_ +(-1) " of %upcase(&contact_history_table) table must not contain commas.";
                  put    "input.data.validation.special.char.name.error.message.fmt.txt,3,CAMPAIGN_CD," _N_ +(-1)",%upcase(&contact_history_table)";
                  putlog ' ';
                  call symput("&_rc",'8');
               end;
               if (length(campaign_cd) > 32) then do;
                  putlog ' ';
                  putlog "%mo_error Length of CAMPAIGN_CD in observation " _N_ +(-1) " of %upcase(&contact_history_table) table cannot be greater than 32.";
                  put    "input.data.validation.column.length.error.message.fmt.txt,3,CAMPAIGN_CD," _N_ +(-1)",%upcase(&contact_history_table)";
                  putlog ' ';
                  call symput("&_rc",'8');
               end;
               if (nomatch_campaign_cd = 1) then do;
                  putlog ' ';
                  putlog "%mo_error Invalid CAMPAIGN_CD detected in observation " _N_ +(-1) " of %upcase(&contact_history_table) table.";
                  put    "input.data.validation.invalid.value.error.message.fmt.txt,3,CAMPAIGN_CD," _N_ +(-1)",%upcase(&contact_history_table)";
                  putlog ' ';
                  call symput("&_rc",'8');
               end;
            end;
            if (communication_cd eq '') then do;
               putlog ' ';
               putlog "%mo_error Blank value detected for COMMUNICATION_CD in observation " _N_ +(-1) " of %upcase(&contact_history_table) table.";
               put    "input.data.validation.blank.value.error.message.fmt.txt,3,COMMUNICATION_CD," _N_ +(-1)",%upcase(&contact_history_table)";
               putlog ' ';
               call symput("&_rc",'8');
            end;
            else do;
               if (substr(communication_cd,1,1) eq ' ') then do;
                  putlog ' ';
                  putlog "%mo_error COMMUNICATION_CD in observation " _N_ +(-1) " of %upcase(&contact_history_table) table must not have leading blanks.";
                  put    "input.data.validation.leading.space.error.message.fmt.txt,3,COMMUNICATION_CD," _N_ +(-1)",%upcase(&contact_history_table)";
                  putlog ' ';
                  call symput("&_rc",'8');
               end;
               if (anyspace(substr(communication_cd,1,length(communication_cd))) gt 0) then do;
                  putlog ' ';
                  putlog "%mo_error COMMUNICATION_CD in observation " _N_ +(-1) " of %upcase(&contact_history_table) table must not have embedded spaces.";
                  put    "input.data.validation.embedded.space.error.message.fmt.txt,3,COMMUNICATION_CD," _N_ +(-1)",%upcase(&contact_history_table)";
                  putlog ' ';
                  call symput("&_rc",'8');
               end;
               if (index(communication_cd,',') gt 0) then do;
                  putlog ' ';
                  putlog "%mo_error COMMUNICATION_CD in observation " _N_ +(-1) " of %upcase(&contact_history_table) table must not contain commas.";
                  put    "input.data.validation.special.char.name.error.message.fmt.txt,3,COMMUNICATION_CD," _N_ +(-1)",%upcase(&contact_history_table)";
                  putlog ' ';
                  call symput("&_rc",'8');
               end;
               if (length(communication_cd) > 32) then do;
                  putlog ' ';
                  putlog "%mo_error Length of COMMUNICATION_CD in observation " _N_ +(-1) " of %upcase(&contact_history_table) table cannot be greater than 32.";
                  put    "input.data.validation.column.length.error.message.fmt.txt,3,COMMUNICATION_CD," _N_ +(-1)",%upcase(&contact_history_table)";
                  putlog ' ';
                  call symput("&_rc",'8');
               end;
            end;
         %end;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered creating &contact_history_table (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;

      %* Verify that all mapping values in the Contact History table ;
      %* appear in the Communication History table.                  ;
      proc sql noprint;
         create table work.comm_hist_mapping_values as
         select 
            %if (&hist_mapping_col_required eq 1) %then %do;
               distinct unique_comm_mapping_id
            %end;
            %else %do;
               distinct trim(left(campaign_cd)) || "/" || trim(left(communication_cd)) 
            %end;
            as hist_mapping_value
         from moinput.&communication_history_table;

         create table work.contact_hist_mapping_values as
         select 
            %if (&hist_mapping_col_required eq 1) %then %do;
               distinct unique_comm_mapping_id
            %end;
            %else %do;
               distinct trim(left(campaign_cd)) || "/" || trim(left(communication_cd)) 
            %end;
            as hist_mapping_value
         from moinput.&contact_history_table;
      quit;
      %if (&sqlrc gt 4) %then %do;
         %let &_rc = 107;
         %put %mo_error SQL error encountered obtaining mapping column values from moinput.&communication_history table and/or moinput.&contact_history table (sqlrc=&sqlrc).;
         %goto ERROR_EXIT;
      %end;

      %local mapping_values_not_defined;
      %let mapping_values_not_defined = 0;
      proc sql noprint;
         create table work.mapping_values_not_defined as
            select hist_mapping_value 
            from work.contact_hist_mapping_values as a
            where not exists (select hist_mapping_value
                              from work.comm_hist_mapping_values
                              where (a.hist_mapping_value = hist_mapping_value));

            select count(*) into :mapping_values_not_defined
            from work.mapping_values_not_defined;
      quit;
      %if (&sqlrc gt 4) %then %do;
         %let &_rc = 107;
         %put %mo_error SQL error encountered comparing work.mo_combo_ids_measure with work.mo_combo_ids (sqlrc=&sqlrc).;
         %goto ERROR_EXIT;
      %end;
           
      %if (&mapping_values_not_defined ne 0) %then %do;
         data _null_;
            set work.mapping_values_not_defined;
            &setfile;
            putlog ' ';

            %if (&hist_mapping_col_required eq 1) %then %do;
               putlog ' ';
               putlog "%mo_error The mapping ID value " hist_mapping_value " in the %upcase(&contact_history_table) table is not defined in the %upcase(&communication_history_table) table.";
               put    "input.data.validation.contactHist.mapping.not.defined.error.message.fmt.txt,3,mapping ID value " hist_mapping_value ",%upcase(&contact_history_table),%upcase(communication_history_table)";
            %end;
            %else %do;
               putlog ' ';
               putlog "%mo_error The mapping combination CAMPAIGN_CD / COMMUNICATION_CD " hist_mapping_value " in the %upcase(&contact_history_table) table is not defined in the %upcase(&communication_history_table) table.";
               put    "input.data.validation.contactHist.mapping.not.defined.error.message.fmt.txt,3,mapping combination CAMPAIGN_CD / COMMUNICATION_CD " hist_mapping_value ",%upcase(&contact_history_table),%upcase(communication_history_table)";
            %end;
            putlog ' ';
            call symput("&_rc",'8');
         run;
      %end;

      %* Check that the Customer ID column(s) type(s) in the Contact ;
      %* History table match the types of the corresponding Customer ;
      %* ID column(s) in the Customer table.                         ;
      data work.compare_types;
         merge work.customer_contents_custIDCols (rename=(type=custType))
               work.history_contents_custIDCols (rename=(type=contType));
         by name;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered creating work.compare_types (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;

      data work.compare_types;
         set work.compare_types;
         &setfile;
         if (custType ne contType) then do;
            putlog ' ';
            putlog "%mo_error Customer ID column has different data types in the %upcase(&customer_table) table and the %upcase(&contact_history_table) table: " name;
            put "input.data.validation.contactHist.type.mismatch.error.message.fmt.txt,3," name ",%upcase(&customer_table),%upcase(&contact_history_table)";
            putlog ' ';
            call symput("&_rc",'8');
         end;
      run;
   %end;
   

   %*-------------------------------------------------------------------------;
   %* Check Communication, Campaign, Channel, and Time Periods tables for     ;
   %* reserved column names, extra character columns that are too long, and   ;
   %* names of extra numeric columns that are too long.                       ;
   %*-------------------------------------------------------------------------;

   %*;
   %* Check for reserved column names, extra character columns that are too ;
   %* long, and names of extra numeric columns that are too long in the     ;
   %* communication table.                                                  ;
   %*;
   proc contents data=work.communication_orig_mo out=work.comm_contents_orig (keep=name type length) noprint;
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered obtaining contents of work.&communication_table (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;

   proc contents data=work.&communication_table out=work.comm_contents_actual (keep=name type length) noprint;
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered obtaining contents of moinput.&communication_table (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;

   proc sort data=work.comm_contents_orig;
      by name;
   run;

   proc sort data=work.comm_contents_actual;
      by name;
   run;

   data work.communication_extra_cols;
      merge work.comm_contents_actual (in=a) work.comm_contents_orig (in=b);
      by name;
      if (a and not b) and (lowcase(name) ne _mo_column_sort_order);
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered creating communication_extra_cols (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;

   data _null_;
      set work.communication_extra_cols;
      &setfile;
      if (lowcase(strip(name)) in &communication_reserved) then do;
         name = upcase(name);
         putlog ' ';
         putlog "%mo_error The column named " name "in the %upcase(&communication_table) table is not allowed.  It is a SAS Marketing Optimization reserved name.";
         put    "input.data.validation.reserved.column.error.message.fmt.txt,2," name ",%upcase(&communication_table)";
         putlog ' ';
         call symput("&_rc",'8');
      end;
      else if (type eq 2 and length gt 256) then do;
         name=upcase(name);
         putlog ' ';
         putlog "%mo_error Character attribute column is defined with length greater than 256 in the %upcase(&communication_table) table: " name ".  The maximum supported length is 256 characters.";
         put    "input.data.validation.long.character.attribute.column.error.message.fmt.txt,2,%upcase(&communication_table)," name;
         putlog ' ';
         call symput("&_rc", '8');
      end;
      else if (type eq 1 and length(name) gt 26) then do;
         name=upcase(name);
         putlog ' ';
         putlog "%mo_error Numeric column name has a length greater than 26 in the %upcase(&communication_table) table: " name ".  The maximum supported length is 26 characters.";
         put    "input.data.validation.invalid.numeric_column_name.length.error.message.fmt.txt,3," name ",%upcase(&communication_table),26";
         putlog ' ';
         call symput("&_rc", '8');
      end;
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered reading work.communication_extra_cols (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;

   %*;
   %* Check for reserved column names, extra character columns that are too ;
   %* long, and names of extra numeric columns that are too long in the     ;
   %* campaign table.                                                       ;
   %*;
   proc contents data=work.&campaign_table out=work.camp_contents_actual (keep=name type length) noprint;
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered obtaining contents of moinput.&campaign_table (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;

   %let num_extra_camp_columns = 0;
   %let list_extra_camp_columns = ;
   proc sql noprint;
      create table work.camp_contents_extra_cols as
         select * from work.camp_contents_actual
         where upcase(name) not in 
            (select upcase(name) from work.comm_contents_actual);
      select count(*), 'b.'||trim(left(name))
         into :num_extra_camp_columns, :list_extra_camp_columns separated by ','
         from work.camp_contents_extra_cols;
   quit;
   %if (&sqlrc gt 4) %then %do;
      %let &_rc = 107;
      %put %mo_error SQL error encountered creating work.camp_contents_extra_cols (sqlrc=&sqlrc).;
      %goto ERROR_EXIT;
   %end;

   data _null_;
      set work.camp_contents_extra_cols;
      &setfile;
      if (lowcase(strip(name)) in &communication_reserved) then do;
         name = upcase(name);
         putlog ' ';
         putlog "%mo_error The column named " name "in the %upcase(&campaign_table) table is not allowed.  It is a SAS Marketing Optimization reserved name.";
         put    "input.data.validation.reserved.column.error.message.fmt.txt,2," name ",%upcase(&campaign_table)";
         putlog ' ';
         call symput("&_rc",'8');
      end;
      else if (type eq 2 and length gt 256) then do;
         name=upcase(name);
         putlog ' ';
         putlog "%mo_error Character attribute column is defined with length greater than 256 in the %upcase(&campaign_table) table: " name ".  The maximum supported length is 256 characters.";
         put    "input.data.validation.long.character.attribute.column.error.message.fmt.txt,2,%upcase(&campaign_table)," name;
         putlog ' ';
         call symput("&_rc", '8');
      end;
      else if (type eq 1 and length(name) gt 26) then do;
         name=upcase(name);
         putlog ' ';
         putlog "%mo_error Numeric column name has a length greater than 26 in the %upcase(&campaign_table) table: " name ".  The maximum supported length is 26 characters.";
         put    "input.data.validation.invalid.numeric_column_name.length.error.message.fmt.txt,3," name ",%upcase(&campaign_table),26";
         putlog ' ';
         call symput("&_rc", '8');
      end;
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered reading work.camp_contents_extra_cols (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;


   %*;
   %* Check for reserved column names, extra character columns that are too ;
   %* long, and names of extra numeric columns that are too long in the     ;
   %* channel table.                                                        ;
   %*;
   %let num_extra_channel_columns = 0;
   %let list_extra_channel_columns = ;
   %if (%bquote(&channel_table) ne %str()) %then %do;
      proc contents data=work.&channel_table out=work.channel_contents_actual (keep=name type length) noprint;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered obtaining contents of moinput.&channel_table (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;

      proc sql noprint;
         create table work.channel_contents_extra_cols as
            select * from work.channel_contents_actual
            where upcase(name) not in 
               (select upcase(name) from work.comm_contents_actual)
            and upcase(name) not in
               (select upcase(name) from work.camp_contents_extra_cols);
         select count(*), 'b.'||trim(left(name))
            into :num_extra_channel_columns, :list_extra_channel_columns separated by ','
            from work.channel_contents_extra_cols;
      quit;
      %if (&sqlrc gt 4) %then %do;
         %let &_rc = 107;
         %put %mo_error SQL error encountered creating work.channel_contents_extra_cols (sqlrc=&sqlrc).;
         %goto ERROR_EXIT;
      %end;

      data _null_;
         set work.channel_contents_extra_cols;
         &setfile;
         if (lowcase(strip(name)) in &communication_reserved) then do;
            name = upcase(name);
            putlog ' ';
            putlog "%mo_error The column named " name "in the %upcase(&channel_table) table is not allowed.  It is a SAS Marketing Optimization reserved name.";
            put    "input.data.validation.reserved.column.error.message.fmt.txt,2," name ",%upcase(&channel_table)";
            putlog ' ';
            call symput("&_rc",'8');
         end;
         else if (type eq 2 and length gt 256) then do;
            name=upcase(name);
            putlog ' ';
            putlog "%mo_error Character attribute column is defined with length greater than 256 in the %upcase(&channel_table) table: " name ".  The maximum supported length is 256 characters.";
            put    "input.data.validation.long.character.attribute.column.error.message.fmt.txt,2,%upcase(&channel_table)," name;
            putlog ' ';
            call symput("&_rc", '8');
         end;
         else if (type eq 1 and length(name) gt 26) then do;
            name=upcase(name);
            putlog ' ';
            putlog "%mo_error Numeric column name has a length greater than 26 in the %upcase(&channel_table) table: " name ".  The maximum supported length is 26 characters.";
            put    "input.data.validation.invalid.numeric_column_name.length.error.message.fmt.txt,3," name ",%upcase(&channel_table),26";
            putlog ' ';
            call symput("&_rc", '8');
         end;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered reading work.channel_contents_extra_cols (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;
   %end;


   %*;
   %* Check for reserved column names, extra character columns that are too ;
   %* long, and names of extra numeric columns that are too long in the     ;
   %* time period table.                                                    ;
   %*;
   %let num_extra_time_period_columns = 0;
   %let list_extra_time_period_columns = ;
   %if (%bquote(&time_period_table) ne %str()) %then %do;

      proc contents data=work.&time_period_table out=work.time_period_contents_actual (keep=name type length) noprint;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered obtaining contents of moinput.&time_period_table (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;

      proc sql noprint;
         create table work.time_period_contents_extra_cols as
            select * from work.time_period_contents_actual
            where upcase(name) ne 'ORDER_NO'
            and upcase(name) not in 
               (select upcase(name) from work.comm_contents_actual)
            and upcase(name) not in
               (select upcase(name) from work.camp_contents_extra_cols)
            %if (%bquote(&channel_table) ne %str()) %then %do;
               and upcase(name) not in
                  (select upcase(name) from work.channel_contents_extra_cols)
            %end;
            ;
         select count(*), 'b.'||trim(left(name))
            into :num_extra_time_period_columns, :list_extra_time_period_columns separated by ','
            from work.time_period_contents_extra_cols;
      quit;
      %if (&sqlrc gt 4) %then %do;
         %let &_rc = 107;
         %put %mo_error SQL error encountered creating work.time_period_contents_extra_cols (sqlrc=&sqlrc).;
         %goto ERROR_EXIT;
      %end;

      data _null_;
         set work.time_period_contents_extra_cols;
         &setfile;
         if (lowcase(strip(name)) in &communication_reserved) then do;
            putlog ' ';
            name = upcase(name);
            putlog "%mo_error The column named " name "in the %upcase(&time_period_table) table is not allowed.  It is a SAS Marketing Optimization reserved name.";
            put    "input.data.validation.reserved.column.error.message.fmt.txt,2," name ",%upcase(&time_period_table)";
            putlog ' ';
            call symput("&_rc",'8');
         end;
         else if (type eq 2 and length gt 256) then do;
            name=upcase(name);
            putlog ' ';
            putlog "%mo_error Character attribute column is defined with length greater than 256 in the %upcase(&time_period_table) table: " name ".  The maximum supported length is 256 characters.";
            put    "input.data.validation.long.character.attribute.column.error.message.fmt.txt,2,%upcase(&time_period_table)," name;
            putlog ' ';
            call symput("&_rc", '8');
         end;
         else if (type eq 1 and length(name) gt 26) then do;
            name=upcase(name);
            putlog ' ';
            putlog "%mo_error Numeric column name has a length greater than 26 in the %upcase(&time_period_table) table: " name ".  The maximum supported length is 26 characters.";
            put    "input.data.validation.invalid.numeric_column_name.length.error.message.fmt.txt,3," name ",%upcase(&time_period_table),26";
            putlog ' ';
            call symput("&_rc", '8');
         end;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered reading work.time_period_contents_extra_cols (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;
   %end;


   %*---------------------------------------------;
   %* Join the campaign, channel, and time_period ;
   %* tables with the communication table.        ;
   %*---------------------------------------------;
   %let working_comm_table = &communication_table;
   %if &num_extra_camp_columns > 0 %then %do;
      proc sql noprint;
         create table work.comm_table_camp as
            select a.*, &list_extra_camp_columns
            from work.&working_comm_table as a left join work.&campaign_table as b
            on (a.campaign_cd=b.campaign_cd);
      quit;
      %if (&sqlrc gt 4) %then %do;
         %let &_rc = 107;
         %put %mo_error SQL error encountered creating work.comm_table_camp (sqlrc=&sqlrc).;
         %goto ERROR_EXIT;
      %end;
      %let working_comm_table = comm_table_camp;
   %end;
   %if &num_extra_channel_columns > 0 %then %do;
      proc sql noprint;
         create table work.comm_table_channel as
            select a.*, &list_extra_channel_columns
            from work.&working_comm_table as a left join work.&channel_table as b
            on (a.channel_cd=b.channel_cd);
      quit;
      %if (&sqlrc gt 4) %then %do;
         %let &_rc = 107;
         %put %mo_error SQL error encountered creating work.comm_table_channel (sqlrc=&sqlrc).;
         %goto ERROR_EXIT;
      %end;
      %let working_comm_table = comm_table_channel;
   %end;
   %if &num_extra_time_period_columns > 0 %then %do;
      proc sql noprint;
         create table work.comm_table_time_period as
            select a.*, &list_extra_time_period_columns
            from work.&working_comm_table as a left join work.&time_period_table as b
            on (a.time_period=b.time_period);
      quit;
      %if (&sqlrc gt 4) %then %do;
         %let &_rc = 107;
         %put %mo_error SQL error encountered creating work.comm_table_time_period (sqlrc=&sqlrc).;
         %goto ERROR_EXIT;
      %end;
      %let working_comm_table = comm_table_time_period;
   %end;

   %* Code warning:                                                            ;
   %* work.comm_table_joined_others used in mo_prepopulate_metadata_tables.sas ;
   data work.&communication_table work.comm_table_joined_others;
      set work.&working_comm_table;
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered reading work.working_comm_table (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;

   proc contents data=work.&communication_table out=work.comm_contents_actual (keep=name type length) noprint;
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered obtaining contents of moinput.&communication_table (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;

   %* S0714454                                                             ;
   %* Augment work.comm_contents_actual with indication of whether the     ;
   %* column with the specified name has all missing values in the         ;
   %* work.&communication_table. The reason for doing this is as follows.  ;
   %*                                                                      ;
   %* Communication table columns that have all values missing will be     ;
   %* excluded from the datastore tables comm_numeric_attributes or        ;
   %* comm_character_attributes (as appropriate) by mo_prepopulate_        ;
   %* metadata_tables.sas, called subsequent to this macro. In that case,  ;
   %* if the opt engine were to attempt to work with communication filter  ;
   %* text that references such an excluded column, the opt engine         ;
   %* processing will crash because the column does not in the datastore   ;
   %* table. To avoid this, communication filter validation should flag as ;
   %* invalid any filter that references a communication numeric attribute ;
   %* character attribute column that has all missing values since that    ;
   %* attribute will not be in the datastore.                              ;
   data work.comm_contents_actual;
      set work.comm_contents_actual;
      flagFilterAsInvalid = 0;
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered adding flagFilterAsInvalid column to row &i of work.comm_contents_actual (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;

   %local num_comm_tbl_rows num_comm_tbl_columns;
   %let num_comm_tbl_rows = 0;
   %let num_comm_tbl_columns = 0;
   proc sql noprint;
      select count(*) into :num_comm_tbl_rows from work.&communication_table;
      select count(*) into :num_comm_tbl_columns from work.comm_contents_actual;
   quit;
   %if (&sqlrc gt 4) %then %do;
      %let &_rc = 107;
      %put %mo_error SQL error encountered counting rows in %upcase(work.&communication_table) or counting columns in %upcase(work.comm_contents_actual) (sqlrc=&sqlrc).;
      %goto ERROR_EXIT;
   %end;

   %do i=1 %to &num_comm_tbl_columns;
      %local colName_&i;
      data _null_;
         index = &i; 
         set work.comm_contents_actual point=index;
         call symput("colName_&i", name);
         stop;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered obtaining column name from row &i of work.comm_contents_actual (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;
      %let colName_&i = %trim(%left(&&colName_&i));

      %local flagFilterAsInvalid_&i;
      %let flagFilterAsInvalid_&i = 0;
      proc sql noprint;
         select (sum(missing(&&colName_&i)) = &num_comm_tbl_rows) as allMissing
         into :flagFilterAsInvalid_&i 
         from work.&communication_table; 
      quit;
      %if (&sqlrc gt 4) %then %do;
         %let &_rc = 107;
         %put %mo_error SQL error encountered counting missing values for column %upcase(&&colName_&i) in %upcase(work.&communication_table) (sqlrc=&sqlrc).;
         %goto ERROR_EXIT;
      %end;
      %let flagFilterAsInvalid_&i = %trim(%left(&&flagFilterAsInvalid_&i));

      %* Reset flagFilterAsInvalid_&i for special case of ;
      %* the priority column when prioritization in use. It    ;
      %* will not be excluded from the datastore even when all ;
      %* values are missing.                                   ;
      %if ((&uses_prioritization eq 1) and (%upcase(&&colName_&i) eq %upcase(&priority_column))) %then %do;
         %let flagFilterAsInvalid_&i = 0;
      %end;

      %* Set flagFilterAsInvalid_&i for special case of   ;
      %* mandatory-related columns. They will be excluded from ;
      %* the datastore regardless of whether all values are    ;
      %* missing or not when mandatory exclusions not in use.  ;
      %if ((&mandatory_cols_required eq 0) and 
           ((%upcase(&&colName_&i) eq MANDATORY) or 
            (%upcase(&&colName_&i) eq EXCLUDE_FROM_CONSTRAINTS) or 
            (%upcase(&&colName_&i) eq EXCLUDE_FROM_CONTACT_POLS))) %then %do;
         %let flagFilterAsInvalid_&i = 1;
      %end;

      %if (&&flagFilterAsInvalid_&i eq 1) %then %do;
         data work.comm_contents_actual;
            set work.comm_contents_actual;
            if (name = "&&colName_&i") then do;
               flagFilterAsInvalid = &&flagFilterAsInvalid_&i;
            end;
         run;
         %if (&syserr gt 4) %then %do;
            %let &_rc = 100;
            %put %mo_error Error encountered setting flagFilterAsInvalid column to value &flagFilterAsInvalid_&i for column name %upcase(&&colName_&i) in row &i of work.comm_contents_actual (syserr=&syserr).;
            %goto ERROR_EXIT;
         %end;
      %end;
   %end;


   %*--------------------------------------------------------;
   %* Check CUSTOMER and CUSTOMER_COLUMN_METADATA table data ;
   %*--------------------------------------------------------;
   %CUST:

   %*;
   %* If this is prioritization-based input data, verify that the specified ;
   %* customer rank measure exists in the Customer Control table as a value ;
   %* in the numeric_measure column.                                        ;
   %*;
   %if (&uses_prioritization eq 1) %then %do;
      %local rankMeasurePresent;
      %let rankMeasurePresent = 0;
      proc sql noprint;
            select count(*) into :rankMeasurePresent
         from moinput.&cust_col_metadata_table
         where (upcase(numeric_measure) = upcase("&customer_rank_measure"));
      quit;
      %if (&sqlrc gt 4) %then %do;
         %let &_rc = 107;
         %put %mo_error SQL error encountered obtaining numeric_measure column value %upcase(&customer_rank_measure) from %upcase(moinput.&cust_col_metadata_table) (sqlrc=&sqlrc).;
         %goto ERROR_EXIT;
      %end;

      %if (&rankMeasurePresent eq 0) %then %do;
         data _null_;
            &setfile;
            putlog ' ';
            putlog "%mo_error Customer Rank numeric_measure column value %upcase(&customer_rank_measure) not found in %upcase(&cust_col_metadata_table) table.";
            put "input.data.validation.missing.prioritization.numeric_measure.error.message.fmt.txt,2,%upcase(&customer_rank_measure),%upcase(&cust_col_metadata_table)";
            putlog ' ';
            call symput("&_rc",'8');
         run;
      %end;
   %end;

   data work.cust_col_metadata_in_mo_tbl;
      set work.cust_col_metadata_in_mo_tbl;
      nomatch_column_nm = 0;
      nomatch_comm_cd = 0;
      nomatch_cell_cd = 0;
      nomatch_campaign_cd = 0;
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered creating nomatch columns in work.cust_col_metadata_in_mo_tbl (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;

   proc sql noprint;
      update work.cust_col_metadata_in_mo_tbl
         set nomatch_column_nm = 1 
         where lowcase(column_nm) not in (select lowcase(name) from work.customer_contents);

      update work.cust_col_metadata_in_mo_tbl
         set nomatch_comm_cd = 1
         where communication_cd not in 
            (select communication_cd from work.distinct_communication_cds);

      update work.cust_col_metadata_in_mo_tbl
         set nomatch_campaign_cd = 1
         where campaign_cd not in 
            (select campaign_cd from work.distinct_campaign_cds);
   quit;
   %if (&sqlrc gt 4) %then %do;
      %let &_rc = 107;
      %put %mo_error SQL error encountered updating nomatch columns in work.cust_col_metadata_in_mo_tbl (sqlrc=&sqlrc).;
      %goto ERROR_EXIT;
   %end;

   %*;
   %* Determine if the optional CELL_CD column is present;
   %*;
   %local cell_cd_in_ctrl_table;
   %let cell_cd_in_ctrl_table = 0;

   %local dsid cell_cd_col_varnum;  
   %let cell_cd_col_varnum = 0;
   %let dsid = %sysfunc(open(moinput.&cust_col_metadata_table));
   %if (&dsid = 0) %then %do;
      data _null_;
         &setfile;
         putlog ' ';
         %putlog "%mo_error The specified &cust_col_metadata_tabletable does not exist in MOINPUT.";
         put "input.data.validation.table.not.exist.error.message.fmt.txt,1,&cust_col_metadata_table";            
         call symput("&_rc",'8');
      run;
      %goto ERROR_EXIT;
   %end;
   %let cell_cd_col_varnum = %sysfunc(varnum(&dsid, cell_cd));
   %if (&cell_cd_col_varnum ne 0) %then %do;
      %let cell_cd_in_ctrl_table = 1;
   %end;
   %let dsid = %sysfunc(close(&dsid));

   %if &cell_cd_in_ctrl_table %then %do;
      proc sql noprint;
         update work.cust_col_metadata_in_mo_tbl
            set nomatch_cell_cd = 1 
            where cell_cd not in 
               (select compbl(trim(left(cell_cd))) from work.communication_in_mo_tbl);
      quit;
      %if (&sqlrc gt 4) %then %do;
         %let &_rc = 107;
         %put %mo_error SQL error encountered updating nomatch_cell_cd column (sqlrc=&sqlrc).;
         %goto ERROR_EXIT;
      %end;
   %end;

   %*;
   %* Do basic CUSTOMER_COLUMN_METADATA data validation;
   %*;
   data work.&cust_col_metadata_table (drop=nomatch_column_nm nomatch_comm_cd nomatch_cell_cd nomatch_campaign_cd);
      set work.cust_col_metadata_in_mo_tbl;
      &setfile;

      column_nm=lowcase(compbl(trim(left(column_nm))));
      campaign_cd=trim(left(campaign_cd));
      communication_cd=trim(left(communication_cd));
      %if (&cell_cd_in_ctrl_table eq 1) %then %do;
         cell_cd=trim(left(cell_cd));
      %end;

      if (column_nm eq '') then do;
         putlog ' ';
         putlog "%mo_error Blank value detected for COLUMN_NM in observation " _N_ +(-1) " of %upcase(&cust_col_metadata_table) table.";
         put    "input.data.validation.blank.value.error.message.fmt.txt,3,COLUMN_NM," _N_ +(-1)",%upcase(&cust_col_metadata_table)";
         putlog ' ';
         call symput("&_rc",'8');
      end;
      if (nomatch_column_nm = 1) then do;
         putlog ' ';
         putlog "%mo_error COLUMN_NM in observation " _N_ +(-1) " of %upcase(&cust_col_metadata_table) table does not exist in %upcase(&customer_table).";
         put    "input.data.validation.missing.column_name.error.message.fmt.txt,3," _N_ +(-1)",%upcase(&cust_col_metadata_table),%upcase(&customer_table)";
         putlog ' ';
         call symput("&_rc",'8');
      end;
      if (length(column_nm) > 32) then do;
         putlog ' ';
         putlog "%mo_error Length of COLUMN_NM in observation " _N_ +(-1) " of %upcase(&cust_col_metadata_table) table cannot be greater than 32.";
         put    "input.data.validation.column.length.error.message.fmt.txt,3,COLUMN_NM," _N_ +(-1)",%upcase(&cust_col_metadata_table)";
         putlog ' ';
         call symput("&_rc",'8');
      end;
      if (lowcase(strip(numeric_measure)) in &customer_control_reserved) then do;
         putlog ' ';
         putlog "%mo_error Value of NUMERIC_MEASURE in observation " _N_ +(-1) " of %upcase(&cust_col_metadata_table) is not allowed: " numeric_measure "  is a SAS Marketing Optimization reserved name.";
         put    "input.data.validation.reserved.name.error.message.fmt.txt,4,NUMERIC_MEASURE," _N_ +(-1)",%upcase(&cust_col_metadata_table)," numeric_measure;
         putlog ' ';
         call symput("&_rc",'8');
      end;
      if (numeric_measure eq '') then do;
         putlog ' ';
         putlog "%mo_error Blank value detected for NUMERIC_MEASURE in observation " _N_ +(-1) " of %upcase(&cust_col_metadata_table) table.";
         put    "input.data.validation.blank.value.error.message.fmt.txt,3,NUMERIC_MEASURE," _N_ +(-1)",%upcase(&cust_col_metadata_table)";
         putlog ' ';
         call symput("&_rc",'8');
      end;
      else do;
         if (nvalid(numeric_measure) ne 1) then do;
            putlog ' ';
            putlog "%mo_error Value of NUMERIC_MEASURE in observation " _N_ +(-1) " of %upcase(&cust_col_metadata_table) table must be a valid SAS column name.";
            put    "input.data.validation.invalid.numeric_field.value.error.message.fmt.txt,2," _N_ +(-1)",%upcase(&cust_col_metadata_table)";
            putlog ' ';
            call symput("&_rc",'8');
         end;
         if (length(trim(left(numeric_measure))) gt 26) then do;
            putlog ' ';
            putlog "%mo_error Value of NUMERIC_MEASURE in observation " _N_ +(-1) " of %upcase(&cust_col_metadata_table) table is too long.  Maximum length is 26 characters.";
            put    "input.data.validation.invalid.numeric_field.length.error.message.fmt.txt,2," _N_ +(-1)",%upcase(&cust_col_metadata_table)";
            putlog ' ';
            call symput("&_rc",'8');
         end;
      end;
      if (campaign_cd eq '') then do;
         putlog ' ';
         putlog "%mo_error Blank value detected for CAMPAIGN_CD in observation " _N_ +(-1) " of %upcase(&cust_col_metadata_table) table.";
         put    "input.data.validation.blank.value.error.message.fmt.txt,3,CAMPAIGN_CD," _N_ +(-1)",%upcase(&cust_col_metadata_table)";
         putlog ' ';
         call symput("&_rc",'8');
      end;
      else do;
         if (substr(campaign_cd,1,1) eq ' ') then do;
            putlog ' ';
            putlog "%mo_error CAMPAIGN_CD in observation " _N_ +(-1) " of %upcase(&cust_col_metadata_table) table must not have leading blanks.";
            put    "input.data.validation.leading.space.error.message.fmt.txt,3,CAMPAIGN_CD," _N_ +(-1)",%upcase(&cust_col_metadata_table)";
            putlog ' ';
            call symput("&_rc",'8');
         end;
         if (anyspace(substr(campaign_cd,1,length(campaign_cd))) gt 0) then do;
            putlog ' ';
            putlog "%mo_error CAMPAIGN_CD in observation " _N_ +(-1) " of %upcase(&cust_col_metadata_table) table must not have embedded spaces.";
            put    "input.data.validation.embedded.space.error.message.fmt.txt,3,CAMPAIGN_CD," _N_ +(-1)",%upcase(&cust_col_metadata_table)";
            putlog ' ';
            call symput("&_rc",'8');
         end;
         if (length(campaign_cd) > 32) then do;
            putlog ' ';
            putlog "%mo_error Length of CAMPAIGN_CD in observation " _N_ +(-1) " of %upcase(&cust_col_metadata_table) table cannot be greater than 32.";
            put    "input.data.validation.column.length.error.message.fmt.txt,3,CAMPAIGN_CD," _N_ +(-1)",%upcase(&cust_col_metadata_table)";
            putlog ' ';
            call symput("&_rc",'8');
         end;
         if (nomatch_campaign_cd = 1) then do;
            putlog ' ';
            putlog "%mo_error Invalid CAMPAIGN_CD detected in observation " _N_ +(-1) " of %upcase(&cust_col_metadata_table) table.";
            put    "input.data.validation.invalid.value.error.message.fmt.txt,3,CAMPAIGN_CD," _N_ +(-1)",%upcase(&cust_col_metadata_table)";
            putlog ' ';
            call symput("&_rc",'8');
         end;
      end;
      if (communication_cd eq '') then do;
         putlog ' ';
         putlog "%mo_error Blank value detected for COMMUNICATION_CD in observation " _N_ +(-1) " of %upcase(&cust_col_metadata_table) table.";
         put    "input.data.validation.blank.value.error.message.fmt.txt,3,COMMUNICATION_CD," _N_ +(-1)",%upcase(&cust_col_metadata_table)";
         putlog ' ';
         call symput("&_rc",'8');
      end;
      else do;
         if (substr(communication_cd,1,1) eq ' ') then do;
            putlog ' ';
            putlog "%mo_error COMMUNICATION_CD in observation " _N_ +(-1) " of %upcase(&cust_col_metadata_table) table must not have leading blanks.";
            put    "input.data.validation.leading.space.error.message.fmt.txt,3,COMMUNICATION_CD," _N_ +(-1)",%upcase(&cust_col_metadata_table)";
            putlog ' ';
            call symput("&_rc",'8');
         end;
         if (anyspace(substr(communication_cd,1,length(communication_cd))) gt 0) then do;
            putlog ' ';
            putlog "%mo_error COMMUNICATION_CD in observation " _N_ +(-1) " of %upcase(&cust_col_metadata_table) table must not have embedded spaces.";
            put    "input.data.validation.embedded.space.error.message.fmt.txt,3,COMMUNICATION_CD," _N_ +(-1)",%upcase(&cust_col_metadata_table)";
            putlog ' ';
            call symput("&_rc",'8');
         end;
         if (length(communication_cd) > 32) then do;
            putlog ' ';
            putlog "%mo_error Length of COMMUNICATION_CD in observation " _N_ +(-1) " of %upcase(&cust_col_metadata_table) table cannot be greater than 32.";
            put    "input.data.validation.column.length.error.message.fmt.txt,3,COMMUNICATION_CD," _N_ +(-1)",%upcase(&cust_col_metadata_table)";
            putlog ' ';
            call symput("&_rc",'8');
         end;
         if (nomatch_comm_cd = 1) then do;
            putlog ' ';
            putlog "%mo_error Invalid COMMUNICATION_CD detected in observation " _N_ +(-1) " of %upcase(&cust_col_metadata_table) table.";
            put    "input.data.validation.invalid.value.error.message.fmt.txt,3,COMMUNICATION_CD," _N_ +(-1)",%upcase(&cust_col_metadata_table)";
            putlog ' ';
            call symput("&_rc",'8');
         end;
      end;
      %if (&cell_cd_in_ctrl_table eq 1) %then %do;
         if (cell_cd eq '') then do;
            putlog ' ';
            putlog "%mo_error Blank value detected for CELL_CD in observation " _N_ +(-1) " of %upcase(&cust_col_metadata_table) table.";
            put    "input.data.validation.blank.value.error.message.fmt.txt,3,CELL_CD," _N_ +(-1)",%upcase(&cust_col_metadata_table)";
            putlog ' ';
            call symput("&_rc",'8');
         end;
         else do;
            if (substr(cell_cd,1,1) eq ' ') then do;
               putlog ' ';
               putlog "%mo_error CELL_CD in observation " _N_ +(-1) " of %upcase(&cust_col_metadata_table) table must not have leading blanks.";
               put    "input.data.validation.leading.space.error.message.fmt.txt,3,CELL_CD," _N_ +(-1)",%upcase(&cust_col_metadata_table)";
               putlog ' ';
               call symput("&_rc",'8');
            end;
            if (anyspace(substr(cell_cd,1,length(cell_cd))) gt 0) then do;
               putlog ' ';
               putlog "%mo_error CELL_CD in observation " _N_ +(-1) " of %upcase(&cust_col_metadata_table) table must not have embedded spaces.";
               put    "input.data.validation.embedded.space.error.message.fmt.txt,3,CELL_CD," _N_ +(-1)",%upcase(&cust_col_metadata_table)";
               putlog ' ';
               call symput("&_rc",'8');
            end;
            if (length(cell_cd) > 32) then do;
               putlog ' ';
               putlog "%mo_error Length of CELL_CD in observation " _N_ +(-1) " of %upcase(&cust_col_metadata_table) table cannot be greater than 32.";
               put    "input.data.validation.column.length.error.message.fmt.txt,3,CELL_CD," _N_ +(-1)",%upcase(&cust_col_metadata_table)";
               putlog ' ';
               call symput("&_rc",'8');
            end;
            if (nomatch_cell_cd = 1) then do;
               putlog ' ';
               putlog "%mo_error Invalid CELL_CD detected in observation " _N_ +(-1) " of %upcase(&cust_col_metadata_table) table.";
               put    "input.data.validation.invalid.value.error.message.fmt.txt,3,CELL_CD," _N_ +(-1)",%upcase(&cust_col_metadata_table)";
               putlog ' ';
               call symput("&_rc",'8');
            end;
         end;
      %end;
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered creating work.&cust_col_metadata_table (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;

   %*;
   %* Check for valid CAMPAIGN_CD/COMMUNICATION_CD combinations, or   ;
   %* if the optional CELL_CD column is present, check for valid      ;
   %* CAMPAIGN_CD/COMMUNICATION_CD/CELL_CD combinations.              ;
   %*;
   %if ((&cell_cd_in_ctrl_table eq 1) and 
        (&&&_cell_cd_indicator eq &_CELL_CD_NOT_PRESENT_)) %then %do;
      data _null_;
         &setfile;
         putlog ' ';
         putlog "%mo_error The %upcase(&cust_col_metadata_table) table has a CELL_CD column, but the %upcase(&communication_table) table does not have a CELL_CD column.";
         put "input.data.validation.missing.column.error.message.fmt.txt,2,%upcase(communication_table),CELL_CD";
         putlog ' ';
         call symput("&_rc",'8');
      run;
   %end; 

   proc sort data=work.&communication_table out=work.communication_table_mo_sort nodupkey;
      by campaign_cd communication_cd 
     %if ((&&&_cell_cd_indicator ne &_CELL_CD_NOT_PRESENT_) and 
          (&cell_cd_in_ctrl_table eq 1)) %then %do;
         cell_cd
      %end;
      ;
      where ((campaign_cd ne '') 
             and (communication_cd ne '')
      %if ((&&&_cell_cd_indicator ne &_CELL_CD_NOT_PRESENT_) and 
           (&cell_cd_in_ctrl_table eq 1)) %then %do;
             and (cell_cd ne '')
      %end;
            );
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered sorting work.&communication_table (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;
   
   proc sort data=work.&cust_col_metadata_table out=work.cust_col_metadata_table_mo_sort;
      by campaign_cd communication_cd
      %if ((&&&_cell_cd_indicator ne &_CELL_CD_NOT_PRESENT_) and 
           (&cell_cd_in_ctrl_table eq 1)) %then %do;
         cell_cd
      %end;
      ;
      where ((campaign_cd ne '') 
             and (communication_cd ne '')
      %if ((&&&_cell_cd_indicator ne &_CELL_CD_NOT_PRESENT_) and 
           (&cell_cd_in_ctrl_table eq 1)) %then %do;
            and (cell_cd ne '')
      %end;
            );
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered sorting work.cust_col_metadata_table (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;
   
   data work.compare;
      length from $8.;
      merge work.cust_col_metadata_table_mo_sort (in=a) work.communication_table_mo_sort (in=b);
      by campaign_cd communication_cd
      %if ((&&&_cell_cd_indicator ne &_CELL_CD_NOT_PRESENT_) and 
           (&cell_cd_in_ctrl_table eq 1)) %then %do;
         cell_cd
      %end;
      ;
      if (a and b) then from = 'both';
      else if (a) then from = 'cntl';
      else from='comm';
      keep campaign_cd communication_cd from
      %if ((&&&_cell_cd_indicator ne &_CELL_CD_NOT_PRESENT_) and 
           (&cell_cd_in_ctrl_table eq 1)) %then %do;
          cell_cd
      %end;
      ;
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered creating work.compare (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;
   
   data _null_;
      set work.compare;
      &setfile;
      if (from eq 'cntl') then do;
         putlog ' ';
         %if ((&&&_cell_cd_indicator ne &_CELL_CD_NOT_PRESENT_) and 
              (&cell_cd_in_ctrl_table eq 1)) %then %do;
            putlog ' ';
            putlog "%mo_error Invalid CAMPAIGN_CD/COMMUNICATION_CD/CELL_CD combination detected in %upcase(&cust_col_metadata_table) table where CAMPAIGN_CD = " campaign_cd " and COMMUNICATION_CD = " communication_cd " and CELL_CD = " cell_cd ".";
            put    "input.data.validation.invalid.combination.cell.error.message.fmt.txt,10,CAMPAIGN_CD,COMMUNICATION_CD,CELL_CD,%upcase(&cust_col_metadata_table),CAMPAIGN_CD," campaign_cd +(-1)",COMMUNICATION_CD," communication_cd +(-1)",CELL_CD," cell_cd;
         %end;
         %else %do;
            putlog ' ';
            putlog "%mo_error Invalid CAMPAIGN_CD/COMMUNICATION_CD combination detected in %upcase(&cust_col_metadata_table) table where CAMPAIGN_CD = " campaign_cd " and COMMUNICATION_CD = " communication_cd ".";
            put    "input.data.validation.invalid.combination.error.message.fmt.txt,7,CAMPAIGN_CD,COMMUNICATION_CD,%upcase(&cust_col_metadata_table),CAMPAIGN_CD," campaign_cd +(-1)",COMMUNICATION_CD," communication_cd;
         %end;
         putlog ' ';
         call symput("&_rc",'8');
      end;
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered processing work.compare (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;

   %*;
   %* Check for duplicate NUMERIC_MEASURE/CAMPAIGN_CD/COMMUNICATION_CD ;
   %* combinations.                                                    ;
   %* If duplicate NUMERIC_MEASURE/CAMPAIGN_CD/COMMUNICATION_CD        ;
   %* combinations exist, and the CELL_CD column is present, check for ;
   %* duplicate NUMERIC_MEASURE/CAMPAIGN_CD/COMMUNICATION_CD/CELL_CD   ;
   %* combinations.                                                    ;
   %*;
   %local ctrl_tbl_row_count;
   %local unique_num_camp_comm_count unique_num_camp_comm_cell_count;
   %let ctrl_tbl_row_count = 0;
   %let unique_num_camp_comm_count = 0;
   %let unique_num_camp_comm_cell_count = 0;
   proc sql noprint;
      select count(*)
            ,count(distinct numeric_measure || "/" ||
                            trim(left(campaign_cd)) || "/" ||
                            trim(left(communication_cd)))
         %if (&cell_cd_in_ctrl_table eq 1) %then %do;
            ,count(distinct numeric_measure || "/" ||
                            trim(left(campaign_cd)) || "/" ||
                            trim(left(communication_cd)) || "/" ||
                            trim(left(cell_cd)))
         %end;
      into :ctrl_tbl_row_count
          ,:unique_num_camp_comm_count 
         %if (&cell_cd_in_ctrl_table eq 1) %then %do;
          ,:unique_num_camp_comm_cell_count
         %end;
      from moinput.&cust_col_metadata_table;
   quit;
   %if (&sqlrc gt 4) %then %do;
      %let &_rc = 107;
      %put %mo_error SQL error encountered obtaining information from moinput.&cust_col_metadata_table (sqlrc=&sqlrc).;
      %goto ERROR_EXIT;
   %end;
   
   %*;
   %* If there are duplicate NUMERIC_MEASURE/CAMPAIGN_CD/COMMUNICATION_CD    ;
   %* combos AND the CELL_CD is either not present or, if present, does not  ;
   %* provide uniqueness, identify duplicate combos.                         ;
   %*;
   %if ((&unique_num_camp_comm_count lt &ctrl_tbl_row_count) and
        (&unique_num_camp_comm_cell_count lt &ctrl_tbl_row_count)) %then %do;
      proc summary data=work.&cust_col_metadata_table nway;
         class numeric_measure campaign_cd communication_cd
         %if (&cell_cd_in_ctrl_table eq 1) %then %do;
            cell_cd
         %end;
         ;
         output out=work.cust_col_metadata_table_sum;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered creating work.cust_col_metadata_table_sum (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;
   
      data _null_;
         set work.cust_col_metadata_table_sum;
         &setfile;
         if (_FREQ_ gt 1) then do;
            %if (&cell_cd_in_ctrl_table eq 1) %then %do;
               putlog ' ';
               putlog "%mo_error Duplicate combination detected in %upcase(&cust_col_metadata_table) table where NUMERIC_MEASURE=" numeric_measure ", CAMPAIGN_CD=" campaign_cd ", COMMUNICATION_CD=" communication_cd ", and CELL_CD=" cell_cd ".";
               put    "input.data.validation.custcomm.quadruple.error.message.fmt.txt,5,%upcase(&cust_col_metadata_table)," numeric_measure +(-1) "," campaign_cd +(-1) "," communication_cd +(-1) "," cell_cd;
               putlog ' ';
               call symput("&_rc",'8');
            %end;
            %else %do;
               putlog ' ';
               putlog "%mo_error Duplicate combination detected in %upcase(&cust_col_metadata_table) table where NUMERIC_MEASURE=" numeric_measure ", CAMPAIGN_CD=" campaign_cd ", and COMMUNICATION_CD=" communication_cd ".";
               put    "input.data.validation.custcomm.triple.error.message.fmt.txt,4,%upcase(&cust_col_metadata_table)," numeric_measure +(-1) "," campaign_cd +(-1) "," communication_cd;
               putlog ' ';
               call symput("&_rc",'8');
            %end;
         end;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered processing work.cust_col_metadata_table_sum (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;
   %end;

   %*;
   %* Check that all measures are defined for the same set of communications;
   %*;
   data work.cust_col_metadata_table_mo_sort;
      retain mo_combo_control_id 0;
      set work.cust_col_metadata_table_mo_sort;
      by campaign_cd communication_cd 
   %if (&cell_cd_in_ctrl_table eq 0) %then %do;
      ;
      if first.communication_cd then mo_combo_control_id + 1;
   %end;
   %else %do;
      cell_cd;
      if first.cell_cd then mo_combo_control_id + 1;
   %end;
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered processing work.cust_col_metadata_table_mo_sort (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;

   %let mo_num_control_measures = 0;
   proc sql noprint;
      create table work.mo_control_measure_names as
         select distinct numeric_measure 
         from work.cust_col_metadata_table_mo_sort;
      select count(distinct numeric_measure) into :mo_num_control_measures
         from work.cust_col_metadata_table_mo_sort;
      create table work.mo_combo_ids as
         select distinct mo_combo_control_id
         from work.cust_col_metadata_table_mo_sort;
   quit;
   %if (&sqlrc gt 4) %then %do;
      %let &_rc = 107;
      %put %mo_error SQL error encountered obtaining information from work.cust_col_metadata_table_mo_sort (sqlrc=&sqlrc).;
      %goto ERROR_EXIT;
   %end;

   %do i = 1 %to &mo_num_control_measures;
      data _null_;
         obsnum = &i;
         set work.mo_control_measure_names point=obsnum;
         call symput("current_measure", numeric_measure);
         stop;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error obtaining numeric_measure from work.mo_control_measure_names (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;
      %let current_measure = %trim(%left(&current_measure));

      %if %sysfunc(nvalid(%nrbquote(&current_measure))) %then %do;
         proc sql noprint;
            create table work.mo_combo_ids_measure_&i as
               select distinct mo_combo_control_id 
               from work.cust_col_metadata_table_mo_sort
               where numeric_measure = "&current_measure";
         quit;
         %if (&sqlrc gt 4) %then %do;
            %let &_rc = 107;
            %put %mo_error SQL error encountered obtaining mo_combo_ids_measure (sqlrc=&sqlrc).;
            %goto ERROR_EXIT;
         %end;

         %let measures_not_defined = 0;
         proc sql noprint;
            create table work.measuresNotDefined_&i as
               select mo_combo_control_id 
               from work.mo_combo_ids as a
               where not exists (select mo_combo_control_id
                                 from work.mo_combo_ids_measure_&i
                                 where (a.mo_combo_control_id = mo_combo_control_id));

               select count(*) into :measures_not_defined
               from work.measuresNotDefined_&i;
         quit;
         %if (&sqlrc gt 4) %then %do;
            %let &_rc = 107;
            %put %mo_error SQL error encountered comparing work.mo_combo_ids_measure with work.mo_combo_ids (sqlrc=&sqlrc).;
            %goto ERROR_EXIT;
         %end;
           
         %if (&measures_not_defined ne 0) %then %do;
            data _null_;
               &setfile;
               putlog ' ';
               putlog "%mo_error The numeric measure &current_measure in the %upcase(&cust_col_metadata_table) table is not defined for all the communications.";
               put    "input.data.validation.custcol.comm.not.defined.error.message.fmt.txt,2,&current_measure,%upcase(&cust_col_metadata_table)";
               putlog ' ';
               call symput("&_rc",'8');
            run;
         %end;
      %end;
   %end;

   %*;
   %* Cross-check campaign code / communication code combos in   ;
   %* the Communication Table with campaign code / communication ;
   %* code combos in the Customer Control table. Warn about      ;
   %* those in the Communication table not found in the Customer ;
   %* Control Table.                                             ;
   %* Bypass this check for MOMA input data because this may now ;
   %* be the norm per defect S0697719.                           ;
   %*;
   %if (&moma_input eq 0) %then %do; /* Not MOMA input; perform check */
      data work.cust_col_metadata_table_combos (keep=campaign_communication_combo);
         length campaign_communication_combo $ 65; 
         set work.cust_col_metadata_table_mo_sort;
         campaign_communication_combo = upcase(strip(campaign_cd)||"*"||strip(communication_cd));
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered obtaining campaign_cd / communication_cd information from work.cust_col_metadata_table_mo_sort (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;
   
      data work.communication_table_combos (keep=campaign_communication_combo);
         length campaign_communication_combo $ 65; 
         set work.communication_table_mo_sort;
         campaign_communication_combo = upcase(strip(campaign_cd)||"*"||strip(communication_cd));
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered obtaining campaign_cd / communication_cd information from work.communication_table_mo_sort (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;
   
      proc sql noprint;
         create table work.combosInCommNotInCustControl as
         select distinct(a.campaign_communication_combo)
         from work.communication_table_combos as a
         where not exists (select distinct(campaign_communication_combo)
                           from work.cust_col_metadata_table_combos 
                           where (a.campaign_communication_combo = campaign_communication_combo));
      quit;
      %if (&sqlrc gt 4) %then %do;
         %let &_rc = 107;
         %put %mo_error SQL error encountered comparing campaign_cd / communication_cd combos between work.&cust_col_metadata_table_mo_sort and work.communication_table_in_mo_sort. (sqlrc=&sqlrc).;
         %goto ERROR_EXIT;
      %end;
   
      data _null_;
         set work.combosInCommNotInCustControl;
         &setfile;
         putlog ' ';
         comboWithSlash = translate(campaign_communication_combo, '/', '*');
         putlog "%mo_warning The campaign_cd/communication_cd combination " comboWithSlash " in the %upcase(&communication_table) table does not exist in the %upcase(&cust_col_metadata_table) table.";
         put    "input.data.validation.commcd.not.in.custcol.warning.message.fmt.txt,3," comboWithSlash ",%upcase(&communication_table),%upcase(&cust_col_metadata_table)";
         putlog ' ';
         call symput("&_rc",put(max(&&&_rc,4),1.));
      run;
   %end;

   %*;
   %* Check for reserved words in customer table;
   %*;
   data _null_;
      set work.customer_contents (keep=name);
      &setfile;

      if (lowcase(strip(name)) in &customer_reserved) then do;
         name = upcase(name);
         putlog ' ';
         putlog "%mo_error The column named " name "in the %upcase(&customer_table) table is not allowed.  It is a SAS Marketing Optimization reserved name.";
         put    "input.data.validation.reserved.column.error.message.fmt.txt,2," name ",%upcase(&customer_table)";
         putlog ' ';
         call symput("&_rc",'8');
      end;

      %do q=1 %to %mo_word_count(&customer_reserved_suffix);
         %let column=%scan(&customer_reserved_suffix,&q);
         len=%length(&column);

         if (length(name) gt len) then do;
            str=lowcase(substr(name,1,len));
            if (str eq "&column") then do;
              if (notdigit(strip(name),len+1) eq 0) then do;
                  length msg1 msg2 $256.;
                  msg1= "%mo_error The column named "|| strip(name) ||" in the %upcase(&customer_table) table is not allowed.  &column.# (where # is an integer) is a SAS Marketing Optimization reserved column name.";
                  msg2= "ERROR:  The column named "|| strip(name) ||" in the %upcase(&customer_table) table is not allowed.  &column.# (where # is an integer) is a SAS Marketing Optimization reserved column name.";
                  msg1=strip(msg1);
                  msg2=strip(msg2);
                  putlog msg1;
                  put    "input.data.validation.reserved.column.integer.error.message.fmt.txt,3," name +(-1)",%upcase(&customer_table),&column";
                  putlog ' ';
                  call symput("&_rc",'8');
               end;
            end;
         end;
      %end;
   run;

   %*;
   %* Identify extra columns in customer table.                     ;
   %* Code warning:                                                 ;
   %* work.extra_columns used in mo_prepopulate_metadata_tables.sas ;
   %*;
   proc sql noprint;
      create table work.extra_columns as
         select name, type, length
         from work.customer_contents
         where upcase(name) in
            (select upcase(column_name) from work.input_data_customer_columns);
   quit;
   %if (&sqlrc gt 4) %then %do;
      %let &_rc = 107;
      %put %mo_error SQL error encountered obtaining list of extra columns (sqlrc=&sqlrc).;
      %goto ERROR_EXIT;
   %end;

   %mo_no_notes(on);
   data _null_;
      set work.extra_columns;
      putlog ' ';
      putlog "%mo_note Additional column detected in %upcase(&customer_table) table: " name;
   run;   
   %mo_no_notes(off);

   %*;
   %* Check for extra character columns in customer table that are too long. ;
   %*;
   data work.contents_long_char;
      set work.extra_columns;
      if (type eq 2 and length gt 256);
   run;

   data _null_;
      set work.contents_long_char;
      name = upcase(name);
      &setfile;
      putlog ' ';
      putlog "%mo_error Character attribute column is defined with length greater than 256 in the %upcase(&customer_table) table: " name ;
      put    "input.data.validation.long.character.attribute.column.error.message.fmt.txt,2,%upcase(&customer_table)," name;
      putlog ' ';
      call symput("&_rc", '8');
   run;

   %*;
   %* Cross-check selected customer attributes (stored in table ;
   %* work.input_data_customer_columns by mo_create_input_data) ;
   %* with control columns ,i.e., column_nm values and          ;
   %* numeric_measure values in the Customer Control table.     ;
   %* 1) Flag an error if there are selected customer           ;
   %*    attributes that are also column_nm values in the       ;
   %*    Customer Control Table.                                ;
   %* 2) Flag as an error if there are selected customer        ;
   %*    attributes that are also numeric_measure values in the ;
   %*    Customer Control Table.                                ;
   %*;
   proc sql noprint;
      create table work.selectedCustAttrsInColumnNms as
      select distinct(a.column_name)
      from work.input_data_customer_columns as a
      where exists (select distinct(column_nm)
                    from work.&cust_col_metadata_table
                    where (upcase(a.column_name) = upcase(column_nm)));

      create table work.selectedCustAttrsInNumMeasures as
      select distinct(a.column_name)
      from work.input_data_customer_columns as a
      where exists (select distinct(numeric_measure)
                    from work.&cust_col_metadata_table
                    where (upcase(a.column_name) = upcase(numeric_measure)));
   quit;
   %if (&sqlrc gt 4) %then %do;
      %let &_rc = 107;
      %put %mo_error SQL error encountered comparing column_nm values and/or distinct numeric_measure values from work.&cust_col_metadata_table and work.input_data_customer_columns. (sqlrc=&sqlrc).;
      %goto ERROR_EXIT;
   %end;

   data _null_;
      set work.selectedCustAttrsInColumnNms;
      &setfile;
      putlog ' ';
      putlog "%mo_error The selected customer attribute " column_name " also exists as a column_nm value in the %upcase(&cust_col_metadata_table) table.";
      put    "input.data.validation.selcustattr.in.columnnm.error.message.fmt.txt,2," column_name ",%upcase(&cust_col_metadata_table)";
      putlog ' ';
      call symput("&_rc",'8');
   run;

   data _null_;
      set work.selectedCustAttrsInNumMeasures;
      &setfile;
      putlog ' ';
      putlog "%mo_error The selected customer attribute " column_name " also exists as a numeric_measure value in the %upcase(&cust_col_metadata_table) table.";
      put    "input.data.validation.selcustattr.in.numericmeasure.error.message.fmt.txt,2," column_name ",%upcase(&cust_col_metadata_table)";
      putlog ' ';
      call symput("&_rc",'8');
   run;


   %*--------------------------------;
   %* Check Numeric Measures         ;
   %*--------------------------------;

   %*;
   %* Create a table containing all numeric measures ;
   %* from CUST, COMM, and CUSTCOMM.                 ;
   %*;
   proc sql noprint;
      create table work.contents_cust_col_metadata as
         select distinct numeric_measure
         from work.cust_col_metadata_in_mo_tbl;
   quit;
   %if (&sqlrc gt 4) %then %do;
      %let &_rc = 107;
      %put %mo_error Error encountered getting contents of CUST_COL_METADATA_IN_MO_TBL (sqlrc=&sqlrc).;
      %goto ERROR_EXIT;
   %end;

   data work.customer_measures;
      set work.extra_columns;
      where type=1;
      origin=&_ATTRIBUTE_TYPE_CUST_;
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered creating work.customer_measures (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;

   data work.communication_measures;
      set work.comm_contents_actual;
      where type=1;
      origin=&_ATTRIBUTE_TYPE_COMM_;
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered creating work.communication_measures (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;
      
   data work.custcomm_measures;
      set work.contents_cust_col_metadata (rename=(numeric_measure=name));
      origin=&_ATTRIBUTE_TYPE_CUST_COMM_;
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered creating work.custcomm_measures (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;

   data work.all_measures (keep=name origin);
      set work.customer_measures work.communication_measures work.custcomm_measures;
      name=upcase(name);
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered creating work.all_measures (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;

   %*;
   %* Make sure there are no duplicate values of NAME in all_measures ;
   %*;
   proc sort data=work.all_measures;
      by name;
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered sorting work.all_measures (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;
   
   data _null_;
      set work.all_measures;
      &setfile;
      retain prev_name;
      if (_N_ gt 1) then do;
         if (name eq prev_name) then do;
            putlog ' ';
            putlog "%mo_error The following measure was defined in more than one place in the input tables: " name ".";
            put    "input.data.validation.duplicate.measure.error.message.fmt.txt,1," name;
            putlog ' ';
            call symput("&_rc",'8');
         end;
      end;
      prev_name = name;
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered processing work.all_measures (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;

   %*;
   %* Make sure there are no duplicate names of character attributes ;
   %* across customer and communication tables.                      ;
   %*;
   data work.customer_char_attributes (keep=name);
      set work.extra_columns;
      where type=2;
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered creating work.customer_char_attributes (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;

   data work.communication_char_attributes (keep=name);
      set work.comm_contents_actual;
      where type=2;
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered creating work.communication_char_attributes (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;
      
   data work.all_char_attributes;
      set work.customer_char_attributes work.communication_char_attributes;
      name=upcase(name);
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered creating work.all_char_attributes (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;

   %*;
   %* Make sure there are no duplicate values of NAME in all_char_attributes. ;
   %*;
   proc sort data=work.all_char_attributes;
      by name;
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered sorting work.all_char_attributes (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;
   
   data _null_;
      set work.all_char_attributes;
      &setfile;
      retain prev_name;
      if (_N_ gt 1) then do;
         if (name eq prev_name) then do;
            putlog ' ';
            putlog "%mo_error The following character attribute was defined in both the %upcase(&customer_table) table and the %upcase(&communication_table) table: " name ".";
            put    "input.data.validation.duplicate.attribute.error.message.fmt.txt,3,%upcase(&customer_table),%upcase(&communication_table)," name;
            putlog ' ';
            call symput("&_rc",'8');
         end;
      end;
      prev_name = name;
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered processing work.all_char_attributes (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;

   %*;
   %* Make sure there are no duplicates of measures and attributes. ;
   %*;
   proc sql noprint;
      create table work.dup_measure_attributes as
         select distinct(name) as dup_measure
         from work.all_char_attributes
         where upcase(name) in 
            (select upcase(name) from work.all_measures);
   quit;
   %if (&sqlrc gt 4) %then %do;
      %let &_rc = 107;
      %put %mo_error SQL error encountered creating work.dup_measure_attributes (sqlrc=&sqlrc).;
      %goto ERROR_EXIT;
   %end;
   
   data _null_;
      set work.dup_measure_attributes;
      &setfile;
      putlog ' ';
      putlog "%mo_error The following column has been defined as both a character attribute and a numeric measure in the input data tables: " dup_measure;
      put    "input.data.validation.both.attribute.measure.error.message.fmt.txt,1," dup_measure;
      putlog ' ';
      call symput("&_rc",'8');
   run;   


   %*;
   %* Helper macro mo_filters_validation ;
   %*;
   %macro mo_filters_validation(tablename);

      %*;
      %* Check for valid COMMUNICATION_FILTER_TXTs;
      %*;
      %if %upcase(&tablename) = CONSTRAINTS %then %let comm_filter_columns = communication_filter_txt;
      %else %if %upcase(&tablename) = CONTACT_POLICY %then 
         %let comm_filter_columns = communication_filter_txt blocking_comm_filter_a_txt blocking_comm_filter_b_txt;

      %local k;
      %do k = 1 %to %mo_word_count(&comm_filter_columns);
         %let filter_column = %upcase(%trim(%scan(&comm_filter_columns,&k)));
         %let num_comm_filters = 0;
         data _null_;
            set work.&&&tablename._table end=eof;
            call symput('comm_filter'||left(_n_),trim(&filter_column));
            if eof then call symput('num_comm_filters',_n_);
         run;
         %if (&syserr gt 4) %then %do;
            %let &_rc = 100;
            %put %mo_error Error encountered creating comm_filter macro variables for &filter_column (syserr=&syserr).;
            %goto EXIT_FILTER_CHECK; 
         %end;

         %local i;
         %do i = 1 %to &num_comm_filters;
            
            %if %nrbquote(&&comm_filter&i) ne %str() %then %do;
               %if %length(&&comm_filter&i) > 1024 %then %do;
                  data _null_;
                     &setfile;
                     putlog ' ';
                     putlog "%mo_error The value of &filter_column in observation &i of the %upcase(&&&tablename._table) table has length greater than 1024.";
                     put    "input.data.validation.filter.length.error.message.fmt.txt,3,&filter_column,&i,%upcase(&&&tablename._table)";
                     putlog ' ';
                     call symput("&_rc",'8');
                  run;
               %end;
               %else %do;
                  data work.tempdata_commfilter;
                     if (&&comm_filter&i);
                  run;
                  %if (&syserr gt 4) %then %do; /* Bad syntax */
                     data _null_;
                        &setfile;
                        putlog ' ';
                        putlog "%mo_error The value of &filter_column in observation &i of the %upcase(&&&tablename._table) table is not a valid SAS IF-clause expression.";
                        put    "input.data.validation.filter.syntax.error.message.fmt.txt,3,&filter_column,&i,%upcase(&&&tablename._table)";
                        putlog ' ';
                        call symput("&_rc",'8');
                     run;
                  %end;
                  %else %do;

                     proc contents data=work.tempdata_commfilter out=work.tempdata_commfilter_contents (keep=name type) noprint;
                     run;
                     %if (&syserr gt 4) %then %do;
                        %let &_rc = 100;
                        %put %mo_error Error encountered obtaining contents of tempdata_commfilter (syserr=&syserr).;
                        %goto EXIT_FILTER_CHECK;
                     %end;

                     %local num_invalid_attrs_in_filter;
                     %let num_invalid_attrs_in_filter = 0;
                     proc sql noprint;
                        create table work.invalid_attributes_in_filter as
                        select upcase(name) as invalid_attribute 
                        from work.tempdata_commfilter_contents
                        where upcase(name) not in
                           (select upcase(name) 
                            from work.comm_contents_actual);

                        insert into work.invalid_attributes_in_filter 
                        select upcase(name) as invalid_attribute
                        from work.tempdata_commfilter_contents
                        where upcase(name) in
                           (select upcase(name) 
                            from work.comm_contents_actual
                            where (flagFilterAsInvalid = 1));

                        select count(*) into :num_invalid_attrs_in_filter
                        from work.invalid_attributes_in_filter;
                     quit;
                     %if (&sqlrc gt 4) %then %do;
                        %let &_rc = 107;
                        %put %mo_error SQL error encountered obtaining invalid_attributes_in_filter (sqlrc=&sqlrc).;
                        %goto EXIT_FILTER_CHECK;
                     %end;

                     %if (&num_invalid_attrs_in_filter ne 0) %then %do;
                        data _null_;
                           set work.invalid_attributes_in_filter;
                           &setfile;
                           putlog ' ';
                           putlog "%mo_error The value of &filter_column in observation &i of the %upcase(&&&tablename._table) table uses an invalid attribute: " invalid_attribute".";
                           put    "input.data.validation.filter.attribute.error.message.fmt.txt,4,&filter_column,&i,%upcase(&&&tablename._table)," invalid_attribute;
                           putlog ' ';
                           call symput("&_rc",'8');
                        run;
                     %end;

                     proc sql noprint;
                        create table work.tempdata_commfilter_type as
                        select upcase(a.name) as name, a.type as type_filter, b.type as type_comm
                        from work.tempdata_commfilter_contents as a left join work.comm_contents_actual as b
                        on upcase(a.name)=upcase(b.name);
                     quit;
                     %if (&sqlrc gt 4) %then %do;
                        %let &_rc = 107;
                        %put %mo_error SQL error encountered creating work.tempdata_commfilter_type (sqlrc=&sqlrc).;
                        %goto EXIT_FILTER_CHECK;
                     %end;

                     %local num_invalid_types_in_filter;
                     %let num_invalid_types_in_filter = 0;
                     proc sql noprint;
                        create table work.invalid_types_in_filter as
                        select upcase(name) as invalid_type
                        from work.tempdata_commfilter_type
                        where (type_comm ne .) and (type_filter ne type_comm);

                        select count(*) into :num_invalid_types_in_filter
                        from work.invalid_types_in_filter;
                     quit;
                     %if (&sqlrc gt 4) %then %do;
                        %let &_rc = 107;
                        %put %mo_error SQL error encountered obtaining invalid_types_in_filter (sqlrc=&sqlrc).;
                        %goto EXIT_FILTER_CHECK;
                     %end;

                     %if (&num_invalid_types_in_filter ne 0) %then %do;
                        data _null_;
                           set work.invalid_types_in_filter;
                           &setfile;
                           putlog ' ';
                           putlog "%mo_error The value of &filter_column in observation &i of the %upcase(&&&tablename._table) table uses an attribute with the wrong type: " invalid_type".";
                           put    "input.data.validation.filter.type.error.message.fmt.txt,4,&filter_column,&i,%upcase(&&&tablename._table)," invalid_type;
                           putlog ' ';
                           call symput("&_rc",'8');
                        run;
                     %end;

                     %*;
                     %* Check whether the filter results in an empty list of ;
                     %* communications. If so, issue a warning.              ;
                     %*;
                     %if ((&num_invalid_attrs_in_filter eq 0) and (&num_invalid_types_in_filter eq 0)) %then %do;
                        %let num_comms_in_filter = 0;
                        proc sql noprint;
                           select count(*) into :num_comms_in_filter
                              from work.&communication_table
                              where &&comm_filter&i;
                        quit;
                        %if (&sqlrc gt 4) %then %do;
                           %let &_rc = 107;
                           %put %mo_error SQL error encountered obtaining num_comms_in_filter (sqlrc=&sqlrc).;
                           %goto EXIT_FILTER_CHECK;
                        %end;

                        %if &num_comms_in_filter eq 0 %then %do;
                           data _null_;
                              &setfile;
                              putlog ' ';
                              putlog "%mo_warning The value of &filter_column in observation &i of the %upcase(&&&tablename._table) table is an expression that results in the selection of 0 (zero) communications.";
                              put    "input.data.validation.empty.filter.warning.message.fmt.txt,4,&filter_column,&i,%upcase(&&&tablename._table),communications";
                              putlog ' ';
                              call symput("&_rc",put(max(&&&_rc,4),1.));
                           run;
                        %end;
                     %end;
                  %end;
               %end;
           %end;
         %end;
      %end;

      %*;
      %* Check for valid CUSTOMER_FILTER_TXTs;
      %*;
      %let num_cust_filters = 0;
      data _null_;
         set work.&&&tablename._table end=eof;
         call symput('cust_filter'||left(_n_),trim(customer_filter_txt));
         if eof then call symput('num_cust_filters',_n_);
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered creating cust_filter macro variables (syserr=&syserr).;
         %goto EXIT_FILTER_CHECK;
      %end;

      %local i;
      %do i = 1 %to &num_cust_filters;

         %if %nrbquote(&&cust_filter&i) ne %str() %then %do;
            %if %length(&&cust_filter&i) > 1024 %then %do;
               data _null_;
                  &setfile;
                  putlog ' ';
                  putlog "%mo_error The value of CUSTOMER_FILTER_TXT in observation &i of the %upcase(&&&tablename._table) table has length greater than 1024.";
                  put    "input.data.validation.filter.length.error.message.fmt.txt,3,CUSTOMER_FILTER_TXT,&i,%upcase(&&&tablename._table)";
                  putlog ' ';
                  call symput("&_rc",'8');
               run;
            %end;
            %else %do;
               data work.tempdata_custfilter;
                  if (&&cust_filter&i);
               run;
               %if (&syserr gt 4) %then %do; /* Bad syntax */
                  data _null_;
                     &setfile;
                     putlog ' ';
                     putlog "%mo_error The value of CUSTOMER_FILTER_TXT in observation &i of the %upcase(&&&tablename._table) table is not a valid SAS IF-clause expression.";
                     put    "input.data.validation.filter.syntax.error.message.fmt.txt,3,CUSTOMER_FILTER_TXT,&i,%upcase(&&&tablename._table)";
                     putlog ' ';
                     call symput("&_rc",'8');
                  run;
               %end;
               %else %do;

                  proc contents data=work.tempdata_custfilter out=work.tempdata_custfilter_contents (keep=name type) noprint;
                  run;
                  %if (&syserr gt 4) %then %do;
                     %let &_rc = 100;
                     %put %mo_error Error encountered obtaining contents of tempdata_custfilter (syserr=&syserr).;
                     %goto EXIT_FILTER_CHECK;
                  %end;


                  %local num_invalid_attrs_in_filter;
                  %let num_invalid_attrs_in_filter = 0;
                  proc sql noprint;
                     create table work.invalid_attributes_in_filter as
                     select upcase(name) as invalid_attribute
                     from work.tempdata_custfilter_contents
                     where upcase(name) not in
                        (select upcase(name) from work.extra_columns);

                     select count(*) into :num_invalid_attrs_in_filter
                     from work.invalid_attributes_in_filter;
                  quit;
                  %if (&sqlrc gt 4) %then %do;
                     %let &_rc = 107;
                     %put %mo_error SQL error encountered obtaining invalid_attributes_in_filter (sqlrc=&sqlrc).;
                     %goto EXIT_FILTER_CHECK;
                  %end;

                  %if (&num_invalid_attrs_in_filter ne 0) %then %do;
                     data _null_;
                        set work.invalid_attributes_in_filter;
                        &setfile;
                        putlog ' ';
                        putlog "%mo_error The value of CUSTOMER_FILTER_TXT in observation &i of the %upcase(&&&tablename._table) table uses an invalid attribute: " invalid_attribute".";
                        put    "input.data.validation.filter.attribute.error.message.fmt.txt,4,CUSTOMER_FILTER_TXT,&i,%upcase(&&&tablename._table)," invalid_attribute;
                        putlog ' ';
                        call symput("&_rc",'8');
                     run;
                  %end;

                  proc sql noprint;
                     create table work.tempdata_custfilter_type as
                     select upcase(a.name) as name, a.type as type_filter, b.type as type_cust
                     from work.tempdata_custfilter_contents as a left join work.extra_columns as b
                     on upcase(a.name)=upcase(b.name);
                  quit;
                  %if (&sqlrc gt 4) %then %do;
                     %let &_rc = 107;
                     %put %mo_error SQL error encountered creating work.tempdata_custfilter_type (sqlrc=&sqlrc).;
                     %goto EXIT_FILTER_CHECK;
                  %end;

                  %local num_invalid_types_in_filter;
                  %let num_invalid_types_in_filter = 0;
                  proc sql noprint;
                     create table work.invalid_types_in_filter as
                     select upcase(name) as invalid_type
                     from work.tempdata_custfilter_type
                     where (type_cust ne .) and (type_filter ne type_cust);

                     select count(*) into :num_invalid_types_in_filter
                     from work.invalid_types_in_filter;
                  quit;
                  %if (&sqlrc gt 4) %then %do;
                     %let &_rc = 107;
                     %put %mo_error SQL error encountered obtaining invalid_types_in_filter (sqlrc=&sqlrc).;
                     %goto EXIT_FILTER_CHECK;
                  %end;

                  %if (&num_invalid_types_in_filter ne 0) %then %do;
                     data _null_;
                        set work.invalid_types_in_filter;
                        &setfile;
                        putlog ' ';
                        putlog "%mo_error The value of CUSTOMER_FILTER_TXT in observation &i of the %upcase(&&&tablename._table) table uses an attribute with the wrong type: " invalid_type".";
                        put    "input.data.validation.filter.type.error.message.fmt.txt,4,CUSTOMER_FILTER_TXT,&i,%upcase(&&&tablename._table)," invalid_type;
                        putlog ' ';
                        call symput("&_rc",'8');
                     run;
                  %end;

                  %*;
                  %* Check whether the filter results in an empty list of ;
                  %* customers. If so, issue a warning.                   ;
                  %*;
                  %if ((&num_invalid_attrs_in_filter eq 0) and (&num_invalid_types_in_filter eq 0)) 
                     and (&mo_validate_input_data_cust_ids eq 1) %then %do;
                     %let cust_in_filter = ;
                     proc sql noprint;
                        select &custIdCol1 into :cust_in_filter
                           from moinput.&customer_table
                           where &&cust_filter&i;
                     quit;
                     %if (&sqlrc gt 4) %then %do;
                        %let &_rc = 107;
                        %put %mo_error SQL error encountered obtaining cust_in_filter (sqlrc=&sqlrc).;
                        %goto EXIT_FILTER_CHECK;
                     %end;

                     %if %bquote(&cust_in_filter) eq %str() %then %do;
                        data _null_;
                           &setfile;
                           putlog ' ';
                           putlog "%mo_warning The value of &filter_column in observation &i of the %upcase(&&&tablename._table) table is an expression that results in the selection of 0 (zero) customers."
                           put    "input.data.validation.empty.filter.warning.message.fmt.txt,4,CUSTOMER_FILTER_TXT,&i,%upcase(&&&tablename._table),customers";
                           putlog ' ';
                           call symput("&_rc",put(max(&&&_rc,4),1.));
                        run;
                     %end;
                  %end;
               %end;
            %end;
         %end;
      %end;
      %EXIT_FILTER_CHECK:

   %mend mo_filters_validation;


   %*--------------------------------;
   %* Check CONSTRAINTS table data   ;
   %*--------------------------------;
   %CONSTRAINTS:

   %*;
   %* Do basic CONSTRAINTS data validation;
   %*;
   %if (%bquote(&constraints_table) ne %str()) %then %do;
      %put;

      data work.constraints_in_mo_tbl;
         set work.constraints_in_mo_tbl;

         constraint_id=compbl(trim(left(constraint_id)));
         constraint_operator=upcase(constraint_operator);
         constraint_type_cd=upcase(constraint_type_cd);
         constraint_scope_cd=upcase(constraint_scope_cd);
         constraint_computation_cd=upcase(constraint_computation_cd);

         if (constraint_type_cd = '') then do;
            constraint_type_cd = 'MISCELLANEOUS';
         end;
         if (constraint_scope_cd = '') then do;
            constraint_scope_cd = 'AGGREGATE';
         end;
         if constraint_computation_cd = '' then do;
            if (numeric_measure = '') then do;
               constraint_computation_cd = 'NUM_OFFERS';
            end;
            else if (scan(numeric_measure,2,' ') ne '') then do;
               constraint_computation_cd = 'RATIO';
            end;
            else do;
               constraint_computation_cd = 'SUM';
            end;
         end;
         if (active_flg = .) then do;
            active_flg = 1;
         end;
         if (sensitivity_flg = .) then do;
            sensitivity_flg = 0;
         end;

         constraint_numerator_measure = upcase(scan(numeric_measure,1,' '));
         constraint_denominator_measure = upcase(scan(numeric_measure,2,' '));

         nomatch_numerator_measure = 0;
         nomatch_comm_numerator_measure = 0;
         nomatch_denominator_measure = 0;
         nomatch_comm_denominator_measure = 0;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered pre-processing work.constraints_in_mo_tbl (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;

      proc sql noprint;
         update work.constraints_in_mo_tbl
            set nomatch_numerator_measure = 1
            where constraint_numerator_measure not in 
               (select upcase(name) from work.all_measures);
         update work.constraints_in_mo_tbl
            set nomatch_comm_numerator_measure = 1
            where constraint_numerator_measure not in 
               (select upcase(name) from work.communication_measures);
         update work.constraints_in_mo_tbl
            set nomatch_comm_denominator_measure = 1
            where constraint_denominator_measure not in 
               (select upcase(name) from work.communication_measures);
      quit;
      %if (&sqlrc gt 4) %then %do;
         %let &_rc = 107;
         %put %mo_error SQL error encountered updating nomatch columns in work.constraints_in_mo_tbl (sqlrc=&sqlrc).;
         %goto ERROR_EXIT;
      %end;

      data work.&constraints_table (drop = constraint_numerator_measure constraint_denominator_measure nomatch_numerator_measure nomatch_denominator_measure nomatch_comm_numerator_measure nomatch_comm_denominator_measure);
         set work.constraints_in_mo_tbl;
         &setfile;

         if (constraint_id eq '') then do;
            putlog ' ';
            putlog "%mo_error Blank value detected for CONSTRAINT_ID in observation " _N_ +(-1) " of %upcase(&constraints_table) table.";
            put    "input.data.validation.blank.value.error.message.fmt.txt,3,CONSTRAINT_ID," _N_ +(-1)",%upcase(&constraints_table)";
            putlog ' ';
            call symput("&_rc",'8');
         end;
         if (indexc(constraint_id, ",/<>:;[]{}\|=%^&*+~`") or indexc(constraint_id, "'") or indexc(constraint_id, '"')) then do;
            putlog ' ';
            putlog "%mo_error Special characters detected in CONSTRAINT_ID in observation " _N_ +(-1) " of %upcase(&constraints_table) table. The following characters are not allowed: ,/<>:;[]{}\|=%^&*+~`'";
            put    "input.data.validation.special.char.name.error.message.fmt.txt,3,CONSTRAINT_ID," _N_ +(-1)",%upcase(&constraints_table)";
            putlog ' ';
            call symput("&_rc", '8');
         end;         
         if (length(constraint_id) > 32) then do;
            putlog ' ';
            putlog "%mo_error Length of CONSTRAINT_ID in observation " _N_ +(-1) " of %upcase(&constraints_table) table cannot be greater than 32.";
            put    "input.data.validation.column.length.error.message.fmt.txt,3,CONSTRAINT_ID," _N_ +(-1)",%upcase(&constraints_table)";
            putlog ' ';
            call symput("&_rc",'8');
         end;
         if (constraint_operator not in ('LE','GE')) then do;
            putlog ' ';
            putlog "%mo_error Invalid CONSTRAINT_OPERATOR detected in observation " _N_ +(-1) " of %upcase(&constraints_table) table. CONSTRAINT_OPERATOR must be LE or GE.";
            put    "input.data.validation.operator.value.error.message.fmt.txt,4,CONSTRAINT_OPERATOR," _N_+(-1)",%upcase(&constraints_table),CONSTRAINT_OPERATOR";
            putlog ' ';
            call symput("&_rc",'8');
         end;
         if (constraint_rhs eq .) then do;
            putlog ' ';
            putlog "%mo_error Missing value detected for CONSTRAINT_RHS in observation " _N_ +(-1) " of %upcase(&constraints_table) table.";
            put    "input.data.validation.missing.value.error.message.fmt.txt,3,CONSTRAINT_RHS," _N_ +(-1)",%upcase(&constraints_table)";
            putlog ' ';
            call symput("&_rc",'8');
         end;
         if (constraint_type_cd not in ('BUDGET','CELLSIZE','CHANNEL','ROI','MISC','MISCELLANEOUS','REPORT','REPORT ONLY')) then do;
            putlog ' ';
            putlog "%mo_error Invalid CONSTRAINT_TYPE_CD detected in observation " _N_ +(-1) " of %upcase(&constraints_table) table. CONSTRAINT_TYPE_CD must be BUDGET, CELLSIZE, CHANNEL, ROI, SUM, MISCELLANEOUS, REPORT, or REPORT ONLY.";
            put    "input.data.validation.type.cd.error.message.fmt.txt,2," _N_+(-1)",%upcase(&constraints_table)";
            putlog ' ';
            call symput("&_rc",'8');
         end;
         if (constraint_scope_cd not in ('AGGREGATE','CUSTOMER','HOUSEHOLD')) then do;
            putlog ' ';
            putlog "%mo_error Invalid CONSTRAINT_SCOPE_CD detected in observation " _N_ +(-1) " of %upcase(&constraints_table) table. CONSTRAINT_SCOPE_CD must be AGGREGATE, CUSTOMER, or HOUSEHOLD.";
            put    "input.data.validation.scope.cd.error.message.fmt.txt,2," _N_+(-1)",%upcase(&constraints_table)";
            putlog ' ';
            call symput("&_rc",'8');
         end;
         %*;
         %* A constraint_scope_cd of household level must be cross-checked to ;
         %* make sure a household column was specified in the customer table. ;
         %*;
         %if (&household_flag eq 0) %then %do;
            if (constraint_scope_cd eq 'HOUSEHOLD') then do;
               putlog ' ';
               putlog "%mo_error Invalid value detected for CONSTRAINT_SCOPE_CD in observation " _N_ +(-1) " of %upcase(&constraints_table) table, because household ID column not found in %upcase(&customer_table).";
               put    "input.data.validation.scope.cd.nohousehold.error.message.fmt.txt,3,CONSTRAINT_SCOPE_CD," _N_ +(-1)",%upcase(&constraints_table)";
               putlog ' ';
               call symput("&_rc",'8');
            end;
         %end;
         if (constraint_computation_cd not in ('NUM_OFFERS','SUM','AVERAGE','RATIO')) then do;
            putlog ' ';
            putlog "%mo_error Invalid CONSTRAINT_COMPUTATION_CD detected in observation " _N_ +(-1) " of %upcase(&constraints_table) table. CONSTRAINT_COMPUTATION_CD must be NUM_OFFERS, SUM, AVERAGE, or RATIO.";
            put    "input.data.validation.computation.cd.error.message.fmt.txt,2," _N_+(-1)",%upcase(&constraints_table)";
            putlog ' ';
            call symput("&_rc",'8');
         end;
         if (active_flg not in (0,1)) then do;
            putlog ' ';
            putlog "%mo_error Invalid ACTIVE_FLG detected in observation " _N_ +(-1) " of %upcase(&constraints_table) table. ACTIVE_FLG must be 0 or 1.";
            put    "input.data.validation.valid.flg.error.message.fmt.txt,3,ACTIVE_FLG," _N_+(-1)",%upcase(&constraints_table)";
            putlog ' ';
            call symput("&_rc",'8');
         end;
         if (sensitivity_flg not in (0,1)) then do;
            putlog ' ';
            putlog "%mo_error Invalid SENSITIVITY_FLG detected in observation " _N_ +(-1) " of %upcase(&constraints_table) table. SENSITIVITY_FLG must be 0 or 1.";
            put    "input.data.validation.valid.flg.error.message.fmt.txt,3,SENSITIVITY_FLG," _N_+(-1)",%upcase(&constraints_table)";
            putlog ' ';
            call symput("&_rc",'8');
         end;
         else if (sensitivity_flg = 1) then do;
            if ((constraint_type_cd in ('REPORT','REPORT ONLY')) or
                (constraint_scope_cd in ('CUSTOMER'))) then do;
               putlog ' ';
               putlog "%mo_error Invalid SENSITIVITY_FLG detected in observation " _N_ +(-1) " of %upcase(&constraints_table) table. SENSITIVITY_FLG must not be set to 1 for a constraint whose type is REPORT ONLY or whose scope is CUSTOMER.";
               put    "input.data.validation.sensitivity.flg.not.allowed.error.message.fmt.txt,2," _N_+(-1)",%upcase(&constraints_table)";
               putlog ' ';
               call symput("&_rc",'8');
            end;
         end;

         if ((constraint_numerator_measure ne '') and (nomatch_numerator_measure = 1)) then do;
            putlog ' ';
            putlog "%mo_error Invalid NUMERIC_MEASURE detected in observation " _N_ +(-1) " of %upcase(&constraints_table) table: " constraint_numerator_measure;
            put    "input.data.validation.invalid.measure.error.message.fmt.txt,3," _N_ + (-1)",%upcase(&constraints_table)," constraint_numerator_measure;
            putlog ' ';
            call symput("&_rc",'8');
         end;

         if ((constraint_denominator_measure ne '') and (nomatch_denominator_measure = 1)) then do;
            putlog ' ';
            putlog "%mo_error Invalid NUMERIC_MEASURE detected in observation " _N_ +(-1) " of %upcase(&constraints_table) table: " constraint_denominator_measure;
            put    "input.data.validation.invalid.measure.error.message.fmt.txt,3," _N_ + (-1)",%upcase(&constraints_table)," constraint_denominator_measure;
            putlog ' ';
            call symput("&_rc",'8');
         end;
         if (constraint_computation_cd eq 'RATIO' and constraint_denominator_measure eq '') then do;
            putlog ' ';
            putlog "%mo_error Missing denominator measure detected in ratio constraint in observation " _N_ +(-1) " of %upcase(&constraints_table) table.";
            put    "input.data.validation.ratio.denominator.error.message.fmt.txt,2," _N_ +(-1)",%upcase(&constraints_table)";
            putlog ' ';
            call symput("&_rc",'8');
         end;
         if (constraint_computation_cd in ('SUM','AVERAGE')) then do;
            if (constraint_denominator_measure ne '') then do;
               putlog ' ';
               putlog "%mo_error Denominator measure detected in non-ratio constraint in observation " _N_ +(-1) " of %upcase(&constraints_table) table.";
               put    "input.data.validation.nonratio.denominator.error.message.fmt.txt,2," _N_ +(-1)",%upcase(&constraints_table)";
               putlog ' ';
               call symput("&_rc",'8');
            end;
            if (constraint_numerator_measure eq '') then do;
               putlog ' ';
               putlog "%mo_error Missing numerator measure detected in SUM or AVERAGE type constraint in observation " _N_ +(-1) " of %upcase(&constraints_table) table.";
               put    "input.data.validation.nonratio.numerator.error.message.fmt.txt,2," _N_ +(-1)",%upcase(&constraints_table)";
               putlog ' ';
               call symput("&_rc",'8');
            end;
         end;
         if (constraint_computation_cd eq 'NUM_OFFERS' and constraint_numerator_measure ne '') then do;
            putlog ' ';
            putlog "%mo_error Numeric measure detected in NUM_OFFERS type constraint in observation " _N_ +(-1) " of %upcase(&constraints_table) table.";
            put    "input.data.validation.numoffers.measure.error.message.fmt.txt,2," _N_ +(-1)",%upcase(&constraints_table)";
            putlog ' ';
            call symput("&_rc",'8');
         end;

         if (constraint_scope_cd in ('CUSTOMER','HOUSEHOLD')) then do;
            if (((constraint_computation_cd in ('SUM','AVERAGE')) and
                 (nomatch_comm_numerator_measure = 1)) or
                ((constraint_computation_cd = 'RATIO') and
                ((nomatch_comm_numerator_measure = 1) or (nomatch_comm_denominator_measure = 1)))) then do;
               putlog ' ';
               putlog "%mo_error Invalid value detected for CONSTRAINT_SCOPE_CD in observation " _N_ +(-1) " of %upcase(&constraints_table) table, because associated numeric measure is not of type COMMUNICATION.";
               put    "input.data.validation.scope.cd.noncomm.error.message.fmt.txt,3,CONSTRAINT_SCOPE_CD," _N_ +(-1)",%upcase(&constraints_table)";
               putlog ' ';
               call symput("&_rc",'8');
            end;
         end;

      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered creating work.&constraints_table (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;

      %*;
      %* Check filters. ;
      %*;
      %mo_filters_validation(constraints);

      %*;
      %* Check for duplicate constraints. ;
      %*;
      proc summary data=work.&constraints_table nway missing;
         class constraint_id constraint_type_cd constraint_operator constraint_rhs numeric_measure
               constraint_scope_cd constraint_computation_cd communication_filter_txt customer_filter_txt;
         output out=work.constraints_table_sum_output;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered creating work.constraints_table_sum_output (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;

      data _null_;
         set work.constraints_table_sum_output;
         &setfile;
         if (_FREQ_ gt 1) then do;
            putlog ' ';
            putlog "%mo_error Duplicate constraint detected in %upcase(&constraints_table) table where CONSTRAINT_ID=" constraint_id ".";
            put    "input.data.validation.duplicate.value.error.message.fmt.txt,4,constraint,%upcase(&constraints_table),CONSTRAINT_ID," constraint_id;
            putlog ' ';
            call symput("&_rc",'8');
         end;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered processing work.constraints_table_sum_output (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;
   %end;


   %*-----------------------------------;
   %* Check CONTACT_POLICY table data   ;
   %*-----------------------------------;
   %CONTACT_POLICY:

   %*;
   %* Do basic CONTACT_POLICY data validation;
   %*;
   %if (%bquote(&contact_policy_table) ne %str()) %then %do;
      %put;

      data work.contact_policy_in_mo_tbl;
         set work.contact_policy_in_mo_tbl;
         nomatch_start_time_period = 0;
         nomatch_end_time_period = 0;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered creating nomatch columns in work.contact_policy_in_mo_tbl (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;

      %if (%bquote(&time_period_table) ne %str()) %then %do;
         proc sql noprint;
            update work.contact_policy_in_mo_tbl
               set nomatch_start_time_period = 1
               where start_time_period not in 
                  (select time_period from work.distinct_time_periods);

            update work.contact_policy_in_mo_tbl
               set nomatch_end_time_period = 1
               where end_time_period not in 
                  (select time_period from work.distinct_time_periods);
         quit;
         %if (&sqlrc gt 4) %then %do;
            %let &_rc = 107;
            %put %mo_error SQL error encountered updating nomatch columns in work.contact_policy_in_mo_tbl (sqlrc=&sqlrc).;
            %goto ERROR_EXIT;
         %end;
      %end;

      data work.&contact_policy_table (drop=nomatch_start_time_period nomatch_end_time_period);
         format _mo_contact_policy_type $8.;
         set work.contact_policy_in_mo_tbl;
         &setfile;

         contact_policy_id=compbl(trim(left(contact_policy_id)));
         contact_policy_operator=upcase(contact_policy_operator);
         target_level_cd=upcase(target_level_cd);

         if (active_flg = .) then do;
            active_flg = 1;
         end;
         if (target_level_cd = '') then do;
            target_level_cd = 'CUSTOMER';
         end;

         _sql_sort_order = _n_;

         %*;
         %* Create a new variable called _mo_contact_policy_type.            ;
         %* Set it either to 'MAXMIN','ROLLING','BLOCKING', or 'MULT'.       ;
         %* Then in the validation, use this policy to check conditions.     ;
         %*;
         if (contact_limit_cnt ne .) then do;
            if (rolling_period_cnt eq .) then do;
               if ((start_time_period ne '') or 
                   (end_time_period ne '')) then do;
                  _mo_contact_policy_type = 'MULT';
               end;
               else do;
                  _mo_contact_policy_type = 'MAXMIN';
               end;
            end;
            else do;
               _mo_contact_policy_type = 'ROLLING';
            end;
            if ((blocking_comm_filter_a_txt ne '') or 
                (blocking_comm_filter_b_txt ne '') or 
                (blocking_period_cnt ne .)) then do;
               _mo_contact_policy_type = 'MULT';
            end;
         end;
         else do;
            if (blocking_period_cnt eq .) then do;
               if ((start_time_period ne '') or 
                   (end_time_period ne '')) then do;
                  _mo_contact_policy_type = 'MULT';
               end;
               else do;
                  _mo_contact_policy_type = 'BLOCKING';
               end;
            end;
            else do;
               _mo_contact_policy_type = 'BLOCKING';
            end;
            if ((contact_policy_operator ne '') or 
                (communication_filter_txt ne '') or 
                (rolling_period_cnt ne .)) then do;
               _mo_contact_policy_type = 'MULT';
            end;
         end;

         if (contact_policy_id eq '') then do;
            putlog ' ';
            putlog "%mo_error Blank value detected for CONTACT_POLICY_ID in observation " _N_ +(-1) " of %upcase(&contact_policy_table) table.";
            put    "input.data.validation.blank.value.error.message.fmt.txt,3,CONTACT_POLICY_ID," _N_ +(-1)",%upcase(&contact_policy_table)";
            putlog ' ';
            call symput("&_rc",'8');
         end;
         if (indexc(contact_policy_id, ",/<>:;[]{}\|=%^&*+~`") or indexc(contact_policy_id, "'") or indexc(contact_policy_id, '"')) then do;
            putlog ' ';
            putlog "%mo_error Special characters detected in CONTACT_POLICY_ID in observation " _N_ +(-1) " of %upcase(&contact_policy_table) table. The following characters are not allowed: ,/<>:;[]{}\|=%^&*+~`'";
            put    "input.data.validation.special.char.name.error.message.fmt.txt,3,CONTACT_POLICY_ID," _N_ +(-1)",%upcase(&contact_policy_table)";
            putlog ' ';
            call symput("&_rc", '8');
         end;         
         if (length(contact_policy_id) > 32) then do;
            putlog ' ';
            putlog "%mo_error Length of CONTACT_POLICY_ID in observation " _N_ +(-1) " of %upcase(&contact_policy_table) table cannot be greater than 32.";
            put    "input.data.validation.column.length.error.message.fmt.txt,3,CONTACT_POLICY_ID," _N_ +(-1)",%upcase(&contact_policy_table)";
            putlog ' ';
            call symput("&_rc",'8');
         end;
         if (_mo_contact_policy_type eq 'MULT') then do;
            putlog ' ';
            putlog "%mo_error Cannot determine the type of contact policy in observation " _N_ +(-1) " of %upcase(&contact_policy_table) table.";
            put    "input.data.validation.invalid.policy.error.message.fmt.txt,2," _N_ +(-1)",%upcase(&contact_policy_table)";
            putlog ' ';
            call symput("&_rc",'8');
         end;
         if (active_flg not in (0,1)) then do;
            putlog ' ';
            putlog "%mo_error Invalid ACTIVE_FLG detected in observation " _N_ +(-1) " of %upcase(&contact_policy_table) table. ACTIVE_FLG must be 0 or 1.";
            put    "input.data.validation.valid.flg.error.message.fmt.txt,3,ACTIVE_FLG," _N_+(-1)",%upcase(&contact_policy_table)";
            putlog ' ';
            call symput("&_rc",'8');
         end;
         if (_mo_contact_policy_type in ('MAXMIN','ROLLING')) then do;
            if ((contact_limit_cnt lt 0) or (floor(contact_limit_cnt) ne contact_limit_cnt)) then do;
               putlog ' ';
               putlog "%mo_error Invalid CONTACT_LIMIT_CNT detected in observation " _N_ +(-1) " of %upcase(&contact_policy_table) table. CONTACT_LIMIT_CNT must be a nonnegative integer.";
               put    "input.data.validation.contact.limit.error.message.fmt.txt,2," _N_ +(-1)",%upcase(&contact_policy_table)";
               putlog ' ';
               call symput("&_rc",'8');
            end;
            if (contact_policy_operator not in ('LE','GE')) then do;
               putlog ' ';
               putlog "%mo_error Invalid CONTACT_POLICY_OPERATOR detected in observation " _N_ +(-1) " of %upcase(&contact_policy_table) table. CONTACT_POLICY_OPERATOR must be LE or GE.";
               put    "input.data.validation.operator.value.error.message.fmt.txt,4,CONTACT_POLICY_OPERATOR," _N_+(-1)",%upcase(&contact_policy_table),CONTACT_POLICY_OPERATOR";
               putlog ' ';
               call symput("&_rc",'8');
            end;
         end;
         if (_mo_contact_policy_type eq 'ROLLING') then do;
            if ((rolling_period_cnt le 0) or (floor(rolling_period_cnt) ne rolling_period_cnt)) then do;
               putlog ' ';
               putlog "%mo_error Invalid ROLLING_PERIOD_CNT detected in observation " _N_ +(-1) " of %upcase(&contact_policy_table) table. ROLLING_PERIOD_CNT must be a positive integer.";
               put    "input.data.validation.rolling.period.error.message.fmt.txt,2," _N_ +(-1)",%upcase(&contact_policy_table)";
               putlog ' ';
               call symput("&_rc",'8');
            end;
         end;
         if (_mo_contact_policy_type eq 'BLOCKING') then do;
            if ((blocking_period_cnt ne .) and ((blocking_period_cnt le 0) or (floor(blocking_period_cnt) ne blocking_period_cnt))) then do;
               putlog ' ';
               putlog "%mo_error Invalid BLOCKING_PERIOD_CNT detected in observation " _N_ +(-1) " of %upcase(&contact_policy_table) table. BLOCKING_PERIOD_CNT must be a nonnegative integer.";
               put    "input.data.validation.blocking.period.error.message.fmt.txt,2," _N_ +(-1)",%upcase(&contact_policy_table)";
               putlog ' ';
               call symput("&_rc",'8');
            end;
            if ((blocking_comm_filter_a_txt eq '') or 
                (blocking_comm_filter_b_txt eq '')) then do;
               putlog ' ';
               putlog "%mo_error Missing BLOCKING_COMM_FILTER_A_TXT or BLOCKING_COMM_FILTER_B_TXT detected in observation " _N_ +(-1) " of %upcase(&contact_policy_table) table. Both must be non-blank and specify disjoint sets of communications.";
               put    "input.data.validation.blocking.filter.error.message.fmt.txt,2," _N_ +(-1)",%upcase(&contact_policy_table)";
               putlog ' ';
               call symput("&_rc",'8');
            end; 
         end;
         if (length(start_time_period) > 32) then do;
            putlog ' ';
            putlog "%mo_error Length of START_TIME_PERIOD in observation " _N_ +(-1) " of %upcase(&contact_policy_table) table cannot be greater than 32.";
            put    "input.data.validation.column.length.error.message.fmt.txt,3,START_TIME_PERIOD," _N_ +(-1)",%upcase(&contact_policy_table)";
            putlog ' ';
            call symput("&_rc",'8');
         end;
         if (length(end_time_period) > 32) then do;
            putlog ' ';
            putlog "%mo_error Length of END_TIME_PERIOD in observation " _N_ +(-1) " of %upcase(&contact_policy_table) table cannot be greater than 32.";
            put    "input.data.validation.column.length.error.message.fmt.txt,3,END_TIME_PERIOD," _N_ +(-1)",%upcase(&contact_policy_table)";
            putlog ' ';
            call symput("&_rc",'8');
         end;

         if ((start_time_period ne '') and 
             (start_time_period ne "&_CONTACT_POLICY_TP_TABLE_RANGE_") and 
             (nomatch_start_time_period = 1)) then do;
            putlog ' ';
            putlog "%mo_error Invalid START_TIME_PERIOD detected in observation " _N_ +(-1) " of %upcase(&contact_policy_table) table.";
            put    "input.data.validation.invalid.value.error.message.fmt.txt,3,START_TIME_PERIOD," _N_ +(-1)",%upcase(&contact_policy_table)";
            putlog ' ';
            call symput("&_rc",'8');
         end;

         if ((end_time_period ne '') and 
             (end_time_period ne "&_CONTACT_POLICY_TP_TABLE_RANGE_") and 
             (nomatch_end_time_period = 1)) then do;
            putlog ' ';
            putlog "%mo_error Invalid END_TIME_PERIOD detected in observation " _N_ +(-1) " of %upcase(&contact_policy_table) table.";
            put    "input.data.validation.invalid.value.error.message.fmt.txt,3,END_TIME_PERIOD," _N_ +(-1)",%upcase(&contact_policy_table)";
            putlog ' ';
            call symput("&_rc",'8');
         end;
         if (target_level_cd not in ('CUSTOMER','HOUSEHOLD')) then do;
            putlog ' ';
            putlog "%mo_error Invalid TARGET_LEVEL_CD detected in observation " _N_ +(-1) " of %upcase(&contact_policy_table) table. TARGET_LEVEL_CD must be CUSTOMER or HOUSEHOLD.";
            put    "input.data.validation.target.level.error.message.fmt.txt,2," _N_+(-1)",%upcase(&contact_policy_table)";
            putlog ' ';
            call symput("&_rc",'8');
         end;
         %*;
         %* A target_level_cd of household level must be cross-checked to     ;
         %* make sure a household column was specified in the customer table. ;
         %*;
         %if (&household_flag eq 0) %then %do;
            if (target_level_cd eq 'HOUSEHOLD') then do;
               putlog ' ';
               putlog "%mo_error Invalid value detected for TARGET_LEVEL_CD in observation " _N_ +(-1) " of %upcase(&contact_policy_table) table, because household ID column not found in %upcase(&customer_table).";
               put    "input.data.validation.target.level.nohousehold.error.message.fmt.txt,3,TARGET_LEVEL_CD," _N_ +(-1)",%upcase(&contact_policy_table)";
               putlog ' ';
               call symput("&_rc",'8');
            end;
         %end;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered creating work.&contact_policy_table (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;

      %*;
      %* Check for duplicate contact policies. ;
      %*;
      proc summary data=work.&contact_policy_table nway missing;
         class contact_policy_id contact_policy_operator contact_limit_cnt 
               start_time_period end_time_period rolling_period_cnt
               communication_filter_txt customer_filter_txt
               blocking_period_cnt blocking_comm_filter_a_txt blocking_comm_filter_b_txt
               target_level_cd;
         output out=work.contact_policies_sum_output;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered creating work.contact_policies_sum_output (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;

      data _null_;
         set work.contact_policies_sum_output;
         &setfile;
         if (_FREQ_ gt 1) then do;
            putlog ' ';
            putlog "%mo_error Duplicate contact policy detected in %upcase(&contact_policy_table) table where CONTACT_POLICY_ID=" contact_policy_id ".";
            put    "input.data.validation.duplicate.value.error.message.fmt.txt,4,contact policy,%upcase(&contact_policy_table),CONTACT_POLICY_ID," contact_policy_id;
            putlog ' ';
            call symput("&_rc",'8');
         end;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered processing work.contact_policies_sum_output (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;

      %if (%bquote(&time_period_table) ne %str()) %then %do;
         proc sql noprint;
            create table work.contact_policy_start as
               select a.*, b.order_no as start_time_id
               from work.&contact_policy_table as a left join
                    work.&time_period_table as b
               on a.start_time_period = b.time_period;
         quit;
         %if (&sqlrc gt 4) %then %do;
            %let &_rc = 107;
            %put %mo_error SQL error encountered creating work.contact_policy_start (sqlrc=&sqlrc).;
            %goto ERROR_EXIT;
         %end;

         %local first_time_period_id last_time_period_id;
         %let first_time_period_id = 0;
         %let last_time_period_id = 0;
         proc sql noprint;
            create table work.contact_policy_end as
               select a.*, b.order_no as end_time_id
               from work.contact_policy_start as a left join
                    work.&time_period_table as b
               on a.end_time_period = b.time_period
               order by _sql_sort_order;
            select min(order_no), max(order_no)
               into :first_time_period_id, :last_time_period_id
               from work.&time_period_table;
         quit;
         %if (&sqlrc gt 4) %then %do;
            %let &_rc = 107;
            %put %mo_error SQL error encountered creating work.contact_policy_end (sqlrc=&sqlrc).;
            %goto ERROR_EXIT;
         %end;

         data work.&contact_policy_table (drop=_sql_sort_order actual_start_id actual_end_id);
            set work.contact_policy_end;
            &setfile;

            actual_start_id = start_time_id;
            if (start_time_id = .) then do;
               start_time_id = &_CONTACT_POLICY_TP_TABLE_RANGE_;
               actual_start_id = &first_time_period_id;
            end;

            actual_end_id = end_time_id;
            if (end_time_id = .) then do;
               end_time_id = &_CONTACT_POLICY_TP_TABLE_RANGE_;
               actual_end_id = &last_time_period_id;
            end;

            if (rolling_period_cnt = .) then do;
               rolling_period_cnt = (actual_end_id - actual_start_id + 1);
            end;
            
            if (actual_start_id > actual_end_id) then do;
               putlog ' ';
               putlog "%mo_error Invalid time sequence detected in observation " _N_ +(-1) " of %upcase(&contact_policy_table) table.";
               put    "input.data.validation.time.sequence.error.message.fmt.txt,2," _N_ +(-1)",%upcase(&contact_policy_table)";
               putlog ' ';
               call symput("&_rc",'8');
            end;
         run;
         %if (&syserr gt 4) %then %do;
            %let &_rc = 100;
            %put %mo_error Error encountered creating work.&contact_policy_table (syserr=&syserr).;
            %goto ERROR_EXIT;
         %end;
      %end;

      %*;
      %* Check filters. ;
      %*;
      %mo_filters_validation(contact_policy);
   %end;


   %*------------------------------------------------;
   %* Check AGENT table data                         ;
   %*------------------------------------------------;
   %AGENT:

   %let agent_constraints_exist = 0;
   %if (%bquote(&agent_capacity_table) ne %str()) %then %do;
      %let agent_column = %upcase(&agent_column);

      proc contents data=work.agent_capacity_in_mo_tbl out=work.agent_contents (keep=name type length) noprint;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered obtaining contents of work.agent_capacity_in_mo_tbl (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;

      %*;
      %* Check to see if the agent channel has time periods defined. ;
      %*;
      %let agent_with_time_periods = 0;
      data _null_;
         set work.communication_in_mo_tbl;
         where upcase(trim(left(channel_cd))) = "%upcase(&agent_column)";
         if time_period ne "" then do;
            call symput('agent_with_time_periods', '1');
            stop;
         end;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered obtaining agent_with_time_periods (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;

      %if &agent_with_time_periods = 0 %then %do;
         data _null_;
            set work.agent_capacity_in_mo_tbl;
            if time_period ne "" then do;
               call symput('agent_with_time_periods', '1');
               stop;
            end;
         run;
         %if (&syserr gt 4) %then %do;
            %let &_rc = 100;
            %put %mo_error Error encountered obtaining agent_with_time_periods (syserr=&syserr).;
            %goto ERROR_EXIT;
         %end;
      %end;

      %*;
      %* Check to see if the agent table has the required columns. If ;
      %* the agent channel has time periods defined, the required     ;
      %* columns are &agent_column, time_period, capacity. Otherwise, ;
      %* the required columns are &agent_column, capacity             ;
      %*;
      %local has_agent_column has_time_period_column has_capacity_column;
      proc sql noprint;
         select count(*) 
            into :has_agent_column 
            from work.agent_contents 
            where upcase(name)="&agent_column";

         select count(*) 
            into :has_time_period_column 
            from work.agent_contents 
            where upcase(name)="TIME_PERIOD";

         select count(*) 
            into :has_capacity_column 
            from work.agent_contents 
            where upcase(name)="CAPACITY";
      quit;
      %if (&sqlrc gt 4) %then %do;
         %let &_rc = 107;
         %put %mo_error SQL error encountered obtaining columns from work.agent_contents (sqlrc=&sqlrc).;
         %goto ERROR_EXIT;
      %end;


      %if &has_agent_column=0 %then %do;
         data _null_;
            &setfile;
            putlog ' ';
            putlog "%mo_error The agent capacity table does not have the specified agent column %upcase(&agent_column).";
            put "input.data.validation.agent.agent.column.missing.error.message.fmt.txt,1,%upcase(&agent_column)";
            putlog ' ';
            call symput("&_rc", '8');
         run;
      %end;

      %if &agent_with_time_periods = 1 %then %do;
         %if &has_time_period_column=0 %then %do;
            data _null_;
               &setfile;
               putlog ' ';
               putlog "%mo_error The agent capacity table does not have the TIME_PERIOD column.";
               put "input.data.validation.agent.period.column.missing.error.message.txt,0";
               putlog ' ';
               call symput("&_rc",'8');
            run;
         %end;
      %end;

      %if &has_capacity_column=0 %then %do;
         data _null_;
            &setfile;
            putlog ' ';
            putlog "%mo_error The agent capacity table does not have the CAPACITY column.";
            put "input.data.validation.agent.capacity.column.missing.error.message.txt,0";
            putlog ' ';
            call symput("&_rc",'8');
         run;
      %end;

      %*;
      %* Check to see if the agent columns are of same type in the customer ;
      %* table and in the agent table. Also check that the max length of    ;
      %* of the agent column is less than 32 (in both tables).              ;
      %*;
      %if &has_agent_column %then %do;
         %let agent_same_type = 1;
         %let max_length_agent = 0;
         %let max_length_customer = 0;
         proc sql noprint;
            select
               case
               when a.type=b.type then 1
                                  else 0
               end as agent_same_type into :agent_same_type
               from work.customer_contents as a, work.agent_contents as b
               where upcase(a.name)="&agent_column" and upcase(b.name)="&agent_column";

            select length into :max_length_agent from work.agent_contents where upcase(name)="&agent_column";
            %if &max_length_agent > 32 %then %do;
               select max(length(&agent_column)) into :max_length_agent from work.agent_capacity_in_mo_tbl;
            %end;

            select length into :max_length_customer from work.customer_contents where upcase(name)="&agent_column";
            %if &max_length_agent > 32 %then %do;
               select max(length(&agent_column)) into :max_length_customer from moinput.&customer_table;
            %end;
         quit;
         %if (&sqlrc gt 4) %then %do;
            %let &_rc = 107;
            %put %mo_error SQL error encountered obtaining agent_same_type (sqlrc=&sqlrc).;
            %goto ERROR_EXIT;
         %end;

         %if &agent_same_type = 0 %then %do;
            data _null_;
               &setfile;
               putlog ' ';
               putlog "%mo_error The specified agent column %upcase(&agent_column) in the customer table and the agent capacity table are of different data type.";
               put "input.data.validation.agent.type.mismatch.error.message.fmt.txt,1,%upcase(&agent_column)";
               putlog ' ';
               call symput("&_rc",'8');
            run; 
         %end;
         %if &max_length_agent > 32 %then %do;
            data _null_;
               &setfile;
               putlog ' ';
               putlog "%mo_error Length of %upcase(&agent_column) in the %upcase(&agent_capacity_table) table must not be greater than 32.";
               put "input.data.validation.agent.length.error.message.fmt.txt,2,%upcase(&agent_column),%upcase(&agent_capacity_table)";
               putlog ' ';
               call symput("&_rc",'8');
            run; 
         %end;
         %if &max_length_customer > 32 %then %do;
            data _null_;
               &setfile;
               putlog ' ';
               putlog "%mo_error Length of %upcase(&agent_column) in the %upcase(&customer_table) table must not be greater than 32.";
               put "input.data.validation.agent.length.error.message.fmt.txt,2,%upcase(&agent_column),%upcase(&customer_table)";
               putlog ' ';
               call symput("&_rc",'8');
            run; 
         %end;
      %end;

      %*;
      %* check to see if all time periods exist in the time period table. ;
      %*;
      %if (&agent_with_time_periods = 1) and (&has_time_period_column) %then %do;
         %let no_non_exist_time_periods = 0;
         %if (%bquote(&time_period_table) ne %str()) %then %do;
            proc sql noprint;
               select count(*) into :no_non_exist_time_periods
                  from work.agent_capacity_in_mo_tbl
                  where time_period ne "" and
                  time_period not in (select time_period from work.time_period_in_mo_tbl);
            quit;
         %end;
         %else %do;
            proc sql noprint;
               select count(*) into :no_non_exist_time_periods
                  from work.agent_capacity_in_mo_tbl
                  where time_period ne "";
            quit;
         %end;
         %if (&sqlrc gt 4) %then %do;
            %let &_rc = 107;
            %put %mo_error SQL error encountered obtaining no_non_exist_time_periods (sqlrc=&sqlrc).;
            %goto ERROR_EXIT;
         %end;

         %if &no_non_exist_time_periods > 0 %then %do;
            data _null_;
               &setfile;
               putlog ' ';
               putlog "%mo_error Some time periods in the agent capacity table do not exist in the time period table.";
               put "input.data.validation.agent.period.value.missing.error.message.txt,0";
               putlog ' ';
               call symput("&_rc",'8');
            run; 
         %end;
      %end;

      %*;
      %* Check for the missing values in the capacity column. ;
      %*;
      %if &has_capacity_column %then %do;

         %let no_missing_capacities = 0;
         proc sql noprint;
            select count(*) into :no_missing_capacities
               from work.agent_capacity_in_mo_tbl
               where capacity eq .;
         quit;
         %if (&sqlrc gt 4) %then %do;
            %let &_rc = 107;
            %put %mo_error SQL error encountered obtaining no_missing_capacities (sqlrc=&sqlrc).;
            %goto ERROR_EXIT;
         %end;
 
         %if &no_missing_capacities ge 1 %then %do;
            data _null_;
               &setfile;
               putlog ' ';
               putlog "%mo_error There are missing values in the CAPACITY column in the agent capacity table.";
               put "input.data.validation.agent.capacity.value.missing.error.message.txt,0";
               putlog ' ';
               call symput("&_rc",'8');
            run;
         %end;
      %end;

      %*;
      %* check for the duplicated rows in the agent table. ;
      %*;
      %if (&has_agent_column) and (&has_capacity_column) %then %do;

         %if &agent_with_time_periods = 1 %then %do;
            %let no_of_distinct_capacities = 0;
            proc sql noprint;
               create table temp as
                  select distinct &agent_column
                     %if &has_time_period_column %then %do;
                        , time_period
                     %end;
                  from work.agent_capacity_in_mo_tbl;

               select count(*) into :no_of_distinct_capacities
                  from temp;
            quit;
         %end;
         %else %do;
            proc sql noprint;
               select count(distinct &agent_column) into :no_of_distinct_capacities
                  from work.agent_capacity_in_mo_tbl;
            quit;
         %end;
         %if (&sqlrc gt 4) %then %do;
            %let &_rc = 107;
            %put %mo_error SQL error encountered obtaining no_of_distinct_capacities (sqlrc=&sqlrc).;
            %goto ERROR_EXIT;
         %end;

         %let no_of_capacities = 0;
         proc sql noprint;
            select count(*) into :no_of_capacities
            from work.agent_capacity_in_mo_tbl;
         quit;
         %if (&sqlrc gt 4) %then %do;
            %let &_rc = 107;
            %put %mo_error SQL error encountered obtaining no_of_capacities (sqlrc=&sqlrc).;
            %goto ERROR_EXIT;
         %end;

         %if &no_of_capacities ne &no_of_distinct_capacities %then %do;
            data _null_;
               &setfile;
               putlog ' ';
               putlog "%mo_error The combination of agent ID and time period in the agent capacity table must be unique.";
               put "input.data.validation.agent.capacity.duplicate.error.message.txt,0";
               putlog ' ';
               call symput("&_rc", '8');
            run;
         %end;
      %end;

      %*;
      %* Check that constraints exist in the table. ;
      %*;
      %let num_agent_rows = 0;
      proc sql noprint;
         select count(*) into :num_agent_rows
         from work.agent_capacity_in_mo_tbl;
      quit;
      %if (&sqlrc gt 4) %then %do;
         %let &_rc = 107;
         %put %mo_error SQL error encountered obtaining num_agent_rows (sqlrc=&sqlrc).;
         %goto ERROR_EXIT;
      %end;
      
      %if (&num_agent_rows > 0) %then %let agent_constraints_exist = 1;
      
   %end;


   %*-----------------------------------------------------------------;
   %*-----------------------------------------------------------------;
   %*-----------------------------------------------------------------;
   %* Performance-impacting customer checks follow ...                ;
   %*-----------------------------------------------------------------;
   %*-----------------------------------------------------------------;
   %*-----------------------------------------------------------------;

   %*---------------------------------------------------------------------;
   %* Check for blank Customer ID and Household ID in the Customer table. ;
   %*---------------------------------------------------------------------;
   data _null_;
      set moinput.&customer_table (keep=&custIdColNameListNoComma
                                        &household_id_column);
      &setfile;
      if (&custIdCol1 eq ' ') 
         %do custIdColNum=2 %to &numCustIdCols;   /* Blank values allowed  */
            and (&&custIdCol&custIdColNum eq ' ') /* in some, but not all, */
         %end;                                    /* key columns in multi- */
         then do;                                 /* part customer ID.     */
         putlog ' ';
         putlog "%mo_error Blank value detected for %upcase(&custIdColNameListComma) in observation " _N_ +(-1) " of %upcase(&customer_table) table.";
         put    "input.data.validation.blank.value.error.message.fmt.txt,3,%upcase(&custIdColNameListComma)," _N_ +(-1)",%upcase(&customer_table)";
         putlog ' ';
         call symput("&_rc",'8');
      end;
      %if (%bquote(&household_id_column) ne %str()) %then %do;
         if (&household_id_column eq ' ') then do;
            putlog ' ';
            putlog "%mo_error Blank value detected for %upcase(&household_id_column) in observation " _N_ +(-1) " of %upcase(&customer_table) table.";
            put    "input.data.validation.blank.value.error.message.fmt.txt,3,%upcase(&household_id_column)," _N_ +(-1)",%upcase(&customer_table)";
            putlog ' ';
            call symput("&_rc",'8');
         end;
      %end;
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered reading %upcase(&customer_table) (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;


   %*----------------------------------------------------------------;
   %* If contact history is being used, check for blank Customer ID  ;
   %* in the Contact History table.                                  ;
   %*----------------------------------------------------------------;
   %if (&uses_contact_history eq 1) %then %do;
      data _null_;
         set moinput.&contact_history_table (keep=&custIdColNameListNoComma);
         &setfile;
         if (&custIdCol1 eq ' ') 
            %do custIdColNum=2 %to &numCustIdCols;   /* Blank values allowed  */
               and (&&custIdCol&custIdColNum eq ' ') /* in some, but not all, */
            %end;                                    /* key columns in multi- */
            then do;                                 /* part customer ID.     */
            putlog ' ';
            putlog "%mo_error Blank value detected for %upcase(&custIdColNameListComma) in observation " _N_ +(-1) " of %upcase(&contact_history_table) table.";
            put    "input.data.validation.blank.value.error.message.fmt.txt,3,%upcase(&custIdColNameListComma)," _N_ +(-1)",%upcase(&contact_history_table)";
            putlog ' ';
            call symput("&_rc",'8');
         end;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered checking %upcase(&contact_history_table) for blank Customer ID column(s) (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;
   %end;


   %*--------------------------------------------------------------------;
   %* If the "validate customer IDs" flag is set, check that Customer ID ;
   %* values are unique in the Customer table.                           ;
   %* If contact history is being used, check that Customer ID values in ;
   %* the Contact History table exist in the Customer table.             ;
   %*--------------------------------------------------------------------;
   %if (&mo_validate_input_data_cust_ids eq 1) %then %do;
      %mo_no_notes(on)
      %put %mo_note Starting Customer Table Customer ID column value uniqueness check ... ;
      %mo_no_notes(off)

      options ibufsize=32767;

      %local custIdColValsConcatLength;
      %let custIdColValsConcatLength = 0;
      data _null_;
         set work.customer_contents_custIDCols end=last;
         retain totalLength 0;
         if (type = 2) then do;
            totalLength = totalLength + length;
         end;
         else do; /* Allow conversion of huge number (e.g., 2**64) to string */
            totalLength = totalLength + 20;
         end;
         if (last = 1) then do;
            call symput("custIdColValsConcatLength", put(totalLength, 8.));
         end;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error determining length of customer ID columns from work.customer_contents_custIDCols (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;
      %let custIdColValsConcatLength = %eval(&custIdColValsConcatLength + (3 * (&numCustIdCols - 1)));
      %if (&custIdColValsConcatLength gt 4096) %then %do;
         %put %mo_warning Total length of Customer ID columns is too large to perform check: &custIdColValsConcatLength;
         %goto END_CUSTID_CHECK; 
      %end; 

      %* Multi-part customer IDs not supported for MOSA, but handle it for  ;
      %* both MOMA and MOSA anyway.  It does not add much overhead to MOSA. ;
      %local custIdColValsConcat;
      %let custIdColValsConcat = ;

      %local dsid custIdCol_varnum custIdCol_vartype;  
      %let dsid = %sysfunc(open(moinput.&customer_table));
      %if (&dsid = 0) %then %do;
         data _null_;
            &setfile;
            putlog ' ';
            %putlog "%mo_error The specified &customer_table table does not exist in MOINPUT.";
            put "input.data.validation.table.not.exist.error.message.fmt.txt,1,&customer_table";            
            call symput("&_rc",'8');
         run;
         %goto ERROR_EXIT;
      %end;

      %do custIdColNum=1 %to &numCustIdCols;
         %let custIdCol_varnum = 0;
         %let custIdCol_vartype = ;

         %let custIdCol_varnum = %sysfunc(varnum(&dsid, &&custIdCol&custIdColNum));
         %if (&custIdCol_varnum ne 0) %then %do;
            %let custIdCol_vartype = %sysfunc(vartype(&dsid, &custIdCol_varnum));
         %end;
         %if (%upcase(&custIdCol_vartype) eq N) %then %do;
            %let custIdColValsConcat = 
               &custIdColValsConcat||%str("^!#")||trim(left(put(&&custIdCol&custIdColNum, 32.)));
         %end;
         %else %if (%upcase(&custIdCol_vartype) eq C) %then %do;
            %let custIdColValsConcat = 
               &custIdColValsConcat||%str("^!#")||trim(left(&&custIdCol&custIdColNum));
         %end;
      %end;
      %let dsid = %sysfunc(close(&dsid));
      %let custIdColValsConcat = %substr(%nrbquote(&custIdColValsConcat), %index(%nrbquote(&custIdColValsConcat), trim));

      proc sql noprint;
         create table work.custTbl_custIDs_concat as
         select &custIdColNameListComma, 
                &custIdColValsConcat as custIDs_concat length=&custIdColValsConcatLength
         from moinput.&customer_table
         order by &custIdColNameListComma;
      quit;
      %if (&sqlrc gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered creating %upcase(work.custTbl_custIDs_concat) from %upcase(&customer_table) (sqlrc=&sqlrc).;
         %goto ERROR_EXIT;
      %end;

      %* First do quick check based on row counts. ;
      %local cust_tbl_row_count;
      %local unique_customer_id_count;
      %let cust_tbl_row_count = 0;
      %let unique_customer_id_count = 0;
      proc sql noprint;
         select count(*), count(distinct custIDs_concat)
         into :cust_tbl_row_count, :unique_customer_id_count 
         from work.custTbl_custIDs_concat;
      quit;
      %if (&sqlrc gt 4) %then %do;
         %let &_rc = 107;
         %put %mo_error SQL error encountered counting rows in moinput.&customer_table (sqlrc=&sqlrc).;
         %goto ERROR_EXIT;
      %end;

      %* Handle duplicates.                                                   ;
      %*                                                                      ;
      %* If MOMA, just report that duplicates exist without identifying them. ;
      %* Technically, this is a system error since those records are          ;
      %* programmatically generated by MA.                                    ;
      %*                                                                      ;
      %* If MOSA, figure out which ones are duplicates.                       ;
      %if (&unique_customer_id_count ne &cust_tbl_row_count) %then %do;

         %if (&moma_input eq 1) %then %do;
            data _null_;
               &setfile;
               putlog ' ';
               putlog "%mo_error The values for the %upcase(&custIdColNameListNoComma) customer ID column(s) in the %upcase(&customer_table) table must be unique.";
               put    "input.data.validation.column.values.notunique.error.message.txt,2,%upcase(&custIdColNameListNoComma),%upcase(&customer_table)";
               putlog ' ';
               call symput("&_rc",put(max(&&&_rc,4),1.));
            run;
         %end;

         %else %do;
            proc sql noprint;
               create table work.custTbl_custIDs_count as
               select &custIdColNameListComma, 
                      custIDs_concat, count(custIDs_concat) as count
               from work.custTbl_custIDs_concat
               group by custIDs_concat;
      
               create table work.custTbl_custIDs_duplicates as
               select &custIdColNameListComma, custIDs_concat, count
               from work.custTbl_custIDs_count
               where (count > 1);
            quit;
            %if (&sqlrc gt 4) %then %do;
               %let &_rc = 100;
               %put %mo_error Error encountered counting duplicate IDs in %upcase(work.custTbl_custIDs_concat) (sqlrc=&sqlrc).;
               %goto ERROR_EXIT;
            %end;
      
            %if %mo_obs_in(work.custTbl_custIDs_duplicates) %then %do;
               data _null_;
                  set work.custTbl_custIDs_duplicates;
                  &setfile;
                  putlog ' ';
                  putlog "%mo_error Detected a (combination of) identifier(s) in the %upcase(&custIdColNameListComma) column(s) of the %upcase(&customer_table) table that is not unique:" custIDs_concat " .";
                  put    "input.data.validation.duplicate.customer_id.error.message.fmt.txt,3,%upcase(&custIdColNameListNoComma),%upcase(&customer_table)," custIDs_concat;
                  putlog ' ';
                  call symput("&_rc",'8');
               run;
            %end;
         %end;
      %end;
      %mo_no_notes(on)
      %put %mo_note Ending Customer Table Customer ID column value uniqueness check ... ;
      %mo_no_notes(off)
   
      %if (&uses_contact_history eq 1) %then %do;
         %mo_no_notes(on)
         %put %mo_note Starting Contact History Table Customer ID column value comparison check ... ;
         %mo_no_notes(off)

         %local contactHistCustIdColValsConcat;
         %let contactHistCustIdColValsConcat = ;

         %local dsid custIdCol_varnum custIdCol_vartype;  
         %let dsid = %sysfunc(open(moinput.&contact_history_table));
         %if (&dsid = 0) %then %do;
            data _null_;
               &setfile;
               putlog ' ';
               %putlog "%mo_error The specified &contact_history_table table does not exist in MOINPUT.";
               put "input.data.validation.table.not.exist.error.message.fmt.txt,1,&contact_history_table";            
               call symput("&_rc",'8');
            run;
            %goto ERROR_EXIT;
         %end;

         %do custIdColNum=1 %to &numCustIdCols;
            %let custIdCol_varnum = 0;
            %let custIdCol_vartype = ;

            %let custIdCol_varnum = %sysfunc(varnum(&dsid, &&custIdCol&custIdColNum));
            %if (&custIdCol_varnum ne 0) %then %do;
               %let custIdCol_vartype = %sysfunc(vartype(&dsid, &custIdCol_varnum));
            %end;
            %if (%upcase(&custIdCol_vartype) eq N) %then %do;
               %let contactHistCustIdColValsConcat = 
                  &contactHistCustIdColValsConcat||%str("^!#")||trim(left(put(&&custIdCol&custIdColNum, 32.)));
            %end;
            %else %if (%upcase(&custIdCol_vartype) eq C) %then %do;
               %let contactHistCustIdColValsConcat = 
                  &contactHistCustIdColValsConcat||%str("^!#")||trim(left(&&custIdCol&custIdColNum));
            %end;
         %end;
         %let dsid = %sysfunc(close(&dsid));
         %let contactHistCustIdColValsConcat = %substr(%nrbquote(&contactHistCustIdColValsConcat), %index(%nrbquote(&contactHistCustIdColValsConcat), trim));

         proc sql noprint;
            create table work.cntctTbl_custIDs_concat as
            select &custIdColNameListComma, 
                   &contactHistCustIdColValsConcat as custIDs_concat length=&custIdColValsConcatLength
            from moinput.&contact_history_table
            order by &custIdColNameListComma;
         quit;
         %if (&sqlrc gt 4) %then %do;
            %let &_rc = 100;
            %put %mo_error Error encountered creating %upcase(work.contactTable_custIDs_concat) from %upcase(&contact_history_table) (sqlrc=&sqlrc).;
            %goto ERROR_EXIT;
         %end;

         proc sql noprint;
            create table work.cntctTbl_custIDs_distinct_concat as
            select distinct custIDs_concat 
            from work.cntctTbl_custIDs_concat
            order by custIDs_concat;
         quit;
         %if (&sqlrc gt 4) %then %do;
            %let &_rc = 100;
            %put %mo_error Error encountered creating %upcase(work.cntctTbl_custIDs_distinct_concat) from %upcase(work.cntctTbl_custIDs_concat) (sqlrc=&sqlrc).;
            %goto ERROR_EXIT;
         %end;
   
         proc sql noprint;
            create table work.custTbl_custIDs_distinct_concat as
            select distinct custIDs_concat
            from work.custTbl_custIDs_concat 
            order by custIDs_concat;
         quit;
         %if (&sqlrc gt 4) %then %do;
            %let &_rc = 100;
            %put %mo_error Error encountered creating %upcase(work.custTbl_custIDs_distinct_concat) from %upcase(work.custTbl_custIDs_concat) (sqlrc=&sqlrc).;
            %goto ERROR_EXIT;
         %end;
   
         %let num_distinct_custIDs_in_history = 0;
         %let num_custIDs_undefined = 0;
         proc sql noprint;
            create table work.cntctTable_custIDs_undefined as
               select a.custIDs_concat, b.custIDs_concat as custTbl_custID_match 
               from work.cntctTbl_custIDs_distinct_concat as a left join
                    work.custTbl_custIDs_distinct_concat as b
               on (a.custIDs_concat = b.custIDs_concat);

               select count(*) into :num_custIDs_undefined
               from work.cntctTable_custIDs_undefined
               where (custTbl_custID_match = "");

               select count(*) into :num_distinct_custIDs_in_history
               from work.cntctTbl_custIDs_distinct_concat;
         quit;
         %if (&sqlrc gt 4) %then %do;
            %let &_rc = 107;
            %put %mo_error SQL error encountered comparing work.cntctTbl_custIDs_distinct_concat with work.custTbl_custIDs_distinct_concat (sqlrc=&sqlrc).;
            %goto ERROR_EXIT;
         %end;

         %if (&num_custIDs_undefined ne 0) %then %do;
            data _null_;
               &setfile;
               putlog ' ';
                  %if (&num_custIDs_undefined eq &num_distinct_custIDs_in_history) %then %do;
                     putlog ' ';
                     putlog "%mo_warning None of the Customer ID values present in the %upcase(&contact_history_table) table exist in the %upcase(&customer_table) table.";
                     put    "input.data.validation.contactHist.custIDs.none.defined.warning.message.fmt.txt,2,%upcase(&contact_history_table),%upcase(&customer_table)";
                     putlog ' ';
                  %end;
                  %else %do;
                     putlog ' ';
                     putlog "%mo_warning There are &num_custIDs_undefined Customer ID values present in the %upcase(&contact_history_table) table that do not exist in the %upcase(&customer_table) table.";
                     put    "input.data.validation.contactHist.custIDs.not.defined.warning.message.fmt.txt,3,&num_custIDs_undefined,%upcase(&contact_history_table),%upcase(&customer_table)";
                     putlog ' ';
                  %end;
               call symput("&_rc",put(max(&&&_rc,4),1.));
               putlog ' ';
            run;
         %end;
         %mo_no_notes(on)
         %put %mo_note Ending Contact History Table Customer ID column value comparison check ... ;
         %mo_no_notes(off)
      %end;

      %END_CUSTID_CHECK:
   %end;


   %*---------------------------------------------------;
   %* Check for character customer attributes that have ;
   %* too many distinct values                          ;
   %* Ensure customer ID column(s), household ID column ;
   %* and agent column are excluded.                    ;
   %*---------------------------------------------------;
   data work.extra_customer_attributes;
      set work.extra_columns;
      where ((type=2) 
             and (upcase(name) ne "%upcase(&agent_column)") 
             and (upcase(name) ne "%upcase(&household_id_column)")
             %do i = 1 %to &numCustIdCols; 
                and (upcase(name) ne "%upcase(%scan(&custIdColNameListNoComma, &i))")
             %end;
             );
   run;
   %if (&syserr gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered creating WORK.EXTRA_CUSTOMER_ATTRIBUTES (syserr=&syserr).;
      %goto ERROR_EXIT;
   %end;

   %local num_extra_customer_attributes;
   %let num_extra_customer_attributes = 0;
   proc sql noprint;
      create table work.extra_customer_attribute_values
         (customer_attribute        char(32)
         ,customer_attribute_value  char(256)
         );

      select count(*) into :num_extra_customer_attributes
         from work.extra_customer_attributes;
   quit;
   %if (&sqlrc gt 4) %then %do;
      %let &_rc = 100;
      %put %mo_error Error encountered creating WORK.EXTRA_CUSTOMER_ATTRIBUTE_VALUES (sqlrc=&sqlrc).;
      %goto ERROR_EXIT;
   %end;

   %do i = 1 %to &num_extra_customer_attributes;
      data _null_;
         obsnum = &i;
         set work.extra_customer_attributes point=obsnum;
         call symput("current_customer_attribute", name);
         stop;
      run;
      %if (&syserr gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error obtaining customer attribute name from work.mo_control_measure_names (syserr=&syserr).;
         %goto ERROR_EXIT;
      %end;
      %let current_customer_attribute = %trim(%left(&current_customer_attribute));

      %let count_distinct_attr_values = 0;
      proc sql noprint;
         create table work.distinct_cust_attr_values as
            select distinct compbl(trim(left(&current_customer_attribute))) as customer_attribute_value,
                   "&current_customer_attribute" as customer_attribute
            from moinput.&customer_table
            where (&current_customer_attribute is not missing);
      quit;
      %let count_distinct_attr_values = &sqlobs;
      %if (&sqlrc gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered creating WORK.DISTINCT_CUST_ATTR_VALUES (sqlrc=&sqlrc).;
         %goto ERROR_EXIT;
      %end;

      proc sql noprint;
         insert into work.extra_customer_attribute_values
            select customer_attribute, customer_attribute_value from work.distinct_cust_attr_values;
      quit;
      %if (&sqlrc gt 4) %then %do;
         %let &_rc = 100;
         %put %mo_error Error encountered inserting into WORK.EXTRA_CUSTOMER_ATTRIBUTE_VALUES (sqlrc=&sqlrc).;
         %goto ERROR_EXIT;
      %end;

      %if &count_distinct_attr_values > &mo_max_distinct_attribute_values %then %do;
         data _null_;
            &setfile;
            putlog ' ';
            putlog "%mo_error The number of distinct values for customer attribute %upcase(&current_customer_attribute) is &count_distinct_attr_values.. This exceeds the maximum number of distinct values of &mo_max_distinct_attribute_values..";
            put    "input.data.validation.new.many.attr.values.error.message.fmt.txt,3,%upcase(&current_customer_attribute),&count_distinct_attr_values,&mo_max_distinct_attribute_values";
            putlog ' ';
            call symput("&_rc",'8');
         run;      
      %end;   
   %end;


   %*--------------------------------;
   %* Exit processing                ;
   %*--------------------------------;
   %put;
   %mo_no_notes(on)
   %if (&&&_rc eq 0) %then
      %put %mo_note Input data validation completed with no errors and no warnings.;
   %else %if (&&&_rc eq 4) %then
      %put %mo_warning Input data validation completed with no errors but with warnings (rc=&&&_rc).;
   %else
      %put %mo_error Input data validation detected errors (rc=&&&_rc).;
   %put;
   %goto EXIT;

   %ERROR_EXIT:
   %put;
   %put %mo_error Error encountered while validating input data (rc=&&&_rc).;


   %EXIT:
   %mo_no_notes(off)

%mend mo_validate_input_data;
